// Name:        RESCUE Explorer
// Purpose:     Explore RESCUE exports
// Copyright:   (C) 2006, Carl Godkin (carl@dgi.com)
// License:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

#include <wx/aboutdlg.h>
#include <wx/dnd.h>
#include <wx/laywin.h>

#include "RescueExplorerFrame.h"

#include "PropertyListCtrl.h"
#include "RescueExplorerTree.h"

#define TREE_WIDTH 300
#define LIST_WIDTH 300
#define BOTH_HEIGHT 300

#define MIN_DIMEN 50

BEGIN_EVENT_TABLE(RescueExplorerFrame, wxFrame)
  EVT_MENU (rexID_OPEN, RescueExplorerFrame::OnOpen)
  EVT_MENU (rexID_RELOAD, RescueExplorerFrame::OnReload)
  EVT_MENU (rexID_CLOSE, RescueExplorerFrame::OnClose)

  EVT_MENU (rexID_ADD_VERTEX, RescueExplorerFrame::OnAddVertex)
  EVT_UPDATE_UI (rexID_ADD_VERTEX, RescueExplorerFrame::OnUpdateAddVertex)
  EVT_MENU (rexID_SHOW_CORNERS_ONLY, RescueExplorerFrame::OnShowCornersOnly)
  EVT_UPDATE_UI (rexID_SHOW_CORNERS_ONLY,
		 RescueExplorerFrame::OnUpdateShowCornersOnly)

  EVT_MENU (rexID_ABOUT, RescueExplorerFrame::OnAbout)

  EVT_SASH_DRAGGED (rexID_SASH_LIST, RescueExplorerFrame::OnSashDrag)
  EVT_SIZE (RescueExplorerFrame::OnResize)
END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(RescueExplorerFrame, wxFrame)

// --------------------------------------------------------------------

class MyFileDropTarget : public wxFileDropTarget {
public:
  MyFileDropTarget (RescueExplorerFrame *frame) : m_frame (frame) { ; }

private:
  bool OnDropFiles (wxCoord x, wxCoord y, const wxArrayString &filenames) {
    return m_frame->dropFiles (x, y, filenames);
  }

  RescueExplorerFrame *m_frame;
};
//-----------------------------------------------------------------------------

RescueExplorerFrame::RescueExplorerFrame (wxWindow *parent,
					  const wxString &inputFile,
					  bool delayGeometryLoad,
					  bool delayPropertyLoad,
					  const wxString &version) :
  wxFrame (parent, -1,
	   "RESCUE Explorer - " + inputFile,
	   wxDefaultPosition, wxSize (600, 600)),
  m_delayGeometryLoad (delayGeometryLoad),
  m_delayPropertyLoad (delayPropertyLoad),
  m_version (version)
{
  wxMenuBar *menuBar = new wxMenuBar;
    
  wxMenu *menu = new wxMenu;
  menu->Append (rexID_OPEN, "&Open RESCUE file\tCtrl+O");
  menu->Append (rexID_RELOAD, "&Reload current file\tCtrl+R");
  menu->AppendSeparator ();
  menu->Append (rexID_CLOSE, "&Close\tCtrl+Q");

  menuBar->Append (menu, "&File");

  menu = new wxMenu;
  menu->AppendCheckItem (rexID_ADD_VERTEX,
			 "&Add vertex coordinates to X,Y,Z values");
  menu->AppendCheckItem (rexID_SHOW_CORNERS_ONLY,
			 "&Show only first && last 5 items in corners"
			 " of grid geometry");
  menuBar->Append (menu, "&View");

  menu = new wxMenu;
  menu->Append (rexID_ABOUT, "&About");
  
  menuBar->Append (menu, "&Help");

  SetMenuBar (menuBar);

  // Put tree on the left.
  m_treeSLW = new wxSashLayoutWindow (this, rexID_SASH_TREE);
  m_treeSLW->SetOrientation (wxLAYOUT_VERTICAL);
  m_treeSLW->SetAlignment (wxLAYOUT_LEFT);
  m_treeSLW->SetDefaultSize (wxSize (TREE_WIDTH, BOTH_HEIGHT));
  m_treeSLW->SetMinimumSizeX (MIN_DIMEN);
  m_treeSLW->SetMinimumSizeY (MIN_DIMEN);

  // Put property list on right.
  m_listSLW = new wxSashLayoutWindow (this, rexID_SASH_LIST);
  m_listSLW->SetOrientation (wxLAYOUT_VERTICAL);
  m_listSLW->SetAlignment (wxLAYOUT_RIGHT);
  m_listSLW->SetDefaultSize (wxSize (LIST_WIDTH, BOTH_HEIGHT));
  m_listSLW->SetMinimumSizeX (MIN_DIMEN);
  m_listSLW->SetMinimumSizeY (MIN_DIMEN);
  m_listSLW->SetSashVisible (wxSASH_LEFT, true);

  // Put the list in a panel
  wxPanel *listPanel = new wxPanel (m_listSLW, -1);
  wxBoxSizer *sizer = new wxBoxSizer (wxVERTICAL);
  PropertyListCtrl *list = new PropertyListCtrl (listPanel, -1);
  sizer->Add (list, 1, wxEXPAND);
  listPanel->SetSizerAndFit (sizer);

  // Put the tree in a panel
  wxPanel *treePanel = new wxPanel (m_treeSLW, -1);
  m_tree = new RescueExplorerTree (treePanel, list, inputFile,
				   m_delayGeometryLoad, m_delayPropertyLoad);
  sizer = new wxBoxSizer (wxVERTICAL);
  sizer->Add (m_tree, 1, wxEXPAND);
  treePanel->SetSizerAndFit (sizer);

  wxLayoutAlgorithm layout;
  layout.LayoutFrame (this);

  SetDropTarget (new MyFileDropTarget (this));

  CreateStatusBar (1);
}
/* ------------------------------------------------------------------ */
// Ask user for RESCUE file name.

void RescueExplorerFrame::OnOpen (wxCommandEvent &event)
{
  wxFileDialog d (this, "Select RESCUE file", "", "",
		  "RESCUE files (*.bin)|*.bin",
		  wxFD_OPEN | wxFD_FILE_MUST_EXIST);
  if (d.ShowModal() == wxID_OK) {
    // Create a new application window
    RescueExplorerFrame *frame =
      new RescueExplorerFrame (0, d.GetPath(),
			       m_delayGeometryLoad, m_delayPropertyLoad,
			       m_version);
    frame->Show (true);
  }
}
/* ------------------------------------------------------------------ */
// Reload current RESCUE file.

void RescueExplorerFrame::OnReload (wxCommandEvent &event)
{
  m_tree->reload ();
}
/* ------------------------------------------------------------------ */
// Quit

void RescueExplorerFrame::OnClose (wxCommandEvent &event)
{
  Close ();
}
//----------------------------------------------------------------------------

void RescueExplorerFrame::OnAddVertex (wxCommandEvent &event)
{
  m_tree->setAddVertex (event.IsChecked());
}
//----------------------------------------------------------------------------

void RescueExplorerFrame::OnUpdateAddVertex (wxUpdateUIEvent &event)
{
  event.Check (m_tree->getAddVertex());
}
//----------------------------------------------------------------------------

void RescueExplorerFrame::OnShowCornersOnly (wxCommandEvent &event)
{
  m_tree->showCornersOnly (event.IsChecked());
}
//----------------------------------------------------------------------------

void RescueExplorerFrame::OnUpdateShowCornersOnly (wxUpdateUIEvent &event)
{
  event.Check (m_tree->getShowCornersOnly());
}
//----------------------------------------------------------------------------

void RescueExplorerFrame::OnAbout (wxCommandEvent &event)
{
  wxAboutDialogInfo info;
  info.SetName ("RescueExplorer");
  info.SetVersion (m_version);
  info.SetDescription ("Browse a RESCUE model");
  info.SetCopyright ("(C) 2007 Carl Godkin (carl@dgi.com)");
  
  wxAboutBox (info);
}
//----------------------------------------------------------------------------

void RescueExplorerFrame::OnSashDrag (wxSashEvent &event)
{
  if (event.GetDragStatus() == wxSASH_STATUS_OUT_OF_RANGE)
    return;
  
  wxRect rect = event.GetDragRect ();
  wxSize size (rect.width, rect.height);
  if (size.GetWidth () < MIN_DIMEN || size.GetHeight () < MIN_DIMEN)
    return;

  m_listSLW->SetDefaultSize (size);
  wxLayoutAlgorithm layout;
  layout.LayoutFrame (this, m_treeSLW);
}
//----------------------------------------------------------------------------

void RescueExplorerFrame::OnResize (wxSizeEvent &event)
{
  wxLayoutAlgorithm layout;
  layout.LayoutFrame(this);

  event.Skip();
}
//----------------------------------------------------------------------------
// User dropped file(s).

bool RescueExplorerFrame::dropFiles (wxCoord x, wxCoord y,
				     const wxArrayString& names)
{
  // Turn on busy cursor in case this takes a while.
  wxBusyCursor wait;

  // Create new frame for each dragged item.
  for (size_t i=0; i<names.GetCount(); i++) {
    RescueExplorerFrame *frame =
      new RescueExplorerFrame (0, names[i],
			       m_delayGeometryLoad, m_delayPropertyLoad,
			       m_version);
    frame->Show (true);
  }
  
  return true;
}
