////////////////////////////////////////////////////////////////////////////
// Name:        RESCUE Explorer
// Purpose:     Explore RESCUE exports
// Copyright:   (C) 2006, Carl Godkin (carl@dgi.com)
// License:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

#include <wx/progdlg.h>

#include "RescueExplorerTree.h"

#include "PropertyListCtrl.h"
#include "RescueExplorerPairs.h"

#include "RescueArrayDouble.h"
#include "RescueArrayFloat.h"
#include "RescueArrayByte.h"
#include "RescueArrayShort.h"
#include "RescueArrayInt.h"
#include "RescueArray2dVector.h"
#include "RescueArray3dVector.h"
#include "RescueDocument.h"
#include "RescueFaultIntersection.h"
#include "RescueGeobody.h"
#include "RescueHistory.h"
#include "RescueLookup.h"
#include "RescueLookupString.h"
#include "RescueLookupTable.h"
#include "RescueMacroVolume.h"
#include "RescueModel.h"
#include "RescueModelPropertyGroup.h"
#include "RescueReservoir.h"
#include "RescueStairSteppedFault.h"
#include "RescueVertex.h"
#include "RescueWellboreProperty.h"

BEGIN_EVENT_TABLE(RescueExplorerTree, wxTreeCtrl)
  EVT_TREE_SEL_CHANGED (-1, RescueExplorerTree::OnSelection)
END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(RescueExplorerTree, wxTreeCtrl)

//----------------------------------------------------------------------

class ProgressMonitor: public RescueProgressReporter {
public:
  ProgressMonitor (wxProgressDialog *pd) : m_pd (pd), m_count (0) { ; }

  void ReportProgress (const char *phaseDescription) {
    if (!m_pd)
      return;
    else if (m_count < 99)
      m_count++;
    else
      m_count = 0;

    m_pd->Update (m_count, phaseDescription);
  }

  void done () {
    m_pd->Update (100);
    delete m_pd;
    m_pd = 0;
  }

private:
  wxProgressDialog *m_pd;
  int m_count;
};

//----------------------------------------------------------------------------

RescueExplorerTree::RescueExplorerTree (wxWindow *parent,
					PropertyListCtrl *list,
					const wxString &inputFile,
					bool delayGeometryLoad,
					bool delayPropertyLoad,
					wxWindowID id,
					const wxPoint &pos,
					const wxSize &size,
					long style,
					const wxValidator& validator,
					const wxString& name):
  wxTreeCtrl (parent, id, pos, size, style, validator, name),
  m_context (0), m_model (0),
  m_list (list),
  m_addVertex (false),
  m_showCornersOnly (true),
  m_delayGeometryLoad (delayGeometryLoad),
  m_delayPropertyLoad (delayPropertyLoad)
{
  if (!inputFile.empty ())
    open (inputFile);
}
//----------------------------------------------------------------------------

void RescueExplorerTree::open (const wxString &inputFile)
{
  m_inputFile = inputFile;

  m_context = new RescueContext;

  if (!m_delayGeometryLoad)
    m_context->SetLoadOnDemand2 (false); 

  if (!m_delayPropertyLoad)
    m_context->SetLoadOnDemand (false);

  // Print out status to a monitor. 
  ProgressMonitor progressMonitor (new wxProgressDialog
				   ("Loading " + inputFile,
				    "Starting to read " + inputFile));
  m_context->RegisterProgressReporter (&progressMonitor);

  // Open the model.
  m_model = RescueModel::UnarchiveModel (m_context,
					 (char *) inputFile.c_str());

  // Close the progress dialog and delete it.
  progressMonitor.done ();

  if (!m_model) {
    ::wxLogError ("Unable to open RESCUE model %s", inputFile.c_str());
    m_inputFile.clear();
    
    delete m_context;
    m_context = 0;
  }

  displayModel ();
  
  if (m_model)
    showProperties (m_model);
  else
    noProperties ();
}
//----------------------------------------------------------------------------

void RescueExplorerTree::reload ()
{
  delete m_model;
  m_model = 0;

  delete m_context;
  m_context = 0;

  open (m_inputFile);
}
//----------------------------------------------------------------------------

void RescueExplorerTree::setAddVertex (bool addVertex)
{
  m_addVertex = addVertex;

  selectItem (GetSelection ());
}
//----------------------------------------------------------------------------

void RescueExplorerTree::showCornersOnly (bool showCornersOnly)
{
  m_showCornersOnly = showCornersOnly;

  selectItem (GetSelection ());
}
//----------------------------------------------------------------------------

void RescueExplorerTree::displayModel ()
{
  // Delete all the nodes in the tree.
  DeleteAllItems ();

  wxTreeItemId rootId = AddRoot ("Root");

  if (m_context)
    addLeaf (rootId, m_context);

  if (m_model)
    addLeaf (rootId, m_model);
}
//----------------------------------------------------------------------------

wxTreeItemId RescueExplorerTree::addItem (const wxTreeItemId &parent,
					  const wxString &type,
					  RCHString *label,
					  RescueObject *object)
{
  return addItem (parent, type, label->String(), object);
}
//----------------------------------------------------------------------------

wxTreeItemId RescueExplorerTree::addItem (const wxTreeItemId &parent,
					  const wxString &type,
					  const wxString &label,
					  RescueObject *object)
{
  wxString name (type);
  if (!label.empty ())
    name += wxString::Format (" (%s)", label.c_str());

  return AppendItem (parent, name, -1, -1,
		     new RescueExplorerTreeItemData (type, label, object));
}
//----------------------------------------------------------------------------

void RescueExplorerTree::expandAll (const wxTreeItemId& item)
{
  if (item != GetRootItem () && !IsExpanded (item))
    Expand(item);

  void *cookie;
  wxTreeItemId child = GetFirstChild(item, cookie);
  while (child.IsOk()) {
    expandAll(child);
    child = GetNextChild(item, cookie);
  }
}
//----------------------------------------------------------------------------

void RescueExplorerTree::OnSelection (wxTreeEvent& event)
{
  wxTreeItemId id = event.GetItem();
  if (!id.IsOk ())
    return;

  selectItem (id);
}
//----------------------------------------------------------------------------

void RescueExplorerTree::selectItem (const wxTreeItemId &id)
{    
  ::wxLogStatus ("");

  if (!id.IsOk ())
    return;

  RescueExplorerTreeItemData *data =
    (RescueExplorerTreeItemData *) GetItemData (id);
  if (!data)
    // No tree data.  Give up.
    return;

  wxString type = data->getType();

  if (data->getObject ()) {
    RescueObject *object = data->getObject();

    // Show busy cursor while loading properties in case there are lots.
    wxBusyCursor wait;

    if (object->IsA() <= R_TOMBSTONE) {
      error ("RescueObject is invalid (isA <= R_TOMBSTONE)");
      return;
    }
    else if (object->IsA() >= R_LAST_OBJECT_TYPE) {
      error ("RescueObject is invalid (isA >= R_LAST_OBJECT_TYPE)");
      return;
    }
    
    switch (object->IsA()) {
    case R_RescueObject:
      noProperties (type);
      break;
    case R_RescueModel:
      showProperties (dynamic_cast<RescueModel *>(object));
      break;
    case R_RescueBlock:
      showProperties (dynamic_cast<RescueBlock *>(object));
      break;
    case R_RescueUnit:
      showProperties (dynamic_cast<RescueUnit *>(object));
      break;
    case R_RescueHorizon:
      showProperties (dynamic_cast<RescueHorizon *>(object));
      break;
    case R_RescueSection:
      showProperties (dynamic_cast<RescueSection *>(object));
      break;
    case R_RescueBlockUnit:
      showProperties (dynamic_cast<RescueBlockUnit *>(object));
      break;
    case R_RescueBlockUnitSide:
      showProperties (dynamic_cast<RescueBlockUnitSide *>(object));
      break;
    case R_RescueProperty:
      showProperties (dynamic_cast<RescueProperty *>(object));
      break;
    case R_RescuePropertyGroup:
      showProperties (dynamic_cast<RescuePropertyGroup *>(object));
      break;
    case R_RescueBlockUnitHorizonSurface:
      showProperties (dynamic_cast<RescueBlockUnitHorizonSurface *>(object));
      break;
    case R_RescueWellbore:
      showProperties (dynamic_cast<RescueWellbore *>(object));
      break;
    case R_RescueWellboreProperty:
      showProperties (dynamic_cast<RescueWellboreProperty *>(object));
      break;
    case R_RescueGrid:
      showProperties (dynamic_cast<RescueGrid *>(object));
      break;
    case R_RescueGridAxis:
      showProperties (dynamic_cast<RescueGridAxis *>(object));
      break;
    case R_RescueArray:
      noProperties (type);
      break;
    case R_RescueTripletArray:
      showProperties (dynamic_cast<RescueTripletArray *>(object));
      break;
    case R_RescueCoordinateSystemAxis:
      showProperties (dynamic_cast<RescueCoordinateSystemAxis *>(object));
      break;
    case R_RescueVertex:
      showProperties (dynamic_cast<RescueVertex *>(object));
      break;
    case R_RescueCoordinateSystem:
      showProperties (dynamic_cast<RescueCoordinateSystem *>(object));
      break;
    case R_RescueSurface:
      showProperties (dynamic_cast<RescueSurface *>(object));
      break;
    case R_RescueDoubletArray:
      noProperties (type);
      break;
    case R_RescueTrimVertex:
      showProperties (dynamic_cast<RescueTrimVertex *>(object));
      break;
    case R_RescueEdgeSet:
      showProperties (dynamic_cast<RescueEdgeSet *>(object));
      break;
    case R_RescueTrimEdge:
      noProperties (type);
      break;
    case R_RescuePolyLine:
      showProperties (dynamic_cast<RescuePolyLine *>(object));
      break;
    case R_RescuePolyLineNode:
      noProperties (type);
      break;
    case R_RescuePolyLineNodeUV:
      noProperties (type);
      break;
    case R_RescueGeometry:
      showProperties (dynamic_cast<RescueGeometry *>(object));
      break;
    case R_RescueGeometryUnit:
      showProperties (dynamic_cast<RescueGeometryUnit *>(object));
      break;
    case R_RescueSurfaceCell:
      noProperties (type);
      break;
    case R_RescueWellboreSampling:
      noProperties (type);
      break;
    case R_RescueWellboreSurface:
      noProperties (type);
      break;
    case R_RescueWellboreCell:
      noProperties (type);
      break;
    case R_RescueReferenceSurface:
      showProperties (dynamic_cast<RescueReferenceSurface *>(object));
      break;
    case R_RescueLookup:
      showProperties (dynamic_cast<RescueLookup *>(object));
      break;
    case R_RescueLookupItem:
      noProperties (type);
      break;
    case R_RescueLookupString:
      showProperties (dynamic_cast<RescueLookupString *>(object));
      break;
    case R_RescueLookupTable:
      showProperties (dynamic_cast<RescueLookupTable *>(object));
      break;
    case R_RescueArrayDouble:
      showProperties (dynamic_cast<RescueArrayDouble *>(object));
      break;
    case R_RescueArrayByte:
      showProperties (dynamic_cast<RescueArrayByte *>(object));
      break;
    case R_RescueArrayShort:
      showProperties (dynamic_cast<RescueArrayShort *>(object));
      break;
    case R_RescueTrimLoop:
      showProperties (dynamic_cast<RescueTrimLoop *>(object));
      break;
    case R_RescueArrayFloat:
      showProperties (dynamic_cast<RescueArrayFloat *>(object));
      break;
    case R_RescueMacroVolume:
      showProperties (dynamic_cast<RescueMacroVolume *>(object));
      break;
    case R_RescueModelPropertyGroup:
      showProperties (dynamic_cast<RescueModelPropertyGroup *>(object));
      break;
    case R_RescueTimeStepGroup:
      showProperties (dynamic_cast<RescueTimeStepGroup *>(object));
      break;
    case R_RescueBitArray:
      noProperties (type);
      break;
    case R_RescueColor:
      noProperties (type);
      break;
    case R_RescueHistory:
      showProperties (dynamic_cast<RescueHistory *>(object));
      break;
    case R_RescueHistoryObject:
      noProperties (type);
      break;
    case R_RescueArray2dVector:
      showProperties (dynamic_cast<RescueArray2dVector *>(object));
      break;
    case R_RescueArray3dVector:
      showProperties (dynamic_cast<RescueArray3dVector *>(object));
      break;
    case R_RescueColorTable:
      noProperties (type);
      break;
    case R_RescueColorTableList:
      noProperties (type);
      break;
    case R_RescueGeobody:
      showProperties (dynamic_cast<RescueGeobody *>(object));
      break;
    case R_RescueGeobodyPart:
      noProperties (type);
      break;
    case R_RescueGeobodySurface:
      noProperties (type);
      break;
    case R_RescueGeobodyVolume:
      noProperties (type);
      break;
    case R_RescueSurfaceToGrid:
      noProperties (type);
      break;
    case R_RescuePolyLineNodeBUG:
      noProperties (type);
      break;
    case R_RescueIJSurface:
      showProperties (dynamic_cast<RescueIJSurface *>(object));
      break;
    case R_RescueCellShape:
      noProperties (type);
      break;
    case R_RescueEventDescr:
      showProperties (dynamic_cast<RescueEventDescr *>(object));
      break;
    case R_RescueLogicalOrder:
      noProperties (type);
      break;
    case R_RescueFaultIntersection:
      showProperties (dynamic_cast<RescueFaultIntersection *>(object));
      break;
    case R_RescuePolyLineNodeAvatar:
      noProperties (type);
      break;
    case R_RescueWireframeStub:
      noProperties (type);
      break;
    case R_RescuePolyLineStub:
      noProperties (type);
      break;
    case R_RescuePolyLineNodeAvatarStub:
      noProperties (type);
      break;
    case R_RescueTrimVertexStub:
      noProperties (type);
      break;
    case R_RescueEdgeSetStub:
      noProperties (type);
      break;
    case R_RescueTrimLoopStub:
      noProperties (type);
      break;
    case R_RescueWireframe:
      showProperties (dynamic_cast<RescueWireframe *>(object));
      break;
    case R_RescueWireframeOwner:
      noProperties (type);
      break;
    case R_RescueArrayInt:
      showProperties (dynamic_cast<RescueArrayInt *>(object));
      break;
    case R_RescueReservoir:
      showProperties (dynamic_cast<RescueReservoir *>(object));
      break;
    case R_RescueQuadrupletArray:
      showProperties (dynamic_cast<RescueQuadrupletArray *>(object));
      break;
    case R_RescuePolyLineNodeUVT:
      noProperties (type);
      break;
    case R_RescueStairSteppedFault:
      showProperties (dynamic_cast<RescueStairSteppedFault *>(object));
      break;
    case R_RescueLogicalOrderEntry:
      noProperties (type);
      break;
    case R_RescueDocumentList:
      showProperties (dynamic_cast<RescueDocumentList *>(object));
      break;
    case R_RescueDocument:
      showProperties (dynamic_cast<RescueDocument *>(object));
      break;
    default:
      noProperties (type);
    }
  }
  else if (data->getContext ())
    showProperties (data->getContext ());

  else if (data->getRadialGrid ())
    showProperties (data->getRadialGrid ());

  else if (data->getIJKCellPair ())
    showProperties (data->getIJKCellPair ());

  else
    error (wxString::Format ("No object found for this %s", type.c_str()));
}
//----------------------------------------------------------------------------

void RescueExplorerTree::noProperties (const wxString &type)
{
  if (!type.empty ())
    ::wxLogStatus ("No properties for %s", type.c_str());

  RescueExplorerPairs pairs;
  m_list->update (pairs);
}
//-----------------------------------------------------------------------

void RescueExplorerTree::error (const wxString &message)
{
  ::wxLogStatus (message);
  
  RescueExplorerPairs pairs;
  m_list->update (pairs);
}
//----------------------------------------------------------------------------

void RescueExplorerTree::updateList (RescueExplorerPairs &pairs)
{
  m_list->update (pairs);
}
