// -*-C++-*-
/////////////////////////////////////////////////////////////////////////////
// Name:        RESCUE Explorer
// Purpose:     Explore RESCUE exports
// Copyright:   (C) 2006, Carl Godkin (carl@dgi.com)
// License:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#ifndef _RESCUEEXPLORERTREE_H_
#define _RESCUEEXPLORERTREE_H_

#include <wx/treectrl.h>

#include "RescueModel.h"

#include "RescueExplorerPairs.h"

class PropertyListCtrl;
class RescueLookupString;
class RescueLookupTable;
class RescueRadialGrid;

class RescueExplorerTree : public wxTreeCtrl {

  DECLARE_DYNAMIC_CLASS(RescueExplorerTree)

public:
  //! Default ctor for wxWindows RTTI
  RescueExplorerTree () {;}

  RescueExplorerTree (wxWindow *parent, PropertyListCtrl *list,
		      const wxString &inputFile,
		      bool delayGeometryLoad,
		      bool delayPropertyLoad,
		      wxWindowID id=-1,
		      const wxPoint &pos = wxDefaultPosition,
		      const wxSize &size = wxDefaultSize,
		      long style = wxTR_HAS_BUTTONS | wxTR_HIDE_ROOT |
		      wxTR_LINES_AT_ROOT,
		      const wxValidator &validator = wxDefaultValidator,
		      const wxString &name = wxTreeCtrlNameStr);
  
  // Open RESCUE file
  void open (const wxString &inputFile);

  // Reload current file.
  void reload ();

  // Add model vertex to X,Y,Z values.
  void setAddVertex (bool addVertex);

  // Are we adding model vertex to X,Y,Z values?
  bool getAddVertex () const { return m_addVertex; }

  // Show only corners of corner point arrays under RescueGeometry
  void showCornersOnly (bool showCornersOnly);

  // Are we showing only corners?
  bool getShowCornersOnly () const { return m_showCornersOnly; }

  // Draw tree showing RESCUE hierarchy
  void displayModel ();

  // Generic version has ExpandAll, but not MSW version.
  void expandAll (const wxTreeItemId& item);

private:

  // When displaying node and cell corner coordinates, only show first
  // and last few coordinates in I, J, and K to avoid overflow on 
  // large grids.
  static int s_NUM_COORDS_IN_CORNER;

  wxTreeItemId addItem (const wxTreeItemId &parent, const wxString &type,
			RCHString *label, RescueObject *object);

  wxTreeItemId addItem (const wxTreeItemId &parent, const wxString &type,
			const wxString &label, RescueObject *object);

  void addLeaf (const wxTreeItemId &parent, RescueArrayDouble *array);
  void addLeaf (const wxTreeItemId &parent, RescueArrayFloat *array);
  void addLeaf (const wxTreeItemId &parent, RescueArrayByte *array);
  void addLeaf (const wxTreeItemId &parent, RescueArrayShort *array);
  void addLeaf (const wxTreeItemId &parent, RescueArrayInt *array);
  void addLeaf (const wxTreeItemId &parent, RescueArray2dVector *array);
  void addLeaf (const wxTreeItemId &parent, RescueArray3dVector *array);
  void addLeaf (const wxTreeItemId &parent, RescueBlock *block);
  void addLeaf (const wxTreeItemId &parent, RescueBlockUnit *blockUnit);
  void addLeaf (const wxTreeItemId &parent,
		RescueBlockUnitHorizonSurface *surface);
  void addLeaf (const wxTreeItemId &parent, RescueBlockUnitSide *side);
  void addLeaf (const wxTreeItemId &parent, RescueContext *context);
  void addLeaf (const wxTreeItemId &parent,
		RescueCoordinateSystem *coordinateSystem);
  void addLeaf (const wxTreeItemId &parent, RescueCoordinateSystemAxis *axis);
  void addLeaf (const wxTreeItemId &parent, RescueDocument *document);
  void addLeaf (const wxTreeItemId &parent, RescueDocumentList *documentList);
  void addLeaf (const wxTreeItemId &parent, RescueEdgeSet *edgeSet);
  void addLeaf (const wxTreeItemId &parent, RescueEventDescr *eventDescr);
  void addLeaf (const wxTreeItemId &parent,
		RescueFaultIntersection *faultIntersection);
  void addLeaf (const wxTreeItemId &parent, RescueGeobody *geobody);
  void addLeaf (const wxTreeItemId &parent, RescueGeometry *geometry);
  void addLeaf (const wxTreeItemId &parent, RescueGeometryUnit *gunit);
  void addLeaf (const wxTreeItemId &parent, RescueGrid *grid);
  void addLeaf (const wxTreeItemId &parent, RescueGridAxis *axis);
  void addLeaf (const wxTreeItemId &parent, RescueHistory *history);
  void addLeaf (const wxTreeItemId &parent, RescueHorizon *horizon);
  void addLeaf (const wxTreeItemId &parent, RescueIJKCellPair *pair);
  void addLeaf (const wxTreeItemId &parent, RescueIJSurface *surface);
  void addLeaf (const wxTreeItemId &parent, RescueLookup *lookup);
  void addLeaf (const wxTreeItemId &parent, RescueLookupString *item);
  void addLeaf (const wxTreeItemId &parent, RescueLookupTable *item);
  void addLeaf (const wxTreeItemId &parent, RescueMacroVolume *macroVolume);
  void addLeaf (const wxTreeItemId &parent, RescueModel *model);
  void addLeaf (const wxTreeItemId &parent,
		RescueModelPropertyGroup *propertyGroup);
  void addLeaf (const wxTreeItemId &parent, RescuePolyLine *polyLine);
  void addLeaf (const wxTreeItemId &parent, RescueProperty *property);
  void addLeaf (const wxTreeItemId &parent,
		RescuePropertyGroup *propertyGroup);
  void addLeaf (const wxTreeItemId &parent, RescueQuadrupletArray *array);
  void addLeaf (const wxTreeItemId &parent, RescueRadialGrid *grid);
  void addLeaf (const wxTreeItemId &parent, RescueReferenceSurface *surface);
  void addLeaf (const wxTreeItemId &parent, RescueReservoir *reservoir);
  void addLeaf (const wxTreeItemId &parent, RescueSection *section);
  void addLeaf (const wxTreeItemId &parent, RescueStairSteppedFault *fault);
  void addLeaf (const wxTreeItemId &parent, RescueSurface *surface);
  void addLeaf (const wxTreeItemId &parent,
		RescueTimeStepGroup *timeStepGroup);
  void addLeaf (const wxTreeItemId &parent, RescueTrimLoop *trimLoop);
  void addLeaf (const wxTreeItemId &parent, RescueTrimVertex *trimVertex);
  void addLeaf (const wxTreeItemId &parent, RescueTripletArray *array);
  void addLeaf (const wxTreeItemId &parent, RescueUnit *unit);
  void addLeaf (const wxTreeItemId &parent, RescueVertex *vertex);
  void addLeaf (const wxTreeItemId &parent, RescueWellbore *wellbore);
  void addLeaf (const wxTreeItemId &parent, RescueWellboreProperty *property);
  void addLeaf (const wxTreeItemId &parent, RescueWellboreSampling *wbs);
  void addLeaf (const wxTreeItemId &parent, RescueWireframe *wireframe);

  void addProperties (RescueExplorerPairs &pairs, RescueArray *array);
  void addProperties (RescueExplorerPairs &pairs, RescueTripletArray *array);
  void addProperties (RescueExplorerPairs &pairs,
		      RescueReferenceSurface *surface);
  void addProperties (RescueExplorerPairs &pairs, RescueSurface *surface);

  void showProperties (RescueArrayDouble *array);
  void showProperties (RescueArrayFloat *array);
  void showProperties (RescueArrayByte *array);
  void showProperties (RescueArrayShort *array);
  void showProperties (RescueArrayInt *array);
  void showProperties (RescueArray2dVector *array);
  void showProperties (RescueArray3dVector *array);
  void showProperties (RescueBlock *block);
  void showProperties (RescueBlockUnit *blockUnit);
  void showProperties (RescueBlockUnitHorizonSurface *surface);
  void showProperties (RescueBlockUnitSide *side);
  void showProperties (RescueContext *context);
  void showProperties (RescueCoordinateSystem *coordinateSystem);
  void showProperties (RescueCoordinateSystemAxis *axis);
  void showProperties (RescueDocument *document);
  void showProperties (RescueDocumentList *documentList);
  void showProperties (RescueEdgeSet *edgeSet);
  void showProperties (RescueEventDescr *eventDescr);
  void showProperties (RescueFaultIntersection *faultIntersection);
  void showProperties (RescueGeobody *geobody);
  void showProperties (RescueGeometry *geometry);
  void showProperties (RescueGeometryUnit *gunit);
  void showProperties (RescueGrid *grid);
  void showProperties (RescueGridAxis *axis);
  void showProperties (RescueHistory *history);
  void showProperties (RescueHorizon *horizon);
  void showProperties (RescueIJKCellPair *pair);
  void showProperties (RescueIJSurface *surface);
  void showProperties (RescueLookup *lookup);
  void showProperties (RescueLookupString *item);
  void showProperties (RescueLookupTable *item);
  void showProperties (RescueMacroVolume *macroVolume);
  void showProperties (RescueModel *model);
  void showProperties (RescueModelPropertyGroup *propertyGroup);
  void showProperties (RescuePolyLine *polyLine);
  void showProperties (RescueProperty *property);
  void showProperties (RescuePropertyGroup *propertyGroup);
  void showProperties (RescueQuadrupletArray *array);
  void showProperties (RescueRadialGrid *grid);
  void showProperties (RescueReferenceSurface *surface);
  void showProperties (RescueReservoir *reservoir);
  void showProperties (RescueSection *section);
  void showProperties (RescueStairSteppedFault *fault);
  void showProperties (RescueSurface *surface);
  void showProperties (RescueTimeStepGroup *timeStepGroup);
  void showProperties (RescueTrimLoop *trimLoop);
  void showProperties (RescueTrimVertex *trimVertex);
  void showProperties (RescueTripletArray *array);
  void showProperties (RescueUnit *unit);
  void showProperties (RescueVertex *vertex);
  void showProperties (RescueWellbore *wellbore);
  void showProperties (RescueWellboreProperty *property);
  void showProperties (RescueWellboreSampling *wbs);
  void showProperties (RescueWireframe *wireframe);

  void noProperties (const wxString &type = wxEmptyString);

  void error (const wxString &message);

  void updateList (RescueExplorerPairs &pairs);

  void OnSelection (wxTreeEvent &event);
  void selectItem (const wxTreeItemId &id);

  const char *orientationName (RescueCoordinateSystem::Orientation o);

  RescueContext *m_context;
  RescueModel *m_model;
  
  PropertyListCtrl *m_list;
  bool m_addVertex;
  bool m_showCornersOnly;
  wxString m_inputFile;

  bool m_delayGeometryLoad;
  bool m_delayPropertyLoad;

  DECLARE_EVENT_TABLE()
};

class RescueExplorerTreeItemData : public wxTreeItemData {
public:
  RescueExplorerTreeItemData (const wxString &type,
			      RescueContext *context) :
    m_type (type), m_context (context), m_object (0),
    m_radial (0), m_pair (0) { ; }

  RescueExplorerTreeItemData (const wxString &type,
			      const wxString &label,
			      RescueObject *object) :
    m_type (type), m_label (label), m_context (0), m_object (object),
    m_radial (0), m_pair (0) { ; }

  RescueExplorerTreeItemData (const wxString &type,
			      const wxString &label,
			      RescueRadialGrid *radial) :
    m_type (type), m_label (label), m_context (0), m_object (0),
    m_radial (radial), m_pair (0) { ; }

  RescueExplorerTreeItemData (const wxString &type,
			      const wxString &label,
			      RescueIJKCellPair *pair) :
    m_type (type), m_label (label), m_context (0), m_object (0),
    m_radial (0), m_pair (pair) { ; }

  wxString getType () const { return m_type; }
  wxString getLabel () const { return m_label; }
  RescueObject *getObject () const { return m_object; }
  RescueContext *getContext () const { return m_context; }
  RescueRadialGrid *getRadialGrid () const { return m_radial; }
  RescueIJKCellPair *getIJKCellPair () const { return m_pair; }

private:
  wxString m_type;
  wxString m_label;
  RescueContext *m_context;
  RescueObject *m_object;
  RescueRadialGrid *m_radial;
  RescueIJKCellPair *m_pair;
};

#endif // _RESCUEEXPLORERTREE_H_
