/////////////////////////////////////////////////////////////////////////////
// Name:        RESCUE Explorer
// Purpose:     Explore RESCUE exports
// Copyright:   (C) 2006, Carl Godkin (carl@dgi.com)
// License:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

#include "RescueVertex.h"
#include "RescueExplorerTree.h"

// When displaying node and cell corner coordinates, only show first
// and last few coordinates in I, J, and K to avoid overflow on 
// large grids.
int RescueExplorerTree::s_NUM_COORDS_IN_CORNER = 5;

//----------------------------------------------------------------------------

void RescueExplorerTree::addLeaf (const wxTreeItemId &parent,
				  RescueGeometry *geometry)
{
  wxTreeItemId leaf =
    addItem (parent,
	     wxString::Format ("Geometry%s",
			       geometry->LGRType() !=
			       RescueGeometry::R_NOT_LGR ? " (LGR)" : ""),
	     geometry->ReferenceName(), geometry);
  
  geometry->EnsureLoaded ();

  if (geometry->Grid())
    addLeaf (leaf, geometry->Grid());
  
  if (geometry->RadialGrid ())
    addLeaf (leaf, geometry->RadialGrid ());

  for (int i=0; i<geometry->GeometryUnitCount(); i++)
    addLeaf (leaf, geometry->NthGeometryUnit (i));

  for (int i=0; i<geometry->StairStepCount(); i++)
    addLeaf (leaf, geometry->NthStairSteppedFault (i));

  for (int i=0; i<geometry->PropertyCount(); i++)
    addLeaf (leaf, geometry->NthRescueProperty(i));

  for (int i=0; i<geometry->LGRCount(); i++)
    addLeaf (leaf, geometry->NthLGR (i));
}
//----------------------------------------------------------------------------

void RescueExplorerTree::showProperties (RescueGeometry *geometry)
{
  RescueExplorerPairs pairs;
  pairs.add ("ReferenceID", geometry->ReferenceID ());
  pairs.add ("LGRCount", geometry->LGRCount ());
  pairs.add ("LGRType",
	     geometry->LGRType() ==
	     RescueGeometry::R_NOT_LGR ? "NOT_LGR" :
	     geometry->LGRType() ==
	     RescueGeometry::R_RULE_BASED ? "RULE_BASED" :
	     geometry->LGRType() ==
	     RescueGeometry::R_WEIGHT_BASED ? "WEIGHT_BASED" :
	     geometry->LGRType() ==
	     RescueGeometry::R_RULE_BASED_NOSPAN ? "RULE_BASED_NOSPAN" :
	     geometry->LGRType() ==
	     RescueGeometry::R_WEIGHT_BASED_NOSPAN ? "WEIGHT_BASED_NOSPAN" :
	     geometry->LGRType() ==
	     RescueGeometry::R_UNCONSTRAINED ? "UNCONSTRAINED" :
	     "unknown");
  pairs.add ("LGRGridType",
	     geometry->LGRGridType () ==
	     RescueGeometry::R_STRUCTURED_CPG ? "STRUCTURED_CPG" :
	     geometry->LGRGridType () ==
	     RescueGeometry::R_STRUCTURED_RADIAL ? "STRUCTURED_RADIAL" :
	     geometry->LGRGridType () ==
	     RescueGeometry::R_UNSTRUCTURED_TETRAHEDRAL ?
	     "UNSTRUCTURED_TETRAHEDRAL" :
	     geometry->LGRGridType () ==
	     RescueGeometry::R_UNSTRUCTURED_2D_POLYHEDRAL ?
	     "UNSTRUCTURED_2D_POLYHEDRAL" :
	     geometry->LGRGridType () ==
	     RescueGeometry::R_UNSTRUCTURED_3D_POLYHEDRAL ?
	     "UNSTRUCTURED_3D_POLYHEDRAL" :
	     "unknown");
  pairs.add ("ParentILowBound", geometry->ParentILowBound ());
  pairs.add ("ParentICount", geometry->ParentICount ());
  pairs.add ("ParentJLowBound", geometry->ParentJLowBound ());
  pairs.add ("ParentJCount", geometry->ParentJCount ());
  pairs.add ("ParentKLowBound", geometry->ParentKLowBound ());
  pairs.add ("ParentKCount", geometry->ParentKCount ());
  pairs.add ("XWeightLength", geometry->XWeightLength ());
  pairs.add ("XWeight", geometry->XWeight(), geometry->XWeightLength ());
  pairs.add ("YWeightLength", geometry->YWeightLength ());
  pairs.add ("YWeight", geometry->YWeight(), geometry->YWeightLength ());
  pairs.add ("ZWeightLength", geometry->ZWeightLength ());
  pairs.add ("ZWeight", geometry->ZWeight(), geometry->ZWeightLength ());
  pairs.add ("MissingValue", geometry->MissingValue ());

  if (geometry->Grid()) {
    int iN = geometry->Grid()->Axis(0)->Count();
    int jN = geometry->Grid()->Axis(1)->Count();
    int kN = geometry->Grid()->Axis(2)->Count();
    float x, y, z;

    double xv, yv, zv;
    if (m_addVertex &&
	geometry->ParentModel() &&
	geometry->ParentModel()->CoordinateSystem() &&
	geometry->ParentModel()->CoordinateSystem()->Vertex()) {
      xv = geometry->ParentModel()->CoordinateSystem()->Vertex()->X();
      yv = geometry->ParentModel()->CoordinateSystem()->Vertex()->Y();
      zv = geometry->ParentModel()->CoordinateSystem()->Vertex()->Z();
    }
    else
      xv = yv = zv = 0.0;

    if (geometry->LGRGridType() == RescueGeometry::R_STRUCTURED_CPG) {
      for (int k=0; k<kN-1; k++) {

	// For large grids, skip middle.
	if (m_showCornersOnly &&
	    k >= s_NUM_COORDS_IN_CORNER &&
	    k < kN - s_NUM_COORDS_IN_CORNER - 1)
	  continue;
	
	for (int j=0; j<jN-1; j++) {
	  // For large grids, skip middle.
	  if (m_showCornersOnly &&
	      j >= s_NUM_COORDS_IN_CORNER &&
	      j < jN - s_NUM_COORDS_IN_CORNER - 1)
	    continue;

	  for (int i=0; i<iN-1; i++) {
	    // For large grids, skip middle.
	    if (m_showCornersOnly &&
		i >= s_NUM_COORDS_IN_CORNER &&
		i < iN - s_NUM_COORDS_IN_CORNER - 1)
	      continue;
	    
	    wxArrayString values;
	    for (int ic=0; ic<8; ic++) {
	      geometry->Values (i, j, k, ic, x, y, z);
	      values.Add (wxString::Format ("(%f, %f, %f)",
					    x + xv,
					    y + yv,
					    z + zv));
	    }
	    pairs.add (wxString::Format ("cell(%d,%d,%d)", i+1, j+1, k+1),
		       "<cell corners>",
		       values);
	  }
	}
      }

      for (int k=0; k<kN; k++) {
	// For large grids, skip middle.
	if (m_showCornersOnly &&
	    k >= s_NUM_COORDS_IN_CORNER &&
	    k < kN - s_NUM_COORDS_IN_CORNER)
	  continue;
	
	for (int j=0; j<jN; j++) {
	  // For large grids, skip middle.
	  if (m_showCornersOnly &&
	      j >= s_NUM_COORDS_IN_CORNER &&
	      j < jN - s_NUM_COORDS_IN_CORNER)
	    continue;
	  
	  for (int i=0; i<iN; i++) {
	    // For large grids, skip middle.
	    if (m_showCornersOnly &&
		i >= s_NUM_COORDS_IN_CORNER &&
		i < iN - s_NUM_COORDS_IN_CORNER)
	      continue;
	    
	    if (geometry->Values (i, j, k, x, y, z))
	      pairs.add (wxString::Format ("node(%d,%d,%d)", i, j, k),
			 wxString::Format ("(%f, %f, %f)",
					   x + xv,
					   y + yv,
					   z + zv));
	    else
	      pairs.add (wxString::Format ("node(%d,%d,%d)", i, j, k),
			 "<split node>");
	  }
	}
      }
    }

    // Cell status.
    for (int k=0; k<kN-1; k++) {
      // For large grids, skip middle.
      if (m_showCornersOnly &&
	  k >= s_NUM_COORDS_IN_CORNER &&
	  k < kN - s_NUM_COORDS_IN_CORNER - 1)
	continue;
      
      for (int j=0; j<jN-1; j++) {
	// For large grids, skip middle.
	if (m_showCornersOnly &&
	    j >= s_NUM_COORDS_IN_CORNER &&
	    j < jN - s_NUM_COORDS_IN_CORNER - 1)
	  continue;
	
	for (int i=0; i<iN-1; i++) {
	  // For large grids, skip middle.
	  if (m_showCornersOnly &&
	      i >= s_NUM_COORDS_IN_CORNER &&
	      i < iN - s_NUM_COORDS_IN_CORNER - 1)
	    continue;
	  
	  wxString value;
	  switch (geometry->CellStatus (i, j, k)) {
	  case RescueGeometry::R_ACTIVE:
	    value = "ACTIVE";
	    break;
	  case RescueGeometry::R_INACTIVE:
	    value = "INACTIVE";
	    break;
	  case RescueGeometry::R_TRUNCATED:
	    value = "TRUNCATED";
	    break;
	  case RescueGeometry::R_INACTIVATED_BY_PILLAR:
	    value = "INACTIVATED_BY_PILLAR";
	    break;
	  default:
	    value = "(unknown)";
	    break;
	  }
	  pairs.add (wxString::Format ("status(%d,%d,%d)", i+1, j+1, k+1),
		     value);
	}
      }
    }
  }
  
  pairs.add ("GeometryType",
	     geometry->GeometryType () ==
	     RescueGeometry::R_XY_ORTHOGONAL ? "XY_ORTHOGONAL" :
	     geometry->GeometryType () ==
	     RescueGeometry::R_SQUASHED_ORTHOGONAL ? "SQUASHED_ORTHOGONAL" :
	     geometry->GeometryType () ==
	     RescueGeometry::R_CONFORMABLE ? "CONFORMABLE" :
	     "unknown");
  pairs.add ("CornerValuesLength", geometry->CornerValuesLength ());
  pairs.add ("NodeValuesLength", geometry->NodeValuesLength ());
  pairs.add ("AllCornerValuesLength", geometry->AllCornerValuesLength ());

  if (geometry->LGRGridType() == RescueGeometry::R_STRUCTURED_CPG)
    pairs.add ("LayerCornerValuesLength",
	       geometry->LayerCornerValuesLength ());

  pairs.add ("StairStepCount", geometry->StairStepCount());

  pairs.add ("IsProportional", geometry->IsProportional () != 0);
  pairs.add ("TopOffset", geometry->TopOffset ());
  pairs.add ("BottomOffset", geometry->BottomOffset ());
  pairs.add ("IsReferenced", geometry->IsReferenced () != 0);
  pairs.add ("SurfaceOffset", geometry->SurfaceOffset ());
  pairs.add ("KLayerThickness", geometry->KLayerThickness ());
  pairs.add ("LapType",
	     geometry->LapType() == RescueGeometry::R_ONLAP ? "ONLAP" :
	     geometry->LapType() == RescueGeometry::R_OFFLAP ? "OFFLAP" :
	     "unknown");
  pairs.add ("GeometryUnitCount", geometry->GeometryUnitCount());

  updateList (pairs);
}
//----------------------------------------------------------------------------

void RescueExplorerTree::addLeaf (const wxTreeItemId &parent,
				  RescueGeometryUnit *gunit)
{
  wxTreeItemId leaf = addItem (parent, "GeometryUnit", wxEmptyString, gunit);
}
//----------------------------------------------------------------------------

void RescueExplorerTree::showProperties (RescueGeometryUnit *gunit)
{
  RescueExplorerPairs pairs;
  pairs.add ("TopLayerNdx", gunit->TopLayerNdx());
  pairs.add ("BottomLayerNdx", gunit->BottomLayerNdx());
  updateList (pairs);
}
