/////////////////////////////////////////////////////////////////////////////
// Name:        RESCUE Explorer
// Purpose:     Explore RESCUE exports
// Copyright:   (C) 2006, Carl Godkin (carl@dgi.com)
// License:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

#include <wx/cmdline.h>
#include <wx/splash.h>

#include "RescueExplorerFrame.h"

#include "RescueModel.h"

#define VERSION  "2.7"

class RescueExplorer : public wxApp {
public:
  virtual bool OnInit();

private:
  wxBitmap rescueBitmapFunc ();
};

DECLARE_APP(RescueExplorer);

//-----------------------------------------------------------------------

IMPLEMENT_APP (RescueExplorer)

//-----------------------------------------------------------------------

bool RescueExplorer::OnInit()
{
  static const wxCmdLineEntryDesc cmdLineDesc[] = {
#ifdef __WXMSW__
    { wxCMD_LINE_SWITCH, "c", "console", "Open console window" },
#endif
    { wxCMD_LINE_OPTION, "d", "directory", "Startup directory" }, \
    { wxCMD_LINE_SWITCH, "g", "no_delay_geometry_load",
      "Do not delay geometry load" },
    { wxCMD_LINE_SWITCH, "h", "help", "Provide help", wxCMD_LINE_VAL_NONE, \
      wxCMD_LINE_OPTION_HELP  }, \
    { wxCMD_LINE_SWITCH, "p", "delay_property_load", "Delay property load" },
    { wxCMD_LINE_SWITCH, "s", "no_splash_screen",
      "Do not display splash screen while loading" },
    { wxCMD_LINE_SWITCH, "V", "version", "Print version" },
    { wxCMD_LINE_PARAM, 0, 0, "RESCUE file", wxCMD_LINE_VAL_STRING,
      wxCMD_LINE_PARAM_OPTIONAL },
    { wxCMD_LINE_NONE }
  };

  wxCmdLineParser parser (cmdLineDesc, argc, argv);

  // Exit if user wanted (automatic) help or made errors on command line.
  if (parser.Parse ())
    return false;

  // Show console on Windows when DGIDEBUG=console
#ifdef __WXMSW__
  if (parser.Found ("c") || parser.Found ("V")) {
    AllocConsole ();
    
    freopen ("CONOUT$", "wt", stdout);
    freopen ("CONOUT$", "wt", stderr);
    freopen ("CONIN$", "rt", stdin);
  }
#endif // __WXMSW__

  wxString version = wxString::Format ("%s v%d.%d",
				       VERSION,
				       FILE_VERSION, FILE_SUBVERSION);

  // If -V is specified, print version and exit.
  if (parser.Found ("V")) {
    wxString s =
      wxString::Format ("%s: Version %s, %s",
			::wxFileNameFromPath (argv[0]),
			version.c_str(), __DATE__);
    fprintf (stderr, "%s\n", s.c_str());
#ifdef __WXMSW__
    wxMessageDialog msg (0, s, "Version",
			 wxOK|wxICON_INFORMATION|wxCENTRE);
    msg.ShowModal ();
#endif    
    exit (0);
  }

  // Check for directory passed in.
  wxString directory;
  if (parser.Found ("directory", &directory))
    // Change directory.
    ::wxSetWorkingDirectory (directory);

  // Get optional input file from command line.
  wxString inputFile;
  if (parser.GetParamCount())
    inputFile = parser.GetParam (0);
  else {
    // If there's no file on the command line, ask for one.
    wxFileDialog dialog (0, "Select RESCUE file", "", "",
			 "RESCUE files (*.bin)|*.bin",
			 wxFD_OPEN | wxFD_FILE_MUST_EXIST);
    if (dialog.ShowModal() == wxID_OK)
      inputFile = dialog.GetPath();
    else
      // If user chooses cancel in file dialog, exit.
      return false;
  }

  if (!parser.Found ("s")) {
    wxSplashScreen *splash =
      new wxSplashScreen (rescueBitmapFunc (),
			  wxSPLASH_CENTRE_ON_PARENT | wxSPLASH_TIMEOUT,
			  1000, NULL, -1);
    
    wxYield();
  }

  // Create the main application window
  RescueExplorerFrame *frame =
    new RescueExplorerFrame (0, inputFile,
			     !parser.Found ("g"), parser.Found ("p"),
			     version);
			     
  frame->Show (true);

  return true;
}
//-----------------------------------------------------------------------

wxBitmap RescueExplorer::rescueBitmapFunc ()
{
  static const unsigned char data[] = 
    {
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,6,7,18,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,8,8,16,18,18,39,23,23,47,34,34,69,39,39,82,47,
      47,100,56,56,116,63,63,125,67,67,134,81,79,138,111,107,128,92,90,148,120,113,105,91,88,
      128,89,84,115,105,96,91,76,74,123,76,72,85,38,37,69,26,26,52,17,17,33,14,14,33,
      3,3,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,2,2,4,8,8,16,22,22,44,40,40,80,55,55,111,66,66,133,80,80,162,86,
      86,174,90,90,181,93,93,187,94,95,195,103,102,196,110,109,209,146,144,190,172,166,181,188,177,
      198,208,190,197,186,175,186,192,180,181,187,176,188,185,172,181,182,168,165,171,161,173,164,158,160,
      153,149,157,144,139,145,133,124,123,99,94,136,82,79,114,59,55,87,29,29,66,18,18,40,14,
      14,28,4,4,8,1,1,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,8,8,17,21,21,42,37,37,74,56,56,116,83,83,170,97,97,195,103,
      103,207,106,106,216,110,110,222,112,112,225,112,112,227,113,113,230,112,112,227,119,120,240,161,156,
      222,175,170,223,169,169,252,185,184,247,195,193,235,194,188,236,195,185,213,200,189,224,208,195,217,
      208,199,227,214,194,198,205,184,183,195,183,189,195,183,181,193,179,175,187,174,170,174,165,163,134,
      131,185,132,123,149,96,93,142,65,66,145,60,60,129,52,52,106,33,33,70,20,20,44,9,9,
      19,3,3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,5,5,12,22,22,45,44,44,88,77,77,157,100,100,205,108,
      108,218,113,113,228,118,118,237,118,118,240,121,121,244,121,121,244,123,123,246,123,123,246,123,123,
      246,123,123,249,122,122,249,132,132,251,148,148,255,138,138,252,138,138,255,155,155,253,173,168,204,
      175,175,250,190,193,255,206,197,233,216,207,228,217,211,243,221,216,248,219,206,219,216,210,238,217,
      193,196,208,195,207,202,191,202,199,185,186,188,179,193,172,167,169,146,141,172,117,115,179,92,90,
      155,73,74,153,66,66,140,61,61,123,52,52,105,36,36,77,22,22,44,8,8,18,2,2,5,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,16,34,36,36,71,68,68,137,99,
      99,203,113,113,230,117,117,237,122,122,244,122,122,247,126,126,252,128,128,253,128,128,253,130,130,
      255,132,132,255,132,132,255,132,132,255,132,132,255,132,132,255,133,133,255,130,130,253,127,127,254,
      134,134,250,158,158,251,188,188,255,201,202,255,201,201,254,209,210,254,218,219,255,227,230,255,235,
      219,229,238,230,242,235,238,255,233,231,248,224,207,218,219,212,241,219,217,245,202,193,213,185,180,
      221,170,167,207,152,151,216,148,144,174,138,134,157,99,97,181,95,93,158,73,73,154,70,70,142,
      64,64,131,58,58,119,46,46,96,31,31,65,20,20,39,5,5,11,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,24,24,50,54,
      54,111,88,88,181,114,114,230,123,123,247,124,124,248,126,126,251,129,129,254,132,132,255,133,133,
      255,137,137,255,137,137,254,137,137,254,138,138,254,140,140,254,141,141,254,141,141,254,140,140,254,
      138,138,254,139,139,254,148,148,254,158,158,254,179,179,254,204,204,254,214,214,254,224,224,254,221,
      221,254,218,218,254,223,223,254,226,225,254,229,230,255,232,233,255,230,230,254,226,228,255,220,217,
      247,212,201,232,202,196,237,188,182,241,188,180,200,170,166,229,170,163,186,142,140,221,151,149,187,
      141,138,205,130,126,158,84,85,190,81,81,168,75,75,154,71,71,143,66,66,134,59,59,122,52,
      52,105,36,36,74,22,22,44,11,11,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,16,16,34,60,60,121,98,98,201,121,121,247,127,127,254,128,128,254,132,132,255,134,134,
      255,136,136,255,138,138,254,141,141,254,142,142,254,144,144,254,146,146,254,146,146,254,146,146,254,
      147,147,254,148,148,254,146,146,254,151,151,254,163,163,254,176,176,254,198,198,254,213,213,254,217,
      217,254,222,222,254,225,225,254,229,229,254,231,231,254,233,233,254,227,227,254,221,221,254,216,216,
      254,210,211,254,202,202,254,199,199,254,197,198,255,190,191,255,178,179,255,171,172,255,171,167,233,
      162,160,244,170,165,227,169,164,203,159,157,210,156,154,209,146,145,196,115,115,196,115,112,170,98,
      97,170,81,81,167,76,76,154,70,70,143,63,63,130,59,59,121,53,53,109,39,39,79,22,22,
      46,9,9,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,23,23,46,69,69,139,93,93,191,121,121,248,129,129,255,130,130,
      254,135,135,255,138,138,255,140,140,254,143,143,254,145,145,254,147,147,254,148,148,254,150,150,254,
      151,151,254,152,152,254,152,152,254,154,154,254,155,155,254,154,154,254,163,163,254,183,183,254,205,
      205,254,222,222,254,232,232,254,235,235,254,230,230,254,227,227,254,219,219,254,213,213,254,212,212,
      254,212,212,254,209,209,254,205,205,254,195,195,254,188,188,254,180,180,254,180,180,254,183,183,254,
      181,181,254,175,175,254,173,173,254,167,168,255,168,169,255,167,167,255,169,167,232,157,157,248,157,
      155,207,154,152,209,151,149,197,133,131,189,106,105,206,93,93,191,89,89,177,80,80,162,71,71,
      151,68,68,139,63,63,128,56,56,115,50,50,103,35,35,73,22,22,45,9,9,19,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,11,23,59,59,118,103,103,207,125,125,
      252,131,131,255,135,135,255,138,138,255,140,140,255,144,144,254,148,148,254,149,149,254,152,152,254,
      153,153,254,154,154,254,156,156,254,157,157,254,157,157,254,160,160,254,160,160,254,160,160,254,161,
      161,254,160,160,254,183,183,254,210,210,254,226,226,254,232,232,254,230,230,254,227,227,254,220,220,
      254,210,210,254,200,200,254,200,200,254,204,204,254,203,203,254,200,200,254,195,195,254,182,182,254,
      174,174,254,170,170,254,166,166,254,167,167,254,168,168,254,167,167,254,166,166,254,171,171,254,176,
      177,255,178,177,248,183,175,212,171,172,255,167,167,246,157,156,242,161,160,200,149,148,216,144,142,
      196,117,116,193,131,126,171,94,93,187,86,86,171,78,78,160,72,72,145,65,65,135,60,60,121,
      52,52,108,44,44,92,33,33,68,20,20,41,3,3,9,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,7,15,47,47,
      95,90,90,182,120,120,241,131,131,255,134,134,255,139,139,255,144,144,254,148,148,254,149,149,254,
      152,152,254,154,154,254,157,157,254,159,159,254,161,161,254,161,161,254,162,162,254,164,164,254,165,
      165,254,165,165,254,165,165,254,165,165,254,165,165,254,167,167,254,187,187,254,210,210,254,222,222,
      254,217,217,254,215,215,254,209,209,254,206,206,254,200,200,254,195,195,254,192,192,254,195,195,254,
      195,195,254,195,195,254,193,193,254,188,188,254,180,180,254,170,170,254,157,157,254,154,154,254,164,
      164,254,175,175,254,183,183,254,189,189,254,191,191,254,195,194,253,197,196,245,196,197,254,195,188,
      231,190,181,223,172,171,241,177,169,206,170,167,225,164,157,184,123,123,220,124,121,194,124,120,164,
      95,95,185,80,80,165,72,72,150,69,69,139,63,63,128,55,55,114,48,48,99,42,42,84,24,
      24,51,6,6,13,1,1,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,20,20,43,69,69,144,114,114,230,129,129,255,135,135,255,139,139,255,143,143,254,
      148,148,254,152,152,254,154,154,254,156,156,254,159,159,254,161,161,254,162,162,254,166,166,254,166,
      166,254,166,166,254,170,170,254,171,171,254,171,171,254,170,170,254,170,170,254,170,170,254,170,170,
      254,171,171,254,176,176,254,187,187,254,197,197,254,197,197,254,196,196,254,192,192,254,196,196,254,
      201,201,254,195,195,254,194,194,254,194,194,254,195,195,254,200,200,254,207,207,254,206,206,254,197,
      197,254,182,182,254,166,166,254,167,167,254,178,178,254,188,188,254,202,202,254,212,212,254,223,223,
      254,229,229,254,229,229,255,230,230,254,232,231,252,219,219,252,208,196,224,191,179,221,173,172,250,
      177,170,208,162,159,219,152,149,191,139,135,187,123,120,179,87,87,184,82,82,164,76,76,155,70,
      70,143,65,65,133,58,58,120,50,50,105,41,41,86,34,34,69,21,21,41,4,4,8,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,6,6,11,42,42,85,102,102,207,127,127,255,132,132,255,
      139,139,255,143,143,254,148,148,254,152,152,254,156,156,254,159,159,254,161,161,254,163,163,254,166,
      166,254,167,167,254,168,168,254,171,171,254,171,171,254,173,173,254,175,175,254,175,175,254,175,175,
      254,175,175,254,175,175,254,175,175,254,175,175,254,175,175,254,175,175,254,174,174,254,174,174,254,
      174,174,254,173,173,254,172,172,254,178,178,254,185,185,254,182,182,254,184,184,254,188,188,254,192,
      192,254,206,206,254,217,217,254,217,217,254,208,208,254,197,197,254,189,189,254,195,195,254,211,211,
      254,219,219,254,225,225,254,230,230,254,244,244,254,249,249,254,249,249,254,251,251,254,255,255,255,
      247,248,255,223,224,255,211,199,222,187,182,242,171,168,227,178,172,217,160,159,233,130,128,196,102,
      103,213,97,97,193,88,88,177,82,82,168,77,77,157,71,71,146,65,65,134,59,59,120,52,52,
      108,46,46,92,36,36,73,24,24,49,8,8,17,1,1,3,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,12,24,67,67,135,
      116,116,234,132,132,255,137,137,255,143,143,254,148,148,254,152,152,254,156,156,254,160,160,254,161,
      161,254,166,166,254,167,167,254,171,171,254,172,172,254,173,173,254,176,176,254,176,176,254,176,176,
      254,178,178,254,180,180,254,180,180,254,180,180,254,180,180,254,180,180,254,180,180,254,180,180,254,
      180,180,254,179,179,254,180,180,254,179,179,254,177,177,254,175,175,254,175,175,254,175,175,254,173,
      173,254,171,171,254,170,170,254,175,175,254,185,185,254,206,206,254,224,224,254,233,233,254,230,230,
      254,225,225,254,229,229,254,235,235,254,240,240,254,246,246,254,250,250,254,251,251,254,254,254,254,
      255,255,254,255,255,254,255,255,254,254,254,254,255,255,254,254,254,254,234,235,255,208,208,252,190,
      183,210,178,174,228,160,159,239,131,127,210,104,105,216,98,98,197,93,93,186,90,90,181,82,82,
      170,78,78,160,73,73,148,67,67,137,60,60,123,53,53,110,45,45,95,37,37,77,30,30,62,
      16,16,32,3,3,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,20,20,42,85,85,171,123,123,249,134,134,255,140,140,255,145,145,254,150,150,254,155,
      155,254,159,159,254,161,161,254,164,164,254,169,169,254,171,171,254,172,172,254,175,175,254,176,176,
      254,177,177,254,180,180,254,180,180,254,180,180,254,181,181,254,184,184,254,184,184,254,184,184,254,
      184,184,254,184,184,254,184,184,254,184,184,254,184,184,254,184,184,254,183,183,254,181,181,254,180,
      180,254,180,180,254,180,180,254,179,179,254,176,176,254,176,176,254,175,175,254,176,176,254,199,199,
      254,228,228,254,247,247,254,253,253,254,255,255,254,253,253,254,253,253,254,255,255,254,255,255,254,
      255,255,254,255,255,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,255,255,254,255,255,254,233,234,255,203,203,252,183,178,225,159,157,236,138,134,209,106,107,
      220,102,102,204,100,100,199,95,95,191,90,90,181,85,85,171,77,77,160,74,74,149,67,67,135,
      60,60,123,53,53,110,46,46,97,38,38,77,30,30,62,18,18,39,2,2,6,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,33,33,70,95,95,194,133,133,255,136,136,255,142,
      142,254,148,148,254,153,153,254,157,157,254,161,161,254,164,164,254,169,169,254,171,171,254,175,175,
      254,177,177,254,177,177,254,180,180,254,181,181,254,182,182,254,185,185,254,185,185,254,185,185,254,
      184,184,254,186,186,254,188,188,254,189,189,254,189,189,254,189,189,254,189,189,254,189,189,254,187,
      187,254,186,186,254,185,185,254,184,184,254,185,185,254,185,185,254,183,183,254,181,181,254,180,180,
      254,179,179,254,179,179,254,200,200,254,235,235,254,252,252,254,255,255,254,254,254,254,254,254,254,
      255,255,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,255,255,254,245,245,254,216,216,
      255,186,186,255,154,154,255,123,124,245,110,110,221,106,106,211,102,102,206,100,100,200,96,96,192,
      90,90,182,85,85,172,80,80,161,72,72,148,67,67,137,60,60,123,54,54,110,46,46,96,39,
      39,80,31,31,63,19,19,40,4,4,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,44,44,90,108,
      108,219,135,135,255,138,138,255,145,145,254,150,150,254,154,154,254,159,159,254,163,163,254,169,169,
      254,172,172,254,174,174,254,177,177,254,179,179,254,181,181,254,181,181,254,184,184,254,185,185,254,
      186,186,254,189,189,254,189,189,254,189,189,254,189,189,254,189,189,254,190,190,254,190,190,254,190,
      190,254,190,190,254,190,190,254,190,190,254,189,189,254,189,189,254,189,189,254,189,189,254,188,188,
      254,186,186,254,186,186,254,185,185,254,184,184,254,191,191,254,201,201,254,230,230,254,255,255,254,
      255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,255,255,254,249,249,254,219,219,254,189,189,255,153,153,249,117,117,231,114,114,230,
      113,113,226,106,106,212,101,101,205,97,97,197,92,92,188,88,88,180,83,83,169,77,77,159,72,
      72,148,68,68,136,60,60,124,54,54,110,46,46,96,38,38,78,30,30,62,20,20,43,5,5,
      10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,49,49,98,111,111,226,135,135,255,141,141,255,146,146,254,153,153,254,158,158,
      254,162,162,254,168,168,254,169,169,254,174,174,254,178,178,254,179,179,254,182,182,254,184,184,254,
      186,186,254,186,186,254,188,188,254,190,190,254,190,190,254,191,191,254,193,193,254,194,194,254,194,
      194,254,194,194,254,194,194,254,194,194,254,194,194,254,194,194,254,194,194,254,194,194,254,193,193,
      254,193,193,254,193,193,254,190,190,254,190,190,254,189,189,254,188,188,254,186,186,254,198,198,254,
      224,224,254,244,244,254,253,253,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,255,255,254,245,245,254,227,227,254,210,210,254,
      187,187,254,150,150,251,118,118,241,119,119,237,115,115,230,110,110,222,104,104,214,100,100,205,95,
      95,196,91,91,187,87,87,178,82,82,169,77,77,158,73,73,146,67,67,136,60,60,123,53,53,
      110,44,44,94,37,37,77,29,29,61,21,21,46,5,5,9,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,1,1,2,50,50,102,115,115,230,135,135,255,141,141,
      255,149,149,254,154,154,254,158,158,254,163,163,254,168,168,254,173,173,254,174,174,254,178,178,254,
      182,182,254,183,183,254,186,186,254,186,186,254,188,188,254,190,190,254,190,190,254,194,194,254,194,
      194,254,194,194,254,194,194,254,196,196,254,198,198,254,198,198,254,198,198,254,198,198,254,198,198,
      254,198,198,254,198,198,254,197,197,254,194,194,254,193,193,254,194,194,254,193,193,254,193,193,254,
      193,193,254,196,196,254,212,212,254,236,236,254,252,252,254,255,255,254,255,255,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,255,255,254,
      255,255,254,244,244,254,214,214,254,185,185,254,157,157,255,133,133,251,123,123,248,121,121,242,117,
      117,233,114,114,229,110,110,220,105,105,213,100,100,205,96,96,196,91,91,187,87,87,178,82,82,
      165,77,77,155,72,72,146,64,64,133,58,58,120,52,52,106,44,44,92,35,35,75,27,27,58,
      20,20,40,4,4,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,3,6,52,52,
      106,115,115,231,135,135,255,141,141,255,149,149,254,155,155,254,160,160,254,164,164,254,168,168,254,
      173,173,254,178,178,254,179,179,254,182,182,254,184,184,254,187,187,254,188,188,254,191,191,254,192,
      192,254,195,195,254,195,195,254,195,195,254,198,198,254,198,198,254,199,199,254,199,199,254,199,199,
      254,199,199,254,199,199,254,199,199,254,198,198,254,197,197,254,198,198,254,200,200,254,203,203,254,
      209,209,254,211,211,254,203,203,254,205,205,254,222,222,254,238,238,254,251,251,254,255,255,254,255,
      255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      255,255,254,255,255,254,255,255,254,255,255,254,253,253,254,249,249,254,219,219,254,174,174,254,138,
      138,254,130,130,254,127,127,253,124,124,249,119,119,241,117,117,236,114,114,228,109,109,218,105,105,
      211,101,101,202,96,96,193,92,92,184,87,87,174,82,82,165,76,76,154,68,68,141,64,64,129,
      58,58,118,49,49,103,43,43,88,34,34,70,26,26,56,20,20,43,3,3,6,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,52,52,104,119,119,240,134,134,255,141,141,254,148,148,254,155,155,254,
      162,162,254,168,168,254,171,171,254,172,172,254,177,177,254,182,182,254,183,183,254,186,186,254,188,
      188,254,191,191,254,192,192,254,195,195,254,195,195,254,197,197,254,199,199,254,199,199,254,199,199,
      254,200,200,254,202,202,254,202,202,254,202,202,254,203,203,254,204,204,254,203,203,254,202,202,254,
      209,209,254,209,209,254,210,210,254,217,217,254,230,230,254,228,228,254,223,223,254,236,236,254,253,
      253,254,255,255,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,255,255,254,253,253,254,239,239,254,229,229,254,221,221,254,207,
      207,254,196,196,254,176,176,254,147,147,254,136,136,254,133,133,254,130,130,254,127,127,254,123,123,
      247,119,119,241,114,114,232,110,110,224,108,108,218,104,104,209,99,99,200,95,95,191,90,90,182,
      86,86,172,78,78,160,72,72,151,68,68,139,62,62,128,55,55,113,48,48,101,40,40,85,31,
      31,66,23,23,49,15,15,35,3,3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,36,36,75,112,112,226,133,133,255,
      141,141,255,149,149,254,155,155,254,162,162,254,167,167,254,173,173,254,176,176,254,177,177,254,182,
      182,254,186,186,254,187,187,254,190,190,254,192,192,254,194,194,254,196,196,254,197,197,254,199,199,
      254,200,200,254,201,201,254,202,202,254,202,202,254,204,204,254,204,204,254,204,204,254,204,204,254,
      205,205,254,205,205,254,205,205,254,225,225,254,239,239,254,222,222,254,208,208,254,208,208,254,215,
      215,254,223,223,254,236,236,254,252,252,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,255,255,254,255,255,254,
      255,255,254,255,255,254,255,255,254,255,255,254,255,255,254,255,255,254,255,255,254,254,254,254,227,
      227,254,178,178,254,167,167,254,164,164,254,154,154,254,148,148,254,144,144,254,140,140,254,138,138,
      254,136,136,254,132,132,254,129,129,254,125,125,253,122,122,246,117,117,237,115,115,231,110,110,223,
      106,106,213,100,100,205,96,96,197,91,91,187,87,87,178,82,82,168,78,78,158,73,73,147,65,
      65,135,60,60,123,53,53,109,47,47,96,39,39,78,30,30,61,22,22,48,14,14,32,2,2,
      6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      18,18,37,101,101,207,135,135,255,141,141,254,149,149,254,156,156,254,162,162,254,167,167,254,172,
      172,254,177,177,254,179,179,254,182,182,254,185,185,254,187,187,254,191,191,254,193,193,254,196,196,
      254,196,196,254,199,199,254,200,200,254,201,201,254,202,202,254,204,204,254,204,204,254,204,204,254,
      206,206,254,206,206,254,206,206,254,207,207,254,206,206,254,209,209,254,226,226,254,249,249,254,239,
      239,254,212,212,254,204,204,254,210,210,254,229,229,254,244,244,254,252,252,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,255,255,254,255,255,254,255,255,254,
      255,255,254,254,254,254,253,253,254,253,253,254,245,245,254,237,237,254,234,234,254,230,230,254,227,
      227,254,227,227,254,228,228,254,217,217,254,186,186,254,158,158,254,155,155,254,154,154,254,151,151,
      254,149,149,254,147,147,254,143,143,254,141,141,254,138,138,254,135,135,254,131,131,254,128,128,254,
      124,124,250,120,120,241,117,117,235,112,112,227,109,109,218,106,106,212,102,102,205,97,97,195,93,
      93,186,88,88,176,82,82,166,74,74,153,69,69,142,64,64,131,57,57,119,51,51,104,44,44,
      90,36,36,74,26,26,55,21,21,43,12,12,26,3,3,6,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,4,4,9,85,85,170,130,130,255,139,139,255,147,147,254,154,
      154,254,161,161,254,168,168,254,173,173,254,177,177,254,179,179,254,182,182,254,187,187,254,189,189,
      254,191,191,254,194,194,254,197,197,254,199,199,254,201,201,254,201,201,254,202,202,254,203,203,254,
      205,205,254,206,206,254,206,206,254,209,209,254,209,209,254,208,208,254,207,207,254,210,210,254,229,
      229,254,238,238,254,246,246,254,253,253,254,239,239,254,221,221,254,216,216,254,225,225,254,247,247,
      254,255,255,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,255,255,254,255,255,254,
      254,254,254,249,249,254,247,247,254,246,246,254,242,242,254,227,227,254,215,215,254,211,211,254,200,
      200,254,191,191,254,189,189,254,184,184,254,181,181,254,179,179,254,177,177,254,174,174,254,167,167,
      254,163,163,254,161,161,254,158,158,254,155,155,254,152,152,254,150,150,254,145,145,254,143,143,254,
      140,140,254,137,137,254,134,134,254,130,130,254,126,126,253,122,122,246,120,120,240,115,115,232,111,
      111,224,108,108,218,103,103,209,98,98,199,94,94,190,89,89,181,85,85,172,78,78,160,74,74,
      149,68,68,137,60,60,126,56,56,114,48,48,100,40,40,84,33,33,67,25,25,49,18,18,37,
      6,6,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,3,6,60,60,122,125,
      125,251,139,139,255,145,145,254,154,154,254,158,158,254,166,166,254,173,173,254,177,177,254,178,178,
      254,181,181,254,186,186,254,190,190,254,192,192,254,194,194,254,196,196,254,199,199,254,201,201,254,
      202,202,254,203,203,254,205,205,254,206,206,254,209,209,254,209,209,254,209,209,254,210,210,254,209,
      209,254,211,211,254,224,224,254,240,240,254,253,253,254,255,255,254,253,253,254,254,254,254,253,253,
      254,249,249,254,241,241,254,243,243,254,253,253,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,252,252,254,249,249,254,245,245,254,239,239,254,230,230,254,219,219,254,209,
      209,254,197,197,254,188,188,254,187,187,254,183,183,254,182,182,254,179,179,254,177,177,254,176,176,
      254,173,173,254,171,171,254,169,169,254,168,168,254,164,164,254,161,161,254,159,159,254,156,156,254,
      154,154,254,150,150,254,148,148,254,144,144,254,141,141,254,139,139,254,135,135,254,130,130,254,128,
      128,254,125,125,250,121,121,245,116,116,236,113,113,228,111,111,222,106,106,213,101,101,203,97,97,
      194,92,92,185,88,88,176,83,83,167,77,77,156,70,70,144,65,65,133,58,58,120,52,52,106,
      44,44,92,39,39,77,29,29,58,21,21,42,12,12,29,3,3,8,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,43,43,88,114,114,232,135,135,255,145,145,254,151,151,254,159,159,254,163,163,
      254,168,168,254,174,174,254,178,178,254,182,182,254,187,187,254,190,190,254,192,192,254,194,194,254,
      197,197,254,198,198,254,201,201,254,203,203,254,205,205,254,206,206,254,208,208,254,209,209,254,210,
      210,254,210,210,254,211,211,254,210,210,254,212,212,254,227,227,254,249,249,254,255,255,254,255,255,
      254,254,254,254,254,254,254,254,254,254,254,254,254,255,255,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,253,253,254,247,247,254,241,241,254,236,236,254,224,224,254,219,219,254,219,
      219,254,220,220,254,220,220,254,206,206,254,195,195,254,193,193,254,190,190,254,190,190,254,186,186,
      254,186,186,254,185,185,254,182,182,254,180,180,254,178,178,254,175,175,254,174,174,254,169,169,254,
      165,165,254,164,164,254,161,161,254,159,159,254,154,154,254,153,153,254,149,149,254,145,145,254,144,
      144,254,139,139,254,137,137,254,134,134,254,130,130,254,126,126,254,124,124,250,119,119,241,115,115,
      231,111,111,224,108,108,218,103,103,208,99,99,199,94,94,190,90,90,181,85,85,172,79,79,161,
      74,74,149,67,67,138,61,61,126,56,56,114,49,49,101,43,43,87,34,34,69,25,25,53,18,
      18,38,13,13,25,3,3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,18,38,97,97,195,135,135,255,141,141,
      255,149,149,254,155,155,254,163,163,254,167,167,254,173,173,254,177,177,254,182,182,254,186,186,254,
      190,190,254,191,191,254,194,194,254,196,196,254,198,198,254,201,201,254,203,203,254,206,206,254,207,
      207,254,207,207,254,210,210,254,210,210,254,211,211,254,213,213,254,213,213,254,213,213,254,229,229,
      254,251,251,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,248,248,254,238,238,254,230,
      230,254,215,215,254,205,205,254,203,203,254,201,201,254,204,204,254,207,207,254,198,198,254,196,196,
      254,194,194,254,194,194,254,191,191,254,189,189,254,186,186,254,185,185,254,182,182,254,180,180,254,
      178,178,254,175,175,254,173,173,254,171,171,254,169,169,254,165,165,254,163,163,254,158,158,254,157,
      157,254,154,154,254,150,150,254,148,148,254,144,144,254,141,141,254,139,139,254,135,135,254,131,131,
      254,129,129,254,125,125,251,120,120,245,116,116,236,113,113,228,111,111,222,106,106,213,102,102,203,
      97,97,194,93,93,185,88,88,176,83,83,167,76,76,155,70,70,144,65,65,132,57,57,120,53,
      53,109,47,47,95,38,38,78,29,29,62,22,22,45,15,15,33,11,11,23,1,1,2,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,7,
      14,77,77,157,131,131,255,139,139,255,146,146,254,154,154,254,160,160,254,168,168,254,173,173,254,
      177,177,254,182,182,254,185,185,254,187,187,254,192,192,254,194,194,254,196,196,254,198,198,254,201,
      201,254,203,203,254,205,205,254,208,208,254,210,210,254,210,210,254,211,211,254,213,213,254,214,214,
      254,213,213,254,215,215,254,231,231,254,255,255,254,255,255,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,253,253,254,247,247,254,229,229,254,209,209,254,205,205,254,206,206,254,203,203,
      254,201,201,254,201,201,254,198,198,254,198,198,254,197,197,254,194,194,254,192,192,254,190,190,254,
      189,189,254,186,186,254,184,184,254,182,182,254,179,179,254,178,178,254,175,175,254,174,174,254,169,
      169,254,166,166,254,164,164,254,161,161,254,158,158,254,155,155,254,153,153,254,148,148,254,146,146,
      254,144,144,254,139,139,254,138,138,254,134,134,254,129,129,254,126,126,253,122,122,247,120,120,240,
      115,115,231,110,110,224,107,107,218,103,103,208,98,98,199,94,94,190,89,89,181,84,84,173,79,
      79,160,73,73,149,67,67,137,61,61,127,56,56,114,49,49,101,42,42,86,34,34,69,25,25,
      54,18,18,38,13,13,26,4,4,8,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,39,39,79,119,119,241,134,134,255,145,145,255,151,151,254,
      158,158,254,164,164,254,172,172,254,176,176,254,180,180,254,185,185,254,188,188,254,192,192,254,194,
      194,254,196,196,254,199,199,254,201,201,254,203,203,254,205,205,254,208,208,254,210,210,254,210,210,
      254,212,212,254,214,214,254,214,214,254,215,215,254,214,214,254,236,236,254,255,255,254,255,255,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,255,255,254,255,255,254,236,236,
      254,211,211,254,207,207,254,206,206,254,206,206,254,203,203,254,202,202,254,201,201,254,199,199,254,
      198,198,254,195,195,254,194,194,254,192,192,254,189,189,254,187,187,254,185,185,254,183,183,254,180,
      180,254,178,178,254,176,176,254,173,173,254,171,171,254,168,168,254,165,165,254,163,163,254,159,159,
      254,157,157,254,153,153,254,150,150,254,148,148,254,143,143,254,142,142,254,139,139,254,134,134,254,
      131,131,254,129,129,254,124,124,250,119,119,241,116,116,235,112,112,227,109,109,219,106,106,212,101,
      101,202,97,97,195,92,92,185,88,88,176,82,82,165,75,75,153,69,69,141,64,64,131,58,58,
      119,53,53,106,44,44,93,39,39,79,29,29,61,21,21,45,15,15,30,12,12,23,0,0,1,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,11,23,84,84,172,
      133,133,255,140,140,254,150,150,254,155,155,254,163,163,254,168,168,254,172,172,254,178,178,254,182,
      182,254,187,187,254,190,190,254,194,194,254,196,196,254,198,198,254,201,201,254,203,203,254,205,205,
      254,207,207,254,210,210,254,210,210,254,212,212,254,214,214,254,214,214,254,215,215,254,216,216,254,
      224,224,254,252,252,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,255,255,254,248,248,254,224,224,254,210,210,254,206,206,254,206,206,254,
      205,205,254,202,202,254,202,202,254,200,200,254,198,198,254,196,196,254,194,194,254,194,194,254,190,
      190,254,189,189,254,186,186,254,184,184,254,182,182,254,179,179,254,177,177,254,174,174,254,173,173,
      254,169,169,254,166,166,254,162,162,254,161,161,254,158,158,254,155,155,254,152,152,254,148,148,254,
      147,147,254,143,143,254,138,138,254,135,135,254,134,134,254,129,129,254,124,124,254,121,121,245,118,
      118,237,115,115,231,110,110,222,106,106,213,101,101,204,97,97,197,92,92,187,88,88,176,83,83,
      168,77,77,158,71,71,146,66,66,136,60,60,124,56,56,113,49,49,100,42,42,85,34,34,67,
      24,24,50,17,17,37,13,13,24,5,5,10,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,48,48,98,126,126,254,135,135,255,146,146,254,155,155,254,159,159,254,166,
      166,254,174,174,254,178,178,254,181,181,254,185,185,254,189,189,254,192,192,254,194,194,254,199,199,
      254,201,201,254,203,203,254,205,205,254,208,208,254,210,210,254,210,210,254,213,213,254,214,214,254,
      214,214,254,216,216,254,216,216,254,222,222,254,243,243,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      244,244,254,219,219,254,207,207,254,205,205,254,206,206,254,204,204,254,202,202,254,201,201,254,199,
      199,254,198,198,254,195,195,254,193,193,254,192,192,254,190,190,254,187,187,254,185,185,254,183,183,
      254,181,181,254,178,178,254,176,176,254,173,173,254,170,170,254,168,168,254,164,164,254,162,162,254,
      158,158,254,157,157,254,153,153,254,151,151,254,148,148,254,143,143,254,140,140,254,138,138,254,133,
      133,254,129,129,254,127,127,253,124,124,249,119,119,241,115,115,233,110,110,225,108,108,217,103,103,
      209,99,99,199,94,94,190,90,90,181,85,85,173,80,80,163,74,74,150,68,68,140,62,62,128,
      58,58,117,51,51,104,44,44,91,36,36,75,28,28,57,20,20,41,13,13,28,11,11,22,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,11,22,91,91,184,132,132,255,140,
      140,254,150,150,254,157,157,254,163,163,254,169,169,254,175,175,254,180,180,254,183,183,254,187,187,
      254,192,192,254,194,194,254,196,196,254,199,199,254,203,203,254,206,206,254,206,206,254,209,209,254,
      210,210,254,212,212,254,214,214,254,214,214,254,217,217,254,217,217,254,219,219,254,236,236,254,254,
      254,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,255,255,254,254,254,254,240,240,254,223,223,254,212,212,254,206,
      206,254,206,206,254,203,203,254,202,202,254,200,200,254,198,198,254,197,197,254,195,195,254,193,193,
      254,190,190,254,188,188,254,186,186,254,183,183,254,182,182,254,179,179,254,177,177,254,174,174,254,
      172,172,254,167,167,254,165,165,254,162,162,254,160,160,254,157,157,254,153,153,254,152,152,254,147,
      147,254,144,144,254,143,143,254,138,138,254,134,134,254,132,132,254,128,128,254,124,124,251,119,119,
      243,116,116,235,114,114,227,110,110,219,105,105,211,101,101,202,96,96,195,92,92,187,87,87,176,
      82,82,164,76,76,154,71,71,145,64,64,133,60,60,122,53,53,109,46,46,95,39,39,80,30,
      30,64,24,24,48,16,16,35,12,12,24,6,6,12,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,45,45,92,122,122,245,135,135,255,146,146,254,152,152,254,159,159,254,166,166,254,173,173,
      254,178,178,254,183,183,254,186,186,254,189,189,254,193,193,254,196,196,254,198,198,254,201,201,254,
      203,203,254,206,206,254,207,207,254,209,209,254,212,212,254,214,214,254,214,214,254,217,217,254,219,
      219,254,228,228,254,241,241,254,251,251,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,255,
      255,254,255,255,254,242,242,254,218,218,254,206,206,254,206,206,254,204,204,254,202,202,254,202,202,
      254,199,199,254,198,198,254,195,195,254,193,193,254,190,190,254,188,188,254,186,186,254,183,183,254,
      182,182,254,179,179,254,177,177,254,174,174,254,173,173,254,169,169,254,167,167,254,162,162,254,161,
      161,254,158,158,254,155,155,254,152,152,254,148,148,254,147,147,254,142,142,254,138,138,254,136,136,
      254,133,133,254,128,128,254,125,125,253,123,123,246,119,119,238,115,115,229,110,110,221,105,105,214,
      101,101,204,96,96,196,92,92,188,87,87,178,82,82,168,77,77,158,73,73,147,67,67,136,60,
      60,124,55,55,114,49,49,100,42,42,86,35,35,69,27,27,55,20,20,41,13,13,29,10,10,
      19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,6,6,14,82,82,170,129,129,255,139,139,255,149,149,
      254,156,156,254,163,163,254,169,169,254,173,173,254,179,179,254,183,183,254,188,188,254,192,192,254,
      193,193,254,197,197,254,203,203,254,204,204,254,205,205,254,207,207,254,209,209,254,211,211,254,213,
      213,254,214,214,254,217,217,254,228,228,254,234,234,254,247,247,254,255,255,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,255,255,254,255,255,254,241,241,254,214,214,254,208,208,
      254,206,206,254,204,204,254,202,202,254,202,202,254,199,199,254,198,198,254,195,195,254,194,194,254,
      193,193,254,190,190,254,188,188,254,186,186,254,183,183,254,181,181,254,178,178,254,177,177,254,172,
      172,254,170,170,254,167,167,254,165,165,254,162,162,254,157,157,254,156,156,254,152,152,254,149,149,
      254,147,147,254,142,142,254,141,141,254,138,138,254,133,133,254,129,129,254,128,128,253,123,123,247,
      119,119,240,114,114,232,110,110,223,105,105,215,101,101,206,100,100,199,96,96,191,91,91,183,87,
      87,174,80,80,163,74,74,152,69,69,143,65,65,130,59,59,119,51,51,104,43,43,90,36,36,
      74,28,28,57,21,21,43,14,14,30,13,13,24,7,7,14,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,30,30,
      62,113,113,229,131,131,252,143,143,255,152,152,254,159,159,254,166,166,254,170,170,254,176,176,254,
      179,179,254,182,182,254,190,190,254,194,194,254,198,198,254,213,213,254,229,229,254,227,227,254,216,
      216,254,212,212,254,216,216,254,222,222,254,219,219,254,222,222,254,235,235,254,240,240,254,240,240,
      254,250,250,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,253,253,254,253,253,
      254,246,246,254,224,224,254,210,210,254,209,209,254,206,206,254,206,206,254,204,204,254,202,202,254,
      200,200,254,198,198,254,198,198,254,194,194,254,193,193,254,190,190,254,188,188,254,186,186,254,183,
      183,254,181,181,254,178,178,254,177,177,254,174,174,254,172,172,254,167,167,254,165,165,254,162,162,
      254,158,158,254,156,156,254,153,153,254,152,152,254,147,147,254,142,142,254,142,142,255,139,139,255,
      134,134,255,131,131,255,129,129,255,123,123,249,119,119,242,114,114,233,113,113,226,109,109,219,106,
      106,212,102,102,205,96,96,193,90,90,181,86,86,172,79,79,158,72,72,148,65,65,134,60,60,
      123,53,53,107,43,43,88,35,35,74,27,27,55,19,19,40,15,15,31,12,12,24,13,13,25,
      9,9,18,1,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,1,1,2,62,62,128,102,102,205,114,114,226,132,132,248,154,154,254,
      163,163,254,167,167,254,178,178,254,190,190,254,189,189,254,198,198,254,209,209,254,212,212,254,229,
      229,254,249,249,254,255,255,254,252,252,254,248,248,254,241,241,254,238,238,254,238,238,254,233,233,
      254,234,234,254,238,238,254,235,235,254,238,238,254,249,249,254,251,251,254,253,253,254,254,254,254,
      254,254,254,254,254,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,253,253,254,240,240,254,229,229,254,222,222,254,211,211,254,210,210,254,209,209,254,
      206,206,254,206,206,254,204,204,254,202,202,254,202,202,254,199,199,254,197,197,254,194,194,254,193,
      193,254,190,190,254,188,188,254,187,187,254,185,185,254,183,183,254,180,180,254,180,180,254,175,175,
      254,172,172,254,168,168,254,166,166,254,162,162,254,161,161,254,156,156,254,153,153,254,152,152,254,
      149,149,255,149,149,255,140,140,255,133,133,253,130,130,252,128,128,249,126,126,251,123,123,251,119,
      119,242,118,118,237,117,117,232,110,110,221,99,99,198,90,90,182,80,80,164,71,71,147,68,68,
      139,60,60,121,54,54,110,47,47,96,41,41,85,36,36,75,34,34,68,30,30,59,25,25,50,
      21,21,42,14,14,30,12,12,24,12,12,24,13,13,26,7,7,13,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,11,22,85,85,172,
      100,100,201,106,106,212,110,110,225,135,135,248,163,163,255,177,177,254,193,193,254,196,196,254,204,
      204,254,206,206,254,205,205,254,214,214,254,229,229,254,236,236,254,240,240,254,241,241,254,239,239,
      254,242,242,254,245,245,254,244,244,254,241,241,254,242,242,254,246,246,254,244,244,254,243,243,254,
      248,248,254,254,254,254,254,254,254,255,255,254,255,255,254,255,255,254,252,252,254,255,255,254,255,
      255,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,241,241,254,222,222,254,216,216,254,
      218,218,254,218,218,254,216,216,254,215,215,254,211,211,254,207,207,254,204,204,254,202,202,254,202,
      202,254,199,199,254,197,197,254,194,194,254,193,193,254,190,190,254,189,189,254,183,183,254,176,176,
      255,173,173,255,169,169,255,164,164,255,171,171,254,172,172,254,170,170,254,167,167,254,162,162,254,
      161,161,254,156,156,254,155,155,254,155,155,255,141,141,251,128,128,235,113,113,221,105,105,211,104,
      104,209,100,100,202,111,111,225,125,125,255,124,124,250,118,118,236,103,103,208,90,90,182,73,73,
      147,62,62,127,58,58,120,57,57,115,52,52,108,51,51,104,48,48,97,44,44,91,40,40,85,
      39,39,78,34,34,69,30,30,60,25,25,52,21,21,42,16,16,32,12,12,24,12,12,24,13,
      13,26,7,7,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,31,31,65,92,92,187,100,100,212,107,107,214,112,112,224,116,116,237,145,
      145,251,192,192,254,208,208,254,213,213,254,224,224,254,197,197,254,180,180,254,193,193,254,199,199,
      254,203,203,254,210,210,254,208,208,254,200,200,254,201,201,254,206,206,254,205,205,254,203,203,254,
      222,222,254,255,255,254,255,255,254,252,252,254,252,252,254,255,255,254,255,255,254,248,248,254,226,
      226,254,217,217,254,212,212,254,221,221,254,234,234,254,253,253,254,255,255,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,255,255,254,
      253,253,254,232,232,254,209,209,254,197,197,254,182,182,254,176,176,254,175,175,254,178,178,254,186,
      186,254,205,205,254,208,208,254,202,202,254,202,202,254,199,199,254,197,197,254,194,194,254,193,193,
      254,192,192,254,180,180,255,149,149,252,136,136,248,135,135,247,129,129,243,131,131,246,166,166,254,
      173,173,254,170,170,254,167,167,254,162,162,254,162,162,254,160,160,255,150,150,255,128,128,234,107,
      107,210,92,92,188,90,90,181,88,88,177,86,86,171,83,83,170,107,107,219,126,126,255,107,107,
      216,90,90,183,74,74,154,66,66,136,66,66,133,63,63,127,60,60,124,58,58,119,53,53,112,
      52,52,106,49,49,98,44,44,92,40,40,85,39,39,78,34,34,69,30,30,61,25,25,54,21,
      21,45,16,16,32,12,12,24,12,12,24,12,12,25,10,10,20,3,3,6,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,59,60,125,104,103,195,138,
      134,190,124,125,240,114,114,232,116,116,238,130,130,249,171,171,255,223,223,254,246,246,254,244,244,
      254,210,210,254,203,203,254,212,212,254,214,214,254,217,217,254,214,214,254,206,206,254,198,198,254,
      190,190,254,186,186,254,185,185,254,184,184,254,195,195,254,234,234,254,255,255,254,254,254,254,255,
      255,254,255,255,254,230,230,254,199,199,254,182,182,254,179,179,254,182,182,254,179,179,254,192,192,
      254,215,215,254,247,247,254,255,255,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,
      254,254,254,254,254,254,254,254,254,255,255,254,249,249,254,199,199,254,161,161,254,153,153,254,142,
      142,254,135,135,254,135,135,254,138,138,254,143,143,254,157,157,254,187,187,254,206,206,254,202,202,
      254,199,199,254,197,197,254,196,196,254,193,193,254,168,168,254,140,140,253,122,122,247,117,117,242,
      118,118,240,112,112,232,130,130,241,174,174,255,172,172,254,170,170,254,168,168,254,164,164,255,154,
      154,255,130,130,236,114,114,219,97,97,200,93,93,187,94,94,188,93,93,187,90,90,183,89,89,
      177,85,85,173,91,91,186,93,93,189,79,79,160,70,70,145,71,71,143,69,69,141,67,67,136,
      63,63,128,60,60,124,58,58,118,53,53,111,52,52,106,49,49,98,44,44,92,40,40,85,39,
      39,78,34,34,69,30,30,62,25,25,55,21,21,46,16,16,35,12,12,24,12,12,24,12,12,
      24,14,14,27,5,5,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,11,11,25,87,87,167,147,144,188,157,154,209,155,153,208,140,140,250,141,141,251,142,142,
      253,166,166,254,215,215,254,255,255,254,246,246,254,213,213,254,218,218,254,224,224,254,225,225,254,
      223,223,254,221,221,254,211,211,254,199,199,254,192,192,254,188,188,254,189,189,254,190,190,254,187,
      187,254,201,201,254,249,249,254,255,255,254,255,255,254,228,228,254,185,185,254,176,176,254,180,180,
      254,182,182,254,183,183,254,185,185,254,189,189,254,191,191,254,218,218,254,251,251,254,255,255,254,
      254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,254,251,251,254,245,245,254,201,
      201,254,155,155,254,139,139,254,143,143,254,142,142,254,141,141,254,141,141,254,140,140,254,136,136,
      254,132,132,254,141,141,254,181,181,254,204,204,254,199,199,254,199,199,254,191,191,254,158,158,254,
      128,128,252,122,122,250,121,121,248,121,121,243,118,118,239,116,116,239,141,141,252,179,179,254,173,
      173,254,172,172,255,162,162,255,134,134,243,115,115,222,100,100,204,98,98,198,99,99,198,96,96,
      193,94,94,189,91,91,184,89,89,179,87,87,175,84,84,170,78,78,160,75,75,153,75,75,153,
      73,73,150,72,72,146,68,68,141,65,65,134,63,63,128,60,60,124,58,58,117,53,53,110,52,
      52,106,49,49,98,44,44,92,40,40,85,39,39,78,34,34,69,30,30,62,25,25,55,21,21,
      46,16,16,37,12,12,27,12,12,24,12,12,24,13,13,26,5,5,11,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,43,41,50,109,108,190,169,164,177,175,168,
      203,171,168,218,164,159,223,164,159,222,175,172,230,187,187,255,205,205,254,222,222,254,233,233,254,
      219,219,254,226,226,254,229,229,254,228,228,254,225,225,254,219,219,254,213,213,254,202,202,254,190,
      190,254,183,183,254,184,184,254,189,189,254,190,190,254,188,188,254,216,216,254,254,254,254,240,240,
      254,193,193,254,175,175,254,181,181,254,185,185,254,184,184,254,183,183,254,187,187,254,192,192,254,
      195,195,254,197,197,254,231,231,254,254,254,254,255,255,254,254,254,254,254,254,254,254,254,254,252,
      252,254,249,249,254,240,240,254,188,188,254,156,156,254,146,146,254,145,145,254,145,145,254,142,142,
      254,141,141,254,141,141,254,140,140,254,137,137,254,137,137,254,132,132,254,134,134,254,182,182,254,
      202,202,254,183,183,254,151,151,254,127,127,254,123,123,253,123,123,248,121,121,246,121,121,243,120,
      120,240,116,116,236,139,139,249,179,179,255,169,169,255,141,141,248,115,115,225,105,105,212,102,102,
      208,100,100,204,100,100,201,98,98,196,95,95,192,93,93,187,90,90,183,89,89,178,86,86,174,
      84,84,169,80,80,164,78,78,159,76,76,155,73,73,150,72,72,146,68,68,138,64,64,133,63,
      63,128,60,60,124,58,58,117,53,53,110,52,52,106,49,49,98,44,44,92,40,40,85,39,39,
      78,34,34,69,30,30,62,25,25,55,21,21,46,16,16,37,12,12,28,12,12,24,12,12,23,
      12,12,24,12,12,23,3,3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,61,61,122,153,151,183,179,176,221,189,181,204,179,175,239,171,170,244,174,171,243,185,183,245,
      197,197,254,209,209,254,213,213,254,219,219,254,225,225,254,233,233,254,235,235,254,228,228,254,222,
      222,254,218,218,254,213,213,254,202,202,254,184,184,254,177,177,254,180,180,254,185,185,254,190,190,
      254,189,189,254,189,189,254,219,219,254,222,222,254,183,183,254,181,181,254,185,185,254,189,189,254,
      186,186,254,187,187,254,190,190,254,192,192,254,197,197,254,194,194,254,203,203,254,250,250,254,255,
      255,254,255,255,254,248,248,254,239,239,254,236,236,254,238,238,254,202,202,254,155,155,254,148,148,
      254,145,145,254,145,145,254,143,143,254,141,141,254,141,141,254,141,141,254,137,137,254,137,137,254,
      137,137,254,135,135,254,129,129,254,148,148,254,176,176,254,143,143,254,127,127,254,124,124,254,125,
      125,253,123,123,248,121,121,243,121,121,242,117,117,237,119,119,236,147,147,250,157,157,251,124,124,
      236,108,108,219,104,104,212,104,104,209,104,104,209,101,101,206,100,100,201,97,97,196,95,95,192,
      93,93,187,90,90,183,89,89,178,86,86,174,84,84,169,80,80,164,78,78,159,76,76,155,73,
      73,150,69,69,143,67,67,137,64,64,133,63,63,128,58,58,122,56,56,115,54,54,111,49,49,
      103,48,48,97,44,44,93,40,40,85,39,39,78,34,34,69,30,30,62,25,25,55,20,20,45,
      15,15,35,11,11,26,12,12,23,13,13,26,12,12,25,13,13,27,4,4,8,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,2,2,6,83,82,142,175,166,179,199,185,192,200,191,227,
      196,184,198,186,180,219,194,182,215,198,192,233,207,208,255,216,216,254,218,218,254,220,220,254,229,
      229,254,238,238,254,240,240,254,230,230,254,223,223,254,223,223,254,217,217,254,203,203,254,185,185,
      254,176,176,254,176,176,254,180,180,254,184,184,254,186,186,254,188,188,254,189,189,254,191,191,254,
      188,188,254,187,187,254,189,189,254,190,190,254,190,190,254,190,190,254,193,193,254,194,194,254,194,
      194,254,192,192,254,192,192,254,241,241,254,255,255,254,250,250,254,238,238,254,228,228,254,227,227,
      254,206,206,254,154,154,254,143,143,254,145,145,254,145,145,254,143,143,254,141,141,254,141,141,254,
      141,141,254,140,140,254,137,137,254,137,137,254,135,135,254,133,133,254,133,133,254,132,132,254,130,
      130,254,128,128,254,127,127,254,125,125,253,125,125,250,123,123,248,121,121,243,118,118,239,117,117,
      235,116,116,233,114,114,227,113,113,225,108,108,218,108,108,217,108,108,217,106,106,211,101,101,206,
      101,101,202,99,99,200,97,97,196,95,95,191,93,93,187,90,90,183,89,89,178,86,86,174,84,
      84,169,80,80,163,78,78,158,76,76,154,73,73,148,69,69,141,67,67,137,64,64,133,63,63,
      127,58,58,119,56,56,115,54,54,110,49,49,103,48,48,97,44,44,90,39,39,83,35,35,74,
      34,34,67,29,29,60,25,25,54,22,22,48,19,19,43,15,15,34,16,16,32,19,19,40,14,
      14,32,12,12,27,4,4,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,12,38,
      129,125,135,195,182,179,208,188,189,211,198,218,202,192,227,197,186,216,200,196,231,207,207,255,215,
      215,254,223,223,254,226,226,254,224,224,254,228,228,254,239,239,254,243,243,254,234,234,254,232,232,
      254,232,232,254,223,223,254,208,208,254,195,195,254,183,183,254,178,178,254,177,177,254,176,176,254,
      178,178,254,181,181,254,186,186,254,189,189,254,191,191,254,192,192,254,191,191,254,192,192,254,194,
      194,254,192,192,254,193,193,254,193,193,254,191,191,254,188,188,254,186,186,254,220,220,254,252,252,
      254,242,242,254,232,232,254,224,224,254,221,221,254,165,165,254,141,141,254,145,145,254,145,145,254,
      144,144,254,141,141,254,141,141,254,141,141,254,140,140,254,137,137,254,137,137,254,137,137,254,134,
      134,254,133,133,254,133,133,254,130,130,254,128,128,254,129,129,254,125,125,254,125,125,252,123,123,
      248,121,121,245,121,121,244,118,118,239,117,117,235,115,115,232,112,112,227,112,112,226,110,110,221,
      108,108,217,106,106,212,105,105,210,102,102,206,100,100,202,97,97,197,95,95,192,93,93,188,92,
      92,185,90,90,182,88,88,177,85,85,173,83,83,167,79,79,160,76,76,155,74,74,151,72,72,
      146,69,69,142,68,68,137,64,64,132,62,62,126,58,58,119,56,56,115,54,54,108,49,49,102,
      47,47,95,42,42,86,38,38,80,38,38,78,37,37,74,38,38,76,35,35,73,35,35,72,34,
      34,71,28,28,60,23,23,48,19,19,42,15,15,33,12,12,27,3,3,7,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,35,33,50,148,142,149,194,181,179,203,186,191,213,203,227,214,
      201,216,207,193,220,211,205,241,218,221,255,223,224,255,229,229,254,231,231,254,226,226,254,230,230,
      254,239,239,254,238,238,254,236,236,254,240,240,254,238,238,254,225,225,254,211,211,254,201,201,254,
      189,189,254,179,179,254,174,174,254,173,173,254,173,173,254,174,174,254,180,180,254,186,186,254,191,
      191,254,194,194,254,195,195,254,195,195,254,193,193,254,192,192,254,188,188,254,187,187,254,185,185,
      254,182,182,254,179,179,254,195,195,254,250,250,254,243,243,254,228,228,254,224,224,254,198,198,254,
      150,150,254,143,143,254,145,145,254,145,145,254,143,143,254,141,141,254,141,141,254,140,140,254,137,
      137,254,137,137,254,137,137,254,135,135,254,133,133,254,133,133,254,132,132,254,130,130,254,129,129,
      254,127,127,254,125,125,253,125,125,249,122,122,248,121,121,244,120,120,240,118,118,237,117,117,234,
      113,113,230,113,113,227,111,111,224,109,109,220,107,107,216,105,105,211,104,104,208,102,102,204,100,
      100,202,97,97,197,95,95,192,93,93,188,91,91,183,89,89,179,86,86,174,84,84,170,80,80,
      164,79,79,160,76,76,155,74,74,151,72,72,146,69,69,142,66,66,136,64,64,129,60,60,124,
      58,58,120,55,55,114,53,53,107,49,49,101,50,50,104,52,52,106,54,54,110,55,55,113,53,
      53,108,54,54,110,50,50,102,43,43,90,36,36,74,28,28,59,24,24,49,22,22,45,17,17,
      36,13,13,26,7,7,15,1,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,58,56,161,
      154,153,191,179,180,204,184,181,216,192,192,216,202,223,220,204,220,229,225,241,230,216,232,232,225,
      245,235,236,255,232,234,255,227,227,255,233,233,255,238,239,255,239,241,255,242,243,255,246,247,255,
      241,242,255,229,230,255,219,220,255,209,209,255,196,196,255,184,184,255,179,180,255,180,180,255,178,
      178,255,174,175,255,176,177,255,185,185,255,188,189,255,193,193,255,194,194,255,193,193,254,188,188,
      254,186,186,254,181,181,255,177,177,255,175,175,255,174,174,255,171,171,254,185,185,254,250,250,254,
      242,242,254,226,226,254,222,222,254,172,172,254,141,141,254,145,145,255,143,143,255,142,143,255,141,
      141,254,141,141,254,140,140,254,137,137,254,137,137,255,137,137,255,136,136,255,133,133,255,133,133,
      254,132,132,254,130,130,254,129,129,254,129,129,255,125,125,255,125,125,252,124,124,249,121,121,245,
      121,121,244,118,118,240,117,117,235,115,115,233,113,113,230,112,112,225,109,109,222,108,108,218,106,
      106,216,106,106,212,102,102,206,100,100,202,98,98,199,96,96,195,94,94,190,93,93,187,91,91,
      184,89,89,179,86,86,174,84,84,170,80,80,164,79,79,160,76,76,156,74,74,150,70,70,144,
      68,68,140,64,64,133,63,63,129,60,60,124,59,59,119,54,54,112,51,51,104,63,63,127,72,
      72,147,72,72,147,73,73,145,68,68,137,59,59,124,54,54,112,50,50,100,42,42,88,36,36,
      73,30,30,62,23,23,51,21,21,44,19,19,37,14,14,27,13,13,25,3,3,5,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,79,71,71,164,158,158,187,177,176,193,183,189,213,199,221,223,198,
      200,229,228,247,235,223,237,240,220,226,239,238,252,237,236,254,225,211,230,208,206,245,196,191,199,
      195,191,198,190,181,188,176,163,170,180,166,172,188,178,186,184,178,188,177,168,177,179,174,213,196,
      198,255,194,195,255,188,182,216,173,170,198,164,164,245,170,167,191,166,164,223,166,165,231,175,174,
      232,180,176,223,175,174,244,184,184,255,182,182,255,174,174,255,167,168,255,166,167,255,164,164,255,
      168,168,255,167,167,255,174,174,255,239,240,255,232,233,255,227,227,255,203,204,255,154,154,255,142,
      143,255,144,145,255,142,143,255,143,143,255,143,143,255,140,140,255,137,138,255,138,138,255,138,139,
      255,139,139,255,137,137,255,136,136,255,132,132,255,129,130,255,129,129,255,129,129,255,128,128,255,
      127,128,255,127,127,255,122,122,249,121,121,248,120,120,246,117,118,242,117,117,239,113,114,234,112,
      112,231,110,110,227,109,109,225,107,107,221,107,107,219,106,106,214,103,103,209,101,101,202,97,97,
      198,96,96,193,93,93,188,92,92,185,90,90,181,87,87,176,85,85,171,82,82,167,79,79,161,
      77,77,157,75,75,153,73,73,148,69,69,142,68,68,138,65,65,133,63,63,128,59,59,121,57,
      57,117,54,54,111,48,48,101,75,75,150,85,85,170,77,77,155,71,71,144,65,65,133,60,60,
      123,54,54,109,48,48,98,41,41,87,36,36,74,31,31,63,24,24,51,22,22,45,18,18,37,
      14,14,27,12,12,24,3,3,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,91,85,85,169,162,
      162,186,176,176,192,181,184,195,185,195,197,185,188,200,193,199,188,175,177,186,178,180,178,168,172,
      171,155,158,163,145,147,156,141,151,141,130,128,137,124,122,131,116,114,118,103,101,111,97,96,132,
      115,113,133,119,117,120,107,102,132,118,125,154,138,153,150,138,153,140,129,128,127,119,115,125,117,
      136,118,110,103,105,95,103,97,86,100,95,86,97,73,70,75,81,77,97,153,151,229,165,160,204,
      123,120,196,132,122,143,120,109,132,116,103,126,110,99,120,114,113,202,159,161,255,228,228,255,226,
      226,255,225,225,255,190,188,212,146,145,227,144,139,192,130,122,143,116,109,132,107,100,142,108,109,
      214,134,133,254,143,143,255,129,129,247,130,122,169,112,105,126,83,78,109,81,82,162,134,134,255,
      132,132,255,125,124,249,142,138,191,129,120,143,89,81,113,73,73,148,114,114,238,122,118,199,116,
      106,128,115,103,131,127,115,143,129,119,147,127,118,144,122,113,139,118,111,137,111,105,131,82,78,
      100,55,54,103,88,88,182,101,101,205,97,97,197,96,96,193,93,93,189,91,91,184,89,89,179,
      86,86,175,84,84,170,81,81,165,79,79,160,76,76,156,74,74,151,72,72,147,69,69,142,68,
      68,138,64,64,131,61,61,126,59,59,120,56,56,116,54,54,109,47,47,98,74,74,150,83,83,
      167,76,76,153,71,71,143,63,63,131,59,59,121,54,54,109,47,47,97,41,41,86,36,36,74,
      31,31,63,24,24,52,23,23,45,18,18,38,14,14,29,12,12,24,3,3,5,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,2,8,8,8,10,9,
      9,15,13,13,30,28,28,112,106,106,155,147,147,151,143,143,158,149,149,150,141,140,143,134,134,
      136,123,121,124,116,115,125,116,116,131,117,117,137,119,118,126,114,113,115,100,98,120,105,105,137,
      118,118,140,120,120,132,115,115,123,111,111,127,115,115,131,117,117,135,120,121,142,125,123,137,116,
      112,112,95,91,80,71,70,61,55,55,64,55,50,58,52,52,55,49,46,62,53,49,82,70,67,
      78,67,65,87,78,75,108,98,109,107,98,100,90,80,89,86,76,70,73,64,58,68,56,50,74,
      63,56,72,63,82,125,120,200,199,187,213,174,168,212,151,137,139,143,131,130,120,108,117,110,98,
      99,99,90,82,96,86,79,88,78,75,99,88,102,129,114,133,127,111,128,94,82,100,59,49,48,
      42,39,30,37,36,25,55,54,78,109,108,212,126,117,179,115,101,113,83,76,75,60,56,47,46,
      41,34,70,61,66,126,111,129,132,117,124,124,111,103,121,110,103,128,112,106,127,113,106,128,114,
      108,112,100,94,104,96,89,73,68,62,40,40,32,17,17,22,41,41,89,91,91,186,98,98,197,
      94,94,191,91,91,185,89,89,181,87,87,176,85,85,172,85,85,170,81,81,165,79,79,160,75,
      75,155,72,72,148,70,70,144,68,68,139,65,65,135,63,63,129,60,60,125,59,59,120,54,54,
      114,52,52,106,54,54,111,77,77,157,81,81,164,75,75,152,69,69,142,65,65,133,58,58,117,
      54,54,109,47,47,97,41,41,83,36,36,73,31,31,63,24,24,52,23,23,45,18,18,39,14,
      14,32,12,12,25,3,3,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,3,2,2,9,8,8,10,9,9,15,14,14,16,15,15,35,34,34,56,52,
      52,68,61,61,95,86,86,136,122,122,140,123,123,140,124,124,129,118,118,128,118,118,128,118,118,
      124,113,113,125,114,114,122,112,112,114,106,106,120,112,112,121,113,113,130,119,119,145,135,135,154,
      142,142,144,131,131,130,119,119,128,117,117,133,119,119,126,112,112,125,111,111,106,94,94,94,81,
      81,86,74,74,86,74,74,72,64,64,67,58,58,69,59,59,65,59,59,84,76,76,106,95,95,
      135,117,117,141,123,123,123,109,109,104,93,93,89,81,81,70,62,62,69,59,56,64,56,55,67,
      61,58,74,67,67,72,64,64,80,70,70,100,88,88,126,109,107,134,119,124,130,117,118,103,91,
      95,77,69,67,82,74,73,90,80,77,86,77,75,90,82,82,106,95,95,110,97,96,89,76,72,
      67,56,51,39,33,28,45,38,33,53,44,43,64,57,57,94,85,84,126,110,110,101,87,94,50,
      44,44,46,40,34,59,54,52,96,89,89,128,117,117,139,124,121,143,126,121,149,131,128,154,136,
      137,154,135,135,145,127,127,126,110,110,123,106,106,84,75,75,39,35,35,38,35,35,38,34,33,
      18,17,23,12,12,27,39,39,79,76,76,156,94,94,191,92,92,186,89,89,179,87,87,175,85,
      85,171,81,81,167,79,79,161,76,76,157,75,75,152,72,72,147,70,70,143,67,67,138,65,65,
      133,63,63,128,60,60,121,56,56,116,54,54,112,50,50,105,57,57,115,76,76,156,80,80,163,
      73,73,150,67,67,140,63,63,130,58,58,118,54,54,109,47,47,97,41,41,84,37,37,74,33,
      33,66,28,28,56,24,24,48,18,18,41,13,13,30,12,12,25,3,3,5,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,6,6,10,
      10,10,15,15,15,26,26,26,34,34,34,44,41,41,64,63,63,75,72,72,102,93,93,112,102,
      102,134,122,122,131,119,119,147,134,134,148,135,135,149,135,135,153,135,135,155,135,135,146,129,129,
      151,135,135,145,129,129,126,112,112,126,115,115,127,118,118,123,114,114,135,123,123,146,136,136,159,
      151,151,161,150,150,158,145,145,152,139,139,146,132,132,138,126,126,118,111,111,100,90,90,77,69,
      69,60,53,53,50,46,46,55,49,49,52,46,46,75,65,65,101,88,88,116,103,103,122,108,108,
      124,112,112,128,116,116,119,109,109,103,95,95,94,87,87,99,88,88,100,86,86,87,73,73,69,
      61,61,69,64,64,84,73,73,96,81,81,104,92,92,124,112,112,138,123,123,134,118,118,112,98,
      98,112,97,97,99,86,84,108,95,94,113,99,98,96,85,85,86,77,77,100,91,91,99,91,91,
      66,59,59,49,43,43,33,29,29,36,31,31,55,48,48,69,62,62,94,84,84,108,98,98,102,
      92,92,79,70,70,54,46,46,57,49,46,74,66,64,106,93,93,136,122,122,152,133,133,155,135,
      135,155,135,135,157,137,137,166,142,142,179,153,153,180,153,153,138,121,121,95,85,85,51,44,44,
      37,33,33,32,30,30,37,35,34,34,30,31,17,16,25,10,10,21,10,10,21,37,37,76,75,
      75,151,89,89,179,91,91,182,86,86,175,84,84,170,80,80,165,78,78,160,76,76,155,74,74,
      151,71,71,146,69,69,142,65,65,135,64,64,129,61,61,125,59,59,121,56,56,115,54,54,111,
      48,48,101,60,60,122,79,79,162,79,79,163,74,74,151,70,70,142,65,65,131,61,61,122,53,
      53,111,46,46,97,41,41,82,33,33,67,27,27,55,23,23,47,20,20,42,15,15,32,11,11,
      23,12,12,25,3,3,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,2,2,13,11,11,32,27,27,40,
      37,37,46,41,41,79,69,69,105,91,91,119,104,104,133,116,116,152,137,137,167,152,152,160,147,
      147,156,141,141,154,140,140,156,140,140,154,139,139,158,143,143,165,145,145,150,131,131,148,132,132,
      152,135,135,142,124,124,142,123,123,158,141,141,169,149,149,169,146,146,180,157,157,188,169,169,174,
      158,158,160,146,146,151,137,137,141,131,131,140,128,128,111,100,100,91,82,82,85,75,75,74,62,
      62,71,60,60,65,58,58,75,64,64,90,76,76,115,98,98,121,105,105,127,113,113,118,108,108,
      134,124,124,133,122,122,129,116,116,133,117,117,114,101,101,96,85,85,97,86,86,74,66,66,54,
      50,50,69,63,63,105,92,92,119,102,102,111,97,97,115,104,104,123,109,109,150,129,129,135,117,
      117,118,104,104,106,96,96,95,87,87,97,90,90,112,100,100,126,111,111,121,106,106,121,107,107,
      109,97,97,76,69,69,53,50,50,31,29,29,27,25,25,42,37,37,73,66,66,98,87,87,114,
      103,103,124,113,113,100,89,89,59,54,54,35,32,32,53,49,49,86,78,78,133,119,119,152,135,
      135,152,136,136,157,140,140,156,139,139,163,144,144,160,141,141,152,133,133,156,136,136,149,131,131,
      129,112,112,73,64,64,33,28,28,30,28,28,39,35,35,43,37,36,36,30,31,17,16,25,10,
      10,23,12,12,24,11,11,21,16,16,32,39,39,82,65,65,131,83,83,169,87,87,176,81,81,
      167,79,79,161,77,77,157,75,75,152,72,72,148,70,70,143,68,68,139,65,65,134,63,63,129,
      60,60,124,58,58,119,55,55,112,52,52,108,48,48,98,62,62,126,79,79,160,72,72,148,65,
      65,131,61,61,122,55,55,110,47,47,96,41,41,86,36,36,72,28,28,58,19,19,40,11,11,
      26,11,11,23,12,12,23,12,12,23,12,12,24,12,12,25,3,3,5,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,2,0,0,31,25,25,54,46,46,62,54,54,55,47,47,87,73,73,114,
      96,96,115,96,96,130,111,111,154,134,134,166,146,146,174,155,155,186,164,164,186,162,162,188,162,
      162,198,166,166,189,160,160,175,150,150,166,148,148,160,142,142,167,147,147,163,142,142,157,138,138,
      158,140,140,170,152,152,167,145,145,181,161,161,198,178,178,212,182,182,216,184,184,208,179,179,195,
      169,169,172,151,151,137,123,123,126,114,114,114,99,99,98,85,85,84,75,75,73,67,67,71,62,
      62,60,53,53,82,72,72,88,78,78,93,84,84,115,104,104,125,113,113,142,124,124,160,137,137,
      176,147,147,172,147,147,147,127,127,126,113,113,108,99,99,87,79,79,81,74,74,76,68,68,82,
      72,72,92,82,82,92,82,82,102,90,90,121,107,107,143,126,126,167,146,146,161,141,141,141,126,
      126,118,107,107,94,85,85,80,72,72,83,73,73,102,91,91,116,105,105,128,115,115,120,110,110,
      124,113,113,112,101,101,74,68,68,44,40,40,25,25,25,23,24,24,31,29,29,70,61,61,99,
      87,87,107,93,93,132,119,119,112,101,101,75,68,68,34,32,32,37,33,33,74,66,66,111,99,
      99,136,122,122,148,132,132,147,134,134,147,133,133,144,131,131,149,132,132,151,136,136,156,141,141,
      154,140,140,148,133,133,131,117,117,76,69,69,36,35,35,36,33,33,39,34,34,35,32,32,39,
      34,32,37,30,31,19,16,26,10,10,23,12,12,24,12,12,24,12,12,24,11,11,23,18,18,
      38,35,35,74,54,54,109,75,75,150,83,83,167,79,79,161,76,76,155,74,74,150,72,72,146,
      69,69,141,67,67,137,65,65,132,61,61,127,59,59,121,56,56,117,55,55,112,51,51,106,48,
      48,99,54,54,108,56,56,114,48,48,98,41,41,86,37,37,77,33,33,66,25,25,53,21,21,
      45,20,20,39,16,16,35,16,16,31,12,12,28,12,12,24,12,12,24,12,12,24,12,12,24,
      12,12,25,3,3,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,11,11,
      19,16,16,22,20,20,58,53,53,66,59,59,94,83,83,120,104,104,131,113,113,164,142,142,187,
      161,161,190,165,165,177,153,153,202,175,175,206,178,178,205,178,178,205,180,180,195,175,175,179,160,
      160,168,147,147,167,147,147,158,139,139,159,141,141,163,144,144,166,148,148,167,148,148,169,154,154,
      171,154,154,183,161,161,196,171,171,206,180,180,209,181,181,207,181,181,209,181,181,190,168,168,184,
      164,164,152,133,133,117,105,105,115,102,102,107,94,94,93,81,81,68,59,59,60,53,53,46,40,
      40,69,60,60,93,85,85,116,103,103,130,112,112,148,126,126,170,146,146,171,147,147,167,147,147,
      159,143,143,140,128,128,129,116,116,109,95,95,100,88,88,98,87,87,82,73,73,77,68,68,79,
      70,70,78,68,68,97,85,85,121,104,104,151,127,127,159,136,136,146,125,125,147,128,128,161,140,
      140,146,127,127,129,114,114,123,107,107,104,89,89,92,82,82,99,92,92,112,102,102,126,115,115,
      126,115,115,122,109,109,100,87,87,73,62,62,52,44,44,25,22,22,24,24,24,30,26,26,62,
      56,56,106,92,92,155,129,129,167,142,142,121,103,103,78,66,66,53,47,47,36,32,32,73,63,
      63,107,93,93,140,118,118,166,142,142,181,154,154,168,143,143,158,136,136,157,139,139,152,135,135,
      149,134,134,163,142,142,164,143,143,169,147,147,161,139,139,107,93,93,61,54,54,35,31,31,31,
      29,29,38,33,33,41,37,36,36,31,32,22,20,27,16,15,24,10,11,24,12,12,24,12,12,
      24,12,12,24,12,12,24,11,11,23,11,11,21,21,21,44,37,37,75,48,48,96,67,67,134,
      77,77,159,76,76,155,72,72,148,71,71,144,68,68,139,66,66,135,64,64,130,61,61,124,57,
      57,119,55,55,115,53,53,109,51,51,103,47,47,98,44,44,90,39,39,82,39,39,78,35,35,
      72,30,30,66,29,29,59,27,27,54,22,22,47,20,20,41,17,17,37,16,16,32,12,12,26,
      12,12,24,12,12,24,12,12,24,12,12,24,12,12,25,3,3,5,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,4,3,3,14,12,12,36,33,33,60,52,52,
      93,81,81,135,117,117,154,135,135,160,141,141,160,142,142,166,145,145,187,165,165,188,165,165,192,
      164,164,206,176,176,215,187,187,216,189,189,204,177,177,193,172,172,174,157,157,156,141,141,152,138,
      138,155,142,142,175,160,160,196,172,172,197,173,173,187,165,165,195,172,172,201,177,177,198,177,177,
      186,169,169,183,167,167,189,173,173,192,174,174,204,177,177,209,180,180,206,177,177,184,159,159,154,
      133,133,134,118,118,116,102,102,93,82,82,96,84,84,66,61,61,40,39,39,56,52,52,90,80,
      80,109,96,96,134,120,120,149,134,134,149,132,132,163,144,144,171,153,153,160,143,143,148,132,132,
      142,128,128,138,124,124,147,126,126,134,115,115,116,102,102,97,88,88,95,84,84,88,77,77,72,
      65,65,82,75,75,92,82,82,123,110,110,136,123,123,137,123,123,130,117,117,142,124,124,165,139,
      139,172,145,145,154,132,132,133,114,114,126,110,110,116,101,101,96,86,86,115,103,103,124,108,108,
      132,119,119,123,112,112,98,89,89,87,81,81,65,58,58,53,48,48,33,27,27,29,21,21,31,
      25,25,58,52,52,90,80,80,149,130,130,159,140,140,146,125,125,123,103,103,91,76,76,50,42,
      42,29,25,25,75,68,68,134,118,118,171,151,151,193,169,169,200,173,173,191,165,165,187,160,160,
      185,159,159,188,160,160,183,159,159,173,151,151,167,147,147,167,149,149,162,146,146,116,100,100,70,
      59,59,46,39,39,30,27,27,33,29,29,41,34,34,41,33,33,39,34,34,19,18,27,9,9,
      23,10,11,24,11,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,23,
      12,12,24,24,24,50,36,36,75,41,41,82,55,55,115,74,74,151,73,73,148,69,69,142,67,
      67,136,65,65,132,62,62,127,60,60,122,56,56,117,55,55,113,51,51,107,49,49,101,46,46,
      96,44,44,90,41,41,84,38,38,78,36,36,72,32,32,65,27,27,58,25,25,51,22,22,46,
      18,18,40,16,16,36,14,14,30,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,
      12,25,3,3,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,
      1,2,2,2,5,4,4,22,18,18,53,46,46,65,57,57,114,99,99,137,119,119,158,136,136,
      175,152,152,177,154,154,188,164,164,202,174,174,208,180,180,221,187,187,217,185,185,213,182,182,217,
      187,187,214,186,186,211,186,186,207,182,182,194,172,172,189,169,169,196,173,173,190,165,165,174,151,
      151,166,149,149,162,150,150,165,151,151,181,164,164,182,167,167,171,161,161,183,165,165,190,170,170,
      187,169,169,182,164,164,181,165,165,182,163,163,186,167,167,194,173,173,185,162,162,153,132,132,127,
      111,111,99,87,87,91,80,80,86,75,75,81,68,68,70,61,61,56,51,51,79,75,75,93,86,
      86,121,108,108,146,132,132,182,163,163,189,166,166,189,162,162,188,163,163,188,164,164,177,158,158,
      158,144,144,144,130,130,130,118,118,115,104,104,94,85,85,86,78,78,97,86,86,102,90,90,94,
      82,82,86,77,77,106,92,92,142,121,121,163,142,142,177,157,157,174,154,154,174,154,154,172,155,
      155,163,144,144,125,113,113,96,88,88,99,88,88,112,96,96,117,100,100,130,112,112,140,122,122,
      147,131,131,151,136,136,141,128,128,110,100,100,75,69,69,55,49,49,36,32,32,24,22,22,28,
      25,25,39,36,36,50,43,43,84,73,73,131,118,118,159,143,143,164,148,148,134,119,119,79,70,
      70,54,47,47,38,34,34,54,46,46,110,93,93,149,131,131,166,151,151,184,166,166,177,161,161,
      170,156,156,171,156,156,169,148,148,164,143,143,165,144,144,178,156,156,187,161,161,184,156,156,186,
      160,160,147,128,128,83,73,73,43,37,37,32,26,26,38,34,34,38,34,34,34,30,30,42,35,
      35,42,36,36,20,19,29,16,15,28,22,19,27,20,19,28,20,18,24,11,11,24,12,12,24,
      12,12,24,12,12,24,12,12,24,12,12,24,11,11,22,11,11,25,25,25,54,35,35,71,36,
      36,72,46,46,95,67,67,137,69,69,141,66,66,136,64,64,131,61,61,124,58,58,118,56,56,
      114,53,53,110,51,51,104,48,48,99,46,46,93,42,42,87,40,40,81,37,37,73,33,33,69,
      31,31,63,27,27,56,22,22,49,20,20,43,18,18,37,16,16,34,13,13,28,12,12,24,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,26,3,3,5,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,3,3,11,10,10,16,14,14,29,25,
      25,48,42,42,71,62,62,96,86,86,122,106,106,141,121,121,181,154,154,197,170,170,189,167,167,
      187,166,166,198,177,177,207,183,183,214,185,185,230,195,195,226,193,193,213,187,187,202,179,179,190,
      171,171,174,154,154,164,143,143,173,151,151,190,163,163,189,163,163,193,166,166,195,167,167,184,162,
      162,181,161,161,190,166,166,198,169,169,210,178,178,220,190,190,230,197,197,233,199,199,230,195,195,
      222,190,190,207,179,179,195,173,173,183,164,164,162,146,146,144,132,132,110,102,102,89,81,81,65,
      60,60,58,53,53,75,65,65,88,74,74,84,72,72,76,67,67,91,78,78,104,89,89,146,125,
      125,149,132,132,144,132,132,144,135,135,130,119,119,143,131,131,153,144,144,170,159,159,169,154,154,
      165,144,144,155,133,133,148,127,127,138,120,120,118,103,103,99,88,88,79,71,71,80,72,72,91,
      81,81,93,83,83,125,111,111,150,131,131,169,145,145,177,152,152,176,153,153,162,144,144,166,147,
      147,173,152,152,162,141,141,141,127,127,126,114,114,118,105,105,119,104,104,103,92,92,105,92,92,
      129,112,112,169,146,146,188,164,164,147,132,132,113,102,102,76,69,69,49,43,43,41,37,37,29,
      26,26,25,22,22,35,31,31,45,39,39,81,69,69,117,102,102,149,130,130,164,145,145,119,109,
      109,72,66,66,54,49,49,38,35,35,37,35,35,64,59,59,109,98,98,151,134,134,170,149,149,
      175,152,152,164,143,143,168,146,146,168,148,148,164,145,145,168,148,148,169,147,147,170,148,148,170,
      148,148,180,159,159,185,165,165,176,153,153,120,103,103,60,50,50,40,35,35,34,30,30,39,35,
      35,39,34,34,38,34,34,35,31,31,40,34,34,46,38,38,40,33,35,40,33,35,41,34,35,
      37,34,35,38,34,33,21,19,27,10,10,23,12,12,24,12,12,24,12,12,24,12,12,24,12,
      12,24,11,11,23,12,12,24,27,27,56,33,33,70,31,31,65,37,37,78,54,54,109,64,64,
      132,63,63,129,60,60,122,57,57,117,56,56,112,52,52,108,48,48,100,46,46,96,44,44,91,
      42,42,86,37,37,79,36,36,72,32,32,68,28,28,60,27,27,54,22,22,46,20,20,42,18,
      18,36,16,16,32,12,12,26,11,11,22,11,11,23,11,11,23,12,12,24,12,12,25,11,11,
      23,3,3,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,2,30,25,25,75,62,
      62,107,90,90,152,131,131,181,159,159,194,168,168,199,166,166,205,174,174,211,181,181,218,186,186,
      213,185,185,200,177,177,184,165,165,168,154,154,162,149,149,163,151,151,169,157,157,182,164,164,198,
      176,176,203,173,173,205,176,176,197,173,173,188,168,168,171,156,156,163,146,146,181,156,156,188,159,
      159,196,171,171,206,183,183,218,188,188,224,193,193,223,192,192,222,192,192,227,194,194,222,186,186,
      219,184,184,221,187,187,199,168,168,188,158,158,169,144,144,150,128,128,128,111,111,85,71,71,68,
      58,58,63,55,55,61,54,54,68,63,63,72,65,65,74,67,67,105,91,91,137,119,119,170,145,
      145,185,158,158,194,167,167,190,164,164,186,160,160,176,153,153,163,146,146,143,127,127,120,111,111,
      107,99,99,105,98,98,116,107,107,119,110,110,105,95,95,94,81,81,91,77,77,102,86,86,115,
      97,97,114,98,98,98,86,86,113,102,102,160,143,143,198,169,169,208,180,180,196,170,170,184,159,
      159,164,143,143,147,131,131,131,119,119,125,114,114,123,111,111,121,106,106,127,111,111,123,111,111,
      126,114,114,154,133,133,187,161,161,185,158,158,152,129,129,109,94,94,82,71,71,51,42,42,40,
      35,35,29,28,28,22,19,19,30,26,26,53,48,48,70,63,63,120,106,106,159,137,137,178,150,
      150,169,144,144,127,110,110,64,58,58,33,31,31,30,28,28,42,40,40,85,78,78,137,127,127,
      166,155,155,169,158,158,168,158,158,164,151,151,167,148,148,168,145,145,172,147,147,177,151,151,176,
      150,150,175,148,148,183,154,154,192,161,161,182,157,157,159,141,141,117,106,106,58,53,53,43,36,
      36,43,35,35,42,35,35,38,34,34,42,38,38,41,36,36,40,37,37,39,35,35,40,34,34,
      41,33,33,41,36,36,38,34,33,38,33,33,39,34,34,41,36,35,23,21,27,10,10,23,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,11,11,22,15,15,29,28,28,
      55,32,32,68,31,31,62,31,31,64,44,44,91,59,59,120,58,58,121,57,57,115,53,53,109,
      51,51,104,48,48,99,46,46,94,42,42,87,39,39,81,36,36,74,32,32,66,30,30,61,25,
      25,55,22,22,47,20,20,42,19,19,40,18,18,37,16,16,33,16,16,33,15,15,30,14,14,
      29,14,14,27,15,15,29,15,15,29,4,4,8,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,7,5,5,114,97,97,200,169,169,207,174,174,216,181,181,218,185,185,225,189,189,
      226,190,190,228,191,191,238,201,201,237,199,199,218,184,184,186,162,162,158,141,141,145,132,132,139,
      128,128,141,131,131,144,135,135,148,136,136,155,145,145,189,165,165,211,180,180,200,175,175,183,162,
      162,172,154,154,169,150,150,182,158,158,191,165,165,192,168,168,193,172,172,203,181,181,209,186,186,
      225,194,194,210,181,181,174,152,152,143,124,124,121,102,102,112,95,95,74,63,63,61,49,49,64,
      52,52,73,60,60,86,73,73,92,79,79,91,83,83,101,92,92,109,98,98,134,121,121,169,151,
      151,194,171,171,198,172,172,197,171,171,198,169,169,191,164,164,174,149,149,152,133,133,130,116,116,
      111,100,100,111,100,100,109,98,98,101,92,92,93,84,84,87,79,79,78,72,72,84,78,78,92,
      83,83,97,84,84,108,94,94,135,118,118,174,152,152,200,174,174,215,186,186,204,180,180,187,166,
      166,170,148,148,161,138,138,158,134,134,158,135,135,146,127,127,122,106,106,110,97,97,111,100,100,
      121,110,110,141,124,124,171,149,149,179,151,151,184,159,159,155,135,135,105,93,93,78,68,68,50,
      41,41,36,29,29,38,33,33,32,27,27,25,23,23,43,41,41,74,65,65,101,92,92,162,150,
      150,182,166,166,164,148,148,155,139,139,112,97,97,47,40,40,24,20,20,37,31,31,67,61,61,
      115,101,101,150,133,133,189,168,168,182,160,160,178,156,156,178,159,159,174,154,154,175,153,153,179,
      155,155,185,162,162,188,164,164,195,170,170,189,166,166,180,157,157,189,166,166,170,147,147,123,108,
      108,75,66,66,30,28,28,32,31,31,45,38,38,49,40,40,41,35,35,36,33,33,43,36,36,
      44,38,38,45,40,40,44,39,39,47,40,40,45,38,38,40,36,36,38,34,34,38,31,31,43,
      37,37,47,40,39,24,23,29,10,10,23,12,12,24,12,12,24,12,12,24,12,12,24,12,12,
      24,12,12,24,12,12,24,12,12,23,15,15,29,25,25,53,31,31,63,27,27,59,27,27,58,
      37,37,75,51,51,107,57,57,116,52,52,109,48,48,101,47,47,96,43,43,91,41,41,85,38,
      38,80,38,38,78,38,38,78,34,34,72,34,34,69,30,30,62,26,26,56,29,29,60,29,29,
      58,26,26,53,31,31,62,28,28,57,24,24,51,23,23,46,18,18,37,14,14,28,4,4,7,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,76,65,65,184,156,156,
      192,162,162,205,174,174,200,168,168,202,172,172,209,178,178,228,192,192,235,199,199,231,195,195,224,
      190,190,199,175,175,160,141,141,138,121,121,142,126,126,143,130,130,139,128,128,130,120,120,132,124,
      124,201,173,173,221,188,188,192,166,166,175,153,153,177,154,154,188,163,163,203,178,178,220,190,190,
      224,189,189,221,191,191,219,192,192,209,187,187,203,184,184,172,149,149,82,72,72,62,53,53,70,
      58,58,65,54,54,65,53,53,64,54,54,68,58,58,101,90,90,130,119,119,150,142,142,166,158,
      158,183,171,171,189,174,174,179,162,162,158,141,141,151,136,136,148,133,133,144,125,125,133,115,115,
      130,111,111,131,113,113,122,105,105,115,100,100,94,83,83,78,70,70,78,70,70,68,61,61,72,
      64,64,97,86,86,127,109,109,139,121,121,147,130,130,163,141,141,184,163,163,182,164,164,174,158,
      158,163,150,150,140,129,129,131,117,117,142,128,128,145,129,129,137,122,122,127,110,110,122,104,104,
      133,114,114,147,127,127,142,125,125,142,131,131,154,141,141,157,139,139,141,127,127,98,86,86,74,
      62,62,37,30,30,28,24,24,37,32,32,47,39,39,37,30,30,48,42,42,80,71,71,98,87,
      87,137,125,125,195,181,181,181,170,170,141,133,133,95,87,87,61,55,55,33,31,31,22,19,19,
      52,46,46,84,78,78,120,110,110,173,155,155,186,165,165,178,160,160,172,152,152,174,154,154,172,
      156,156,171,155,155,173,153,153,172,151,151,178,153,153,187,160,160,195,167,167,201,173,173,195,169,
      169,187,162,162,144,127,127,88,81,81,38,32,32,29,24,24,32,31,31,35,34,34,45,39,39,
      48,40,40,40,35,35,37,35,35,43,37,37,46,39,39,44,37,37,43,35,35,45,38,38,43,
      38,38,37,34,34,32,30,30,38,33,33,41,37,36,36,30,31,18,18,27,11,11,23,12,12,
      24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,23,
      14,14,29,25,25,52,29,29,63,28,28,57,27,27,52,32,32,67,43,43,92,51,51,105,49,
      49,102,46,46,94,42,42,87,40,40,83,53,53,107,58,58,122,59,59,122,57,57,116,55,55,
      114,51,51,108,47,47,97,47,47,96,41,41,84,37,37,75,31,31,65,28,28,56,24,24,51,
      22,22,45,17,17,37,13,13,27,4,4,7,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,57,49,49,176,149,149,187,158,158,198,169,169,190,163,163,192,165,165,201,
      172,172,219,184,184,229,193,193,229,192,192,225,191,191,207,181,181,192,167,167,152,132,132,139,121,
      121,147,127,127,136,120,120,121,107,107,104,94,94,184,156,156,222,186,186,196,166,166,187,161,161,
      188,165,165,201,176,176,224,195,195,233,200,200,229,194,194,234,197,197,226,193,193,206,178,178,207,
      181,181,187,162,162,104,90,90,69,58,58,74,62,62,70,58,58,68,58,58,69,61,61,111,101,
      101,163,154,154,171,166,166,164,161,161,166,159,159,171,159,159,181,166,166,190,173,173,167,149,149,
      129,115,115,118,107,107,111,98,98,101,87,87,99,84,84,99,84,84,102,88,88,99,84,84,85,
      75,75,71,65,65,59,54,54,63,58,58,121,109,109,184,158,158,226,191,191,226,191,191,210,175,
      175,198,165,165,175,150,150,154,135,135,130,115,115,134,121,121,128,115,115,105,93,93,90,80,80,
      99,86,86,107,92,92,101,88,88,112,98,98,166,146,146,178,158,158,176,160,160,133,124,124,81,
      74,74,48,44,44,34,31,31,33,30,30,41,32,32,40,32,32,35,28,28,39,34,34,48,40,
      40,79,67,67,140,124,124,191,169,169,224,198,198,179,162,162,114,108,108,79,75,75,43,42,42,
      23,24,24,33,29,29,74,64,64,117,103,102,153,136,135,187,172,172,185,171,171,170,159,159,160,
      149,149,156,149,149,151,143,143,156,145,145,161,148,148,162,147,147,165,147,147,161,144,144,164,142,
      141,161,135,134,154,136,136,158,142,142,130,116,116,89,78,78,43,36,36,30,27,27,37,32,32,
      32,29,29,31,30,30,36,34,34,43,39,39,45,40,40,45,39,39,41,36,36,41,34,34,37,
      34,34,38,36,36,41,36,36,44,38,38,43,38,38,35,32,32,35,31,31,42,38,38,35,32,
      33,15,14,23,10,11,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,
      12,12,24,12,12,24,12,12,24,12,12,24,12,12,23,14,14,30,23,23,48,28,28,57,26,
      26,54,23,23,48,25,25,52,35,35,73,44,44,90,44,44,92,41,41,84,41,41,84,63,63,
      130,73,73,150,66,66,134,61,61,125,57,57,115,51,51,105,46,46,98,43,43,90,38,38,80,
      34,34,71,29,29,61,27,27,56,23,23,51,19,19,42,15,15,33,12,12,28,4,4,9,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,38,32,32,175,149,149,193,
      163,163,197,167,167,185,159,159,189,162,162,197,169,169,206,177,177,215,182,182,221,186,186,220,187,
      187,207,180,180,208,180,180,179,153,153,135,117,117,137,117,117,140,119,119,131,113,113,111,98,98,
      172,147,147,206,174,174,192,164,164,189,166,166,194,173,173,205,182,182,219,190,190,215,186,186,208,
      177,177,214,180,180,217,183,183,206,171,171,205,174,174,199,171,171,118,101,101,65,54,54,72,60,
      60,65,55,55,62,56,56,92,84,84,162,154,154,174,167,167,177,167,167,167,162,162,159,153,153,
      163,153,153,171,157,157,181,165,165,188,167,167,145,128,128,99,88,88,97,85,85,93,80,80,82,
      70,70,82,69,69,92,78,78,88,74,74,82,70,70,71,64,64,73,66,66,156,139,139,217,193,
      193,235,203,203,238,201,201,233,197,197,228,191,191,214,179,179,197,170,170,182,156,156,149,127,127,
      113,99,99,96,82,82,97,85,85,80,69,69,68,59,59,76,65,65,81,70,70,121,104,104,194,
      170,170,171,151,151,162,148,148,119,110,110,38,34,34,30,28,28,39,35,35,39,34,34,43,36,
      36,42,36,36,36,32,32,34,31,31,45,38,38,159,138,138,211,185,185,217,192,192,241,209,209,
      149,127,127,27,24,24,31,29,29,31,30,30,25,24,24,64,62,62,209,195,196,235,209,212,242,
      236,239,223,202,204,203,178,177,193,175,175,187,169,169,189,171,171,193,173,173,196,172,172,195,171,
      171,187,166,166,192,169,168,202,173,172,206,179,182,208,192,204,205,179,178,139,127,127,49,47,47,
      23,21,21,34,29,29,42,36,36,39,33,33,31,27,27,31,30,30,36,34,34,42,36,36,42,
      37,37,40,36,36,38,33,33,37,32,32,37,32,32,39,34,34,40,35,35,45,38,38,41,36,
      36,37,34,34,38,35,34,35,31,32,16,15,25,10,11,23,12,12,24,12,12,24,12,12,24,
      12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,
      12,24,11,11,22,13,13,25,21,21,44,24,24,54,24,24,49,24,24,46,20,20,44,26,26,
      55,36,36,73,39,39,81,40,40,82,61,61,125,71,71,143,63,63,128,60,60,120,54,54,110,
      51,51,103,46,46,95,42,42,86,37,37,78,33,33,68,29,29,60,28,28,56,23,23,49,19,
      19,42,14,14,32,8,8,17,2,2,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,47,39,39,181,152,152,195,163,163,205,171,171,194,165,165,200,172,172,207,179,
      179,211,183,183,205,176,176,211,180,180,211,181,181,215,186,186,222,193,193,218,186,186,157,135,135,
      132,112,112,123,106,106,116,99,99,107,91,91,161,140,140,191,167,167,183,164,164,192,173,173,204,
      184,184,196,177,177,200,174,174,201,173,173,192,163,163,189,159,159,199,166,166,206,172,172,207,174,
      174,196,170,170,108,93,93,57,49,49,60,53,53,53,48,48,63,58,58,124,114,114,167,158,158,
      162,159,159,175,166,166,169,163,163,159,155,155,161,154,154,164,152,152,178,162,162,196,173,173,190,
      163,163,115,98,98,72,63,63,79,67,67,82,68,68,81,69,69,79,67,67,76,64,64,67,55,
      55,93,80,80,168,149,149,221,196,196,222,196,196,222,194,194,229,196,196,235,198,198,233,197,197,
      230,194,194,212,180,180,199,171,171,202,174,174,157,137,137,76,66,66,75,65,65,84,72,72,64,
      55,55,60,52,52,61,53,53,96,83,83,188,168,168,171,153,153,164,150,150,125,116,116,49,44,
      44,35,32,32,37,33,33,36,33,33,42,36,36,42,37,37,34,33,33,30,27,27,41,37,37,
      151,131,131,209,181,181,213,186,186,232,201,201,138,117,117,30,26,26,41,38,38,39,35,35,24,
      23,23,67,61,61,239,224,226,251,247,249,244,233,236,241,223,231,228,206,209,216,190,189,198,182,
      182,204,184,184,214,190,190,222,195,195,225,197,197,226,197,196,224,210,230,239,208,212,253,243,246,
      244,237,251,253,213,212,139,120,120,15,16,16,30,28,28,40,37,37,41,37,37,39,31,31,30,
      28,28,29,30,30,34,32,32,43,36,36,47,39,39,42,38,38,39,35,35,35,32,32,34,32,
      32,37,32,32,36,32,32,42,35,35,38,32,32,36,33,33,31,28,29,16,15,25,10,10,23,
      12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,10,10,21,16,16,35,25,25,
      54,24,24,48,22,22,46,20,20,43,19,19,39,21,21,43,29,29,60,38,38,78,58,58,120,
      68,68,138,61,61,123,57,57,115,51,51,108,49,49,101,47,47,94,42,42,84,37,37,76,33,
      33,66,28,28,60,25,25,53,23,23,47,19,19,39,15,15,31,4,4,9,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,37,32,32,176,149,149,203,168,
      168,215,181,181,209,178,178,215,184,184,222,191,191,218,187,187,210,179,179,210,180,180,209,181,181,
      218,189,189,232,199,199,227,193,193,158,137,137,118,102,102,117,101,101,114,99,99,98,85,85,145,
      130,130,187,167,167,179,163,163,188,175,175,196,180,180,187,166,166,183,158,158,194,167,167,190,161,
      161,180,150,150,181,151,151,187,157,157,197,167,167,183,157,157,98,85,85,47,40,40,51,47,47,
      48,44,44,99,89,89,180,163,163,173,157,157,160,152,152,160,157,157,174,165,165,170,164,164,168,
      160,160,172,157,157,179,161,161,190,168,168,203,175,175,158,135,135,76,66,66,60,53,53,73,63,
      63,76,64,64,74,61,61,66,56,56,61,52,52,143,121,121,221,192,192,223,196,196,215,190,190,
      218,191,191,226,195,195,235,198,198,234,198,198,235,199,199,221,188,188,209,181,181,211,183,183,212,
      185,185,141,125,125,57,49,49,64,53,53,60,51,51,54,46,46,42,35,35,82,73,73,193,174,
      174,181,162,162,179,162,162,142,128,128,55,49,49,32,29,29,32,28,28,34,30,30,42,38,38,
      41,37,37,32,30,30,30,28,28,40,35,35,142,121,121,205,175,175,208,178,178,218,187,187,127,
      108,108,29,25,25,42,36,36,40,36,36,27,24,24,56,46,45,202,182,185,247,236,240,242,221,
      223,241,223,227,226,197,198,215,188,188,199,180,180,198,181,180,204,185,184,201,183,181,215,190,190,
      219,193,195,231,223,239,236,212,217,248,245,250,243,234,240,250,210,209,142,121,121,19,18,18,33,
      29,29,44,37,37,42,37,37,38,33,33,31,28,28,31,29,29,39,36,36,47,38,38,41,35,
      35,38,36,36,37,33,33,32,31,31,31,32,32,38,33,33,35,32,32,36,31,31,41,33,33,
      35,30,31,15,15,24,10,11,23,13,13,24,13,13,24,12,12,24,12,12,24,12,12,24,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,
      24,12,12,24,12,12,24,12,12,25,20,20,39,22,22,47,20,20,44,20,20,41,20,20,40,
      18,18,37,16,16,34,27,27,56,55,55,112,67,67,134,59,59,119,55,55,112,52,52,105,47,
      47,99,43,43,90,40,40,83,35,35,72,32,32,64,28,28,57,24,24,52,23,23,47,19,19,
      38,15,15,30,5,5,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,30,26,26,177,152,152,214,180,180,217,185,185,151,131,131,122,106,106,133,117,117,
      125,111,111,150,132,132,145,129,129,174,153,153,201,177,177,209,183,183,212,183,183,166,142,142,113,
      97,97,111,97,97,105,93,93,98,83,83,138,124,124,178,165,165,181,167,167,185,173,173,172,159,
      159,166,148,148,168,146,146,184,159,159,192,164,164,186,157,157,176,150,150,180,155,155,180,155,155,
      167,142,142,92,80,80,43,37,37,50,43,43,50,46,46,120,107,107,206,182,182,188,168,168,166,
      152,152,157,152,152,173,166,166,172,167,167,174,164,164,179,165,165,182,165,165,193,172,172,201,176,
      176,204,175,175,104,90,90,47,41,41,58,50,50,72,61,61,72,60,60,55,46,46,99,86,86,
      198,171,171,218,188,188,220,193,193,221,194,194,215,189,189,218,189,189,225,191,191,226,190,190,223,
      186,186,223,189,189,223,194,194,224,194,194,221,193,193,195,169,169,95,81,81,41,34,34,50,42,
      42,49,41,41,36,29,29,68,61,61,200,179,179,199,177,177,201,182,182,166,151,151,63,58,58,
      27,24,24,31,28,28,34,32,32,41,36,36,39,36,36,34,34,34,32,30,30,41,34,34,141,
      122,122,198,169,169,207,179,179,215,183,183,122,101,101,27,21,21,43,38,38,44,37,37,26,26,
      26,55,47,46,197,164,164,246,226,229,242,225,227,243,228,230,238,214,216,223,195,199,214,191,193,
      213,192,197,205,186,188,199,183,188,215,189,185,211,200,222,231,211,225,245,231,232,237,211,214,237,
      202,203,238,204,203,124,109,109,25,22,22,34,30,30,44,37,37,42,37,37,38,34,34,33,30,
      30,39,33,33,47,38,38,49,41,41,44,36,36,39,33,33,33,28,28,30,26,26,31,27,27,
      33,28,28,31,30,30,35,30,30,42,35,35,35,34,34,28,26,28,29,25,26,32,29,30,35,
      30,30,21,17,25,10,11,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,
      24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,23,12,12,24,
      17,17,36,21,21,42,20,20,39,16,16,36,16,16,35,15,15,30,15,15,33,43,43,88,61,
      61,123,58,58,119,53,53,112,52,52,105,47,47,96,43,43,87,39,39,79,34,34,69,30,30,
      62,29,29,58,25,25,52,21,21,45,17,17,36,12,12,26,4,4,9,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,33,28,28,179,156,156,205,181,181,
      218,189,189,114,100,100,56,50,50,60,56,56,58,54,54,52,49,49,40,38,38,76,70,70,165,
      149,149,197,174,174,195,168,168,180,155,155,108,94,94,90,78,78,83,73,73,80,68,68,135,122,
      122,184,167,167,194,177,177,180,167,167,99,90,90,99,86,86,126,111,111,127,112,112,134,116,116,
      129,113,113,119,105,105,131,115,115,115,102,102,110,96,96,78,69,69,45,39,39,50,45,45,52,
      47,47,171,146,146,229,196,196,206,181,181,179,165,165,174,168,168,165,158,158,146,139,139,151,143,
      143,170,157,157,182,165,165,191,171,171,203,180,180,210,184,184,140,123,123,56,49,49,55,47,47,
      65,57,57,56,48,48,48,42,42,148,128,128,229,197,197,215,185,185,215,188,188,219,192,192,210,
      184,184,211,183,183,223,192,192,222,187,187,218,181,181,226,190,190,233,197,197,223,190,190,207,178,
      178,210,182,182,144,127,127,48,41,41,47,40,40,45,37,37,32,26,26,67,58,58,210,187,187,
      206,183,183,213,186,186,179,156,156,65,59,59,27,23,23,31,27,27,34,30,30,41,37,37,41,
      37,37,40,36,36,36,32,32,38,34,34,144,125,125,198,169,169,207,179,179,216,185,185,118,101,
      101,25,21,21,46,38,38,45,41,41,30,28,28,56,48,49,199,166,166,242,218,221,241,225,230,
      242,227,231,239,212,214,228,212,227,222,200,209,214,201,222,212,197,216,208,200,230,214,193,195,217,
      193,198,236,220,226,239,219,223,236,220,228,236,197,196,237,203,202,123,108,108,25,22,22,34,32,
      32,44,37,37,45,38,38,41,36,36,41,36,36,43,37,37,41,37,37,40,35,35,38,33,33,
      39,34,34,39,33,33,36,30,30,32,26,26,30,26,26,29,26,26,35,29,29,46,39,39,45,
      39,38,42,35,35,39,35,34,37,32,32,37,35,34,21,21,28,10,10,23,12,12,24,12,12,
      24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,
      12,12,24,12,12,24,12,12,24,12,12,24,12,12,23,17,17,34,17,17,38,16,16,35,16,
      16,32,14,14,29,12,12,29,34,34,70,45,45,95,50,50,104,50,50,101,48,48,100,43,43,
      89,42,42,83,36,36,73,31,31,63,26,26,55,25,25,49,21,21,43,18,18,39,15,15,32,
      6,6,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,20,18,18,144,131,131,202,183,183,217,194,194,116,103,103,53,49,49,63,57,57,59,
      55,55,54,51,51,49,45,45,38,36,36,117,107,107,187,166,166,183,161,161,179,154,154,118,102,
      102,87,76,76,75,68,68,66,61,61,120,106,106,191,167,167,203,179,179,175,158,158,64,55,55,
      57,47,47,61,51,51,57,48,48,55,46,46,49,42,42,41,36,36,40,38,38,41,41,41,39,
      38,38,41,38,38,50,46,46,54,47,47,54,48,48,193,164,164,240,202,202,225,193,193,197,177,
      177,156,150,150,100,94,94,70,61,61,81,74,74,110,100,100,156,144,144,192,173,173,201,178,178,
      211,187,187,174,155,155,69,62,62,46,40,40,55,48,48,48,40,40,78,66,66,184,156,156,221,
      186,186,215,183,183,212,183,183,215,188,188,210,184,184,184,159,159,181,157,157,196,168,168,213,181,
      181,236,198,198,234,197,197,215,179,179,209,174,174,212,178,178,189,162,162,82,71,71,34,28,28,
      44,36,36,34,29,29,67,57,57,207,184,184,210,184,184,219,186,186,175,149,149,63,54,54,30,
      25,25,33,29,29,33,33,33,41,37,37,43,39,39,41,37,37,36,31,31,37,32,32,150,129,
      129,203,175,175,207,178,178,215,183,183,120,101,101,24,20,20,45,37,37,45,39,39,33,30,30,
      58,52,52,201,172,172,237,198,198,238,215,218,234,211,214,225,191,191,220,193,193,212,189,189,216,
      189,187,217,199,209,214,199,213,218,202,214,222,210,227,233,202,202,243,237,244,235,206,209,237,208,
      213,239,204,203,126,114,114,26,23,23,34,31,31,40,34,34,45,38,38,44,36,36,41,37,37,
      38,35,35,32,31,31,31,27,27,32,30,30,35,33,33,36,34,34,32,29,29,30,27,27,30,
      29,29,28,26,26,35,31,31,39,36,36,36,33,33,35,33,33,35,32,32,35,30,30,42,36,
      35,24,24,31,10,10,23,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,
      12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,
      12,24,12,12,24,14,14,32,16,16,35,15,15,31,12,12,27,11,11,24,18,18,35,25,25,
      52,25,25,53,26,26,52,28,28,58,27,27,56,25,25,50,21,21,43,19,19,39,13,13,27,
      12,12,22,12,12,23,12,12,23,13,13,27,7,7,14,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,18,18,144,133,133,194,181,181,206,
      188,188,105,96,96,50,46,46,63,60,60,61,57,57,59,53,53,53,50,50,46,43,43,86,79,
      79,159,143,143,178,157,157,173,150,150,111,97,97,65,57,57,56,51,51,44,41,41,97,84,84,
      186,162,162,208,180,180,183,160,160,78,69,69,58,50,50,59,49,49,54,47,47,51,43,43,44,
      39,39,38,36,36,41,38,38,44,44,44,44,43,43,46,42,42,56,48,48,58,50,50,56,50,
      50,192,161,161,240,203,203,241,205,205,190,169,169,81,74,74,44,37,37,50,43,43,48,42,42,
      38,33,33,91,80,80,181,163,163,189,167,167,196,177,177,193,176,176,78,69,69,42,38,38,45,
      39,39,44,36,36,113,96,96,214,182,182,216,181,181,220,184,184,216,186,186,198,172,172,144,126,
      126,85,73,73,62,54,54,83,74,74,129,111,111,195,166,166,239,201,201,224,187,187,219,182,182,
      222,186,186,231,194,194,122,103,103,34,28,28,42,34,34,36,29,29,61,54,54,204,178,178,230,
      197,197,233,196,196,179,151,151,65,56,56,31,29,29,37,34,34,37,36,36,40,35,35,43,37,
      37,43,39,39,39,32,32,38,29,29,133,114,114,200,172,172,195,167,167,207,176,176,123,102,102,
      22,19,19,41,35,35,43,36,36,33,30,30,55,50,50,197,168,168,240,203,203,226,190,189,208,
      180,180,202,184,184,211,189,189,203,185,185,219,194,194,217,191,190,219,191,190,225,194,192,232,212,
      222,234,206,211,237,215,219,235,202,204,239,224,234,249,226,229,138,120,119,30,27,27,37,33,33,
      34,30,30,37,32,32,40,34,34,37,33,33,34,32,32,33,30,30,34,31,31,37,33,33,37,
      32,32,36,30,30,32,30,30,33,30,30,34,31,31,36,32,32,36,34,34,39,34,34,38,32,
      32,40,36,36,37,34,34,36,33,33,44,37,36,26,23,29,9,10,23,12,12,24,12,12,24,
      12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,14,14,29,14,14,
      30,12,12,27,12,12,24,11,11,23,11,11,21,11,11,21,11,11,21,11,11,22,13,13,26,
      11,11,24,11,11,23,11,11,22,12,12,23,12,12,24,12,12,24,12,12,25,11,11,22,4,
      4,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,15,13,13,128,119,119,191,177,177,194,181,181,106,99,99,53,48,48,63,57,57,63,56,
      56,59,51,51,54,50,50,50,47,47,54,50,50,135,125,125,173,159,159,162,145,145,93,82,82,
      40,37,37,47,42,42,44,40,40,83,76,76,176,157,157,209,187,187,189,168,168,95,84,84,58,
      50,50,58,50,50,52,43,43,46,39,39,38,32,32,34,32,32,38,36,36,45,43,43,45,43,
      43,46,42,42,57,49,49,58,50,50,61,53,53,204,171,171,234,199,199,222,194,194,147,131,131,
      45,40,40,45,39,39,45,41,41,47,42,42,40,36,36,54,47,47,141,126,126,189,168,168,179,
      158,158,182,162,162,81,72,72,35,32,32,35,29,29,46,37,37,129,111,111,210,179,179,215,180,
      180,220,185,185,208,180,180,115,98,98,49,41,41,38,31,31,35,30,30,30,27,27,32,27,27,
      71,60,60,171,145,145,233,197,197,230,194,194,230,194,194,245,207,207,169,144,144,54,46,46,39,
      31,31,33,28,28,63,58,58,212,183,183,233,196,196,241,204,204,191,162,162,68,59,59,32,31,
      31,41,38,38,39,34,34,41,36,36,44,36,36,44,39,39,40,36,36,39,32,32,135,116,116,
      198,170,170,190,160,160,196,166,166,116,98,98,22,19,19,37,33,33,39,36,36,32,28,28,50,
      44,44,182,158,158,241,203,203,224,191,191,205,180,180,172,156,156,166,150,150,180,165,165,186,171,
      171,194,178,178,207,189,189,217,197,197,219,195,194,217,194,194,243,207,208,249,226,233,245,215,216,
      255,233,238,133,119,118,24,22,22,35,29,29,33,29,29,33,32,32,38,34,34,42,36,36,39,
      34,34,38,33,33,38,32,32,40,35,35,42,34,34,37,31,31,36,34,34,34,32,32,36,30,
      30,39,34,34,36,32,32,36,32,32,36,32,32,38,35,35,38,37,37,38,35,35,32,28,31,
      16,16,26,11,11,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,
      24,12,12,24,12,12,24,12,12,24,12,12,26,12,12,25,12,12,24,12,12,24,12,12,24,
      12,12,24,12,12,24,12,12,24,12,12,23,12,12,24,12,12,24,12,12,24,12,12,24,12,
      12,24,12,12,24,13,13,27,8,8,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,95,84,84,206,186,186,199,182,
      182,120,109,109,61,56,56,64,57,57,62,53,53,60,52,52,55,48,48,48,45,45,59,54,54,
      134,123,123,167,158,158,155,144,144,99,90,90,39,38,38,45,42,42,45,41,41,84,78,78,176,
      161,161,210,189,189,187,170,170,94,82,82,56,50,50,51,46,46,46,38,38,40,34,34,35,31,
      31,25,22,22,36,35,35,44,42,42,44,43,43,45,42,42,56,49,49,59,51,51,63,54,54,
      196,169,169,220,192,192,205,186,186,134,123,123,42,37,37,42,40,40,43,40,40,43,40,40,44,
      37,37,35,32,32,102,90,90,200,176,176,179,153,153,175,152,152,94,83,83,32,29,29,30,27,
      27,59,50,50,147,125,125,188,158,158,204,171,171,217,184,184,144,125,125,51,41,41,42,35,35,
      40,35,35,36,32,32,38,34,34,39,34,34,31,25,25,66,55,55,216,180,180,240,204,204,237,
      200,200,246,208,208,203,172,172,75,63,63,32,29,29,37,32,32,69,60,60,203,174,174,223,188,
      188,238,201,201,204,173,173,70,61,61,29,27,27,41,36,36,39,33,33,42,37,37,46,38,38,
      44,37,37,42,36,36,38,32,32,130,111,111,195,167,167,183,158,158,178,155,155,97,87,87,23,
      21,21,36,32,32,41,35,35,29,26,26,53,48,48,194,167,167,227,193,193,218,187,187,214,186,
      186,117,103,103,59,56,56,65,63,63,67,65,65,70,70,70,79,78,78,87,84,84,85,83,83,
      87,84,84,105,96,97,107,106,112,105,92,92,108,95,95,68,65,65,29,28,28,32,30,30,33,
      32,32,36,32,32,41,38,38,47,40,40,45,37,37,44,37,37,41,34,34,37,33,33,47,40,
      40,41,36,36,40,36,36,37,34,34,35,32,32,33,32,32,32,30,30,34,31,31,34,30,30,
      34,32,32,34,33,33,38,36,35,26,24,30,9,9,22,12,12,24,12,12,24,12,12,24,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,
      24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,
      12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,24,13,13,25,11,11,22,3,3,6,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,3,2,2,107,93,93,215,191,191,207,186,186,133,121,121,67,60,60,63,55,55,63,55,55,
      62,52,52,56,48,48,49,44,44,72,66,66,141,130,130,156,147,147,153,142,142,105,97,97,39,
      37,37,41,42,42,40,38,38,80,73,73,173,161,161,202,187,187,180,164,164,92,82,82,50,44,
      44,46,38,38,42,36,36,37,33,33,33,31,31,29,26,26,37,37,37,44,44,44,44,44,44,
      44,42,42,51,46,46,45,41,41,48,42,42,174,155,155,209,187,187,206,188,188,153,142,142,56,
      52,52,34,33,33,40,36,36,43,39,39,43,39,39,31,27,27,93,78,78,193,169,169,169,147,
      147,170,148,148,97,84,84,36,31,31,32,27,27,79,66,66,184,156,156,180,151,151,187,159,159,
      202,175,175,77,68,68,37,33,33,44,40,40,38,34,34,36,32,32,40,35,35,42,36,36,42,
      35,35,37,31,31,128,108,108,241,203,203,232,196,196,183,154,154,103,88,88,48,40,40,38,34,
      34,41,35,35,70,57,57,194,164,164,225,189,189,233,196,196,198,168,168,67,58,58,27,25,25,
      39,36,36,41,35,35,44,38,38,45,39,39,44,36,36,41,35,35,36,30,30,126,107,107,188,
      163,163,175,155,155,169,152,152,98,88,88,22,21,21,35,32,32,40,35,35,30,27,27,53,47,
      47,189,165,165,218,189,189,201,175,175,210,184,184,85,75,75,20,22,22,27,28,28,28,29,29,
      23,24,24,24,24,24,21,22,22,19,20,20,23,24,24,20,23,22,25,22,21,29,27,27,25,
      24,24,33,31,31,40,36,36,41,35,35,43,37,37,43,38,38,43,36,36,40,36,36,35,30,
      30,40,35,35,43,39,39,39,33,33,47,39,39,41,37,37,38,37,37,44,38,38,46,40,40,
      46,40,40,42,36,36,42,35,35,42,33,33,33,29,29,34,32,31,30,30,32,14,14,25,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,
      24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,
      12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,
      24,13,13,25,9,9,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,4,4,4,109,93,93,219,190,190,208,184,184,
      142,126,126,65,57,57,54,46,46,55,48,48,51,43,43,54,46,46,40,35,35,72,64,64,150,
      135,135,153,140,140,149,137,137,95,87,87,38,38,38,38,39,39,41,38,38,79,71,71,159,150,
      150,183,171,171,166,153,153,83,75,75,47,42,42,49,41,41,41,37,37,37,36,36,35,33,33,
      33,31,31,38,35,35,37,38,38,35,35,35,35,33,33,34,32,32,29,26,26,34,30,30,141,
      129,129,193,180,180,190,177,177,183,173,173,112,105,105,48,44,44,40,36,36,44,40,40,47,40,
      40,40,34,34,46,39,39,58,51,51,50,47,47,52,45,45,47,41,41,40,35,35,36,31,31,
      91,79,79,195,167,167,179,153,153,179,158,158,125,112,112,44,39,39,34,31,31,42,38,38,42,
      38,38,39,36,36,42,36,36,44,36,36,44,36,36,31,27,27,59,50,50,187,156,156,117,99,
      99,52,43,43,40,33,33,41,36,36,44,37,37,40,33,33,74,61,61,213,178,178,230,194,194,
      228,190,190,174,147,147,58,50,50,27,24,24,40,35,35,42,36,36,41,36,36,42,36,36,45,
      36,36,38,33,33,34,30,30,116,101,101,179,156,156,176,159,159,174,161,161,102,95,95,23,22,
      22,36,35,35,39,36,36,27,24,24,51,46,46,185,164,164,224,194,194,198,175,175,198,176,176,
      96,85,85,32,29,29,38,38,38,36,37,37,33,33,33,32,32,32,30,30,30,30,30,30,35,
      35,35,36,35,35,40,37,37,40,39,39,39,35,35,39,35,35,40,36,36,39,34,34,36,33,
      33,37,34,34,37,32,32,36,33,33,35,33,33,37,33,33,43,36,36,41,36,36,43,36,36,
      40,34,34,38,35,35,38,34,34,38,33,33,39,32,32,47,39,39,46,38,38,44,35,35,40,
      33,33,31,29,31,13,13,25,11,11,23,12,12,24,12,12,24,12,12,24,12,12,24,12,12,
      24,12,12,24,12,12,24,12,12,23,11,11,21,10,10,20,10,10,20,11,11,22,11,11,23,
      12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,
      24,12,12,24,12,12,24,12,12,24,12,12,25,11,11,22,1,1,3,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,69,59,59,214,186,186,195,172,172,141,125,125,72,63,63,63,54,54,59,50,50,62,
      55,55,78,67,67,73,63,63,133,113,113,178,151,151,158,141,141,149,139,139,92,85,85,41,38,
      38,43,39,39,49,44,44,69,62,62,142,134,134,168,157,157,162,149,149,97,86,86,43,37,37,
      45,42,42,36,33,33,33,30,30,32,31,31,27,24,24,30,25,25,32,28,28,33,32,32,38,
      35,35,40,37,37,44,42,42,43,39,39,114,102,102,190,177,177,188,176,176,191,179,179,187,172,
      172,135,120,120,87,74,74,69,60,60,61,53,53,60,51,51,46,39,39,30,27,27,23,23,23,
      20,19,19,28,25,25,35,31,31,36,33,33,97,84,84,193,168,168,181,160,160,178,162,162,103,
      94,94,36,33,33,38,34,34,44,39,39,45,39,39,42,36,36,42,36,36,42,36,36,40,36,
      36,31,27,27,33,30,30,56,48,48,37,33,33,34,31,31,41,37,37,45,41,41,44,36,36,
      34,28,28,73,61,61,210,176,176,213,179,179,200,169,169,160,138,138,57,50,50,23,20,20,39,
      34,34,41,34,34,39,33,33,42,35,35,45,39,39,39,35,35,36,32,32,109,94,94,164,145,
      145,167,156,156,186,176,176,110,105,105,23,23,23,42,36,36,38,33,33,21,21,21,52,49,49,
      199,175,175,232,199,199,209,187,187,199,178,178,76,67,67,27,24,24,39,35,35,33,33,33,34,
      34,34,32,32,32,32,32,32,32,32,32,38,35,35,37,36,36,36,35,35,35,32,32,34,33,
      33,37,35,35,37,34,34,36,32,32,33,32,32,32,30,30,32,28,28,33,33,33,35,34,34,
      33,30,30,41,35,35,42,36,36,41,33,33,41,36,36,39,37,37,40,36,36,40,35,35,44,
      36,36,47,40,40,42,37,37,38,31,30,44,37,35,27,27,31,10,10,23,12,12,24,12,12,
      24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,11,11,22,11,11,21,20,20,41,
      21,21,43,17,17,34,14,14,27,12,12,23,10,10,19,11,11,21,11,11,22,12,12,24,12,
      12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,24,12,12,23,12,12,
      23,12,12,23,11,11,23,11,11,22,11,11,22,11,11,23,11,11,23,12,12,25,12,12,25,
      9,9,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,58,51,51,198,174,174,180,158,158,166,
      145,145,141,125,125,143,125,125,158,137,137,185,161,161,197,168,168,196,164,164,196,162,162,185,155,
      155,169,150,150,154,142,142,83,76,76,45,42,42,54,45,45,61,50,50,62,53,53,145,132,132,
      178,160,160,173,150,150,110,96,96,52,45,45,64,55,55,61,55,55,56,47,47,54,47,47,52,
      46,46,59,51,51,66,57,57,82,70,70,106,92,92,98,87,87,97,90,90,64,59,59,94,83,
      83,194,175,175,200,182,182,197,179,179,194,175,175,204,181,181,180,157,157,117,101,101,83,72,72,
      80,69,69,83,73,73,64,59,59,46,43,43,36,34,34,28,28,28,27,25,25,37,33,33,115,
      100,100,194,170,170,180,160,160,178,163,163,88,83,83,29,28,28,39,35,35,45,37,37,46,38,
      38,40,33,33,39,31,31,38,31,31,35,31,31,29,26,26,31,28,28,34,31,31,36,33,33,
      35,32,32,42,39,39,48,41,41,43,35,35,34,29,29,69,57,57,197,166,166,196,164,164,179,
      153,153,151,131,131,57,50,50,25,22,22,40,32,32,42,35,35,38,31,31,42,36,36,41,38,
      38,39,35,35,39,35,35,100,89,89,155,140,140,160,148,148,188,174,174,118,112,112,29,27,27,
      44,38,38,34,32,32,21,19,19,49,45,45,179,158,158,224,196,196,218,192,192,208,187,187,73,
      65,65,24,21,21,40,36,36,34,31,31,29,29,29,33,33,33,35,35,35,34,34,34,35,33,
      33,35,34,34,36,36,36,35,31,31,35,33,33,35,35,35,35,32,32,32,30,30,34,31,31,
      34,28,28,32,28,28,36,33,33,40,37,37,33,31,31,39,35,35,41,34,34,44,35,35,46,
      39,39,43,39,39,41,37,37,38,35,35,37,34,34,36,32,32,42,35,35,39,34,32,33,30,
      39,8,8,16,9,9,17,11,11,21,11,11,22,12,12,23,12,12,23,11,11,21,10,10,20,
      8,8,17,12,12,24,27,27,55,55,55,112,63,63,130,54,54,109,40,40,79,32,32,65,20,
      20,43,16,16,33,14,14,28,10,10,20,9,9,19,10,10,20,10,10,21,10,10,20,10,10,
      19,10,10,21,11,11,22,11,11,23,11,11,24,13,13,26,14,14,28,15,15,30,16,16,32,
      15,15,30,15,15,29,16,16,34,13,13,26,1,1,1,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,53,46,46,185,165,165,176,160,160,166,151,151,166,150,150,176,156,156,194,170,170,214,189,
      189,231,199,199,223,188,188,200,167,167,186,157,157,186,159,159,172,152,152,77,70,70,57,49,49,
      63,52,52,63,55,55,59,52,52,154,135,135,196,170,170,188,159,159,153,132,132,139,119,119,172,
      146,146,185,159,159,173,147,147,166,140,140,154,130,130,144,122,122,152,129,129,139,120,120,118,104,
      104,106,96,96,83,76,76,60,56,56,70,65,65,180,161,161,205,184,184,200,180,180,192,171,171,
      198,175,175,198,174,174,179,154,154,139,119,119,97,83,83,68,60,60,67,61,61,67,61,61,58,
      54,54,44,40,40,30,30,30,33,29,29,115,100,100,196,170,170,182,158,158,173,157,157,68,65,
      65,26,24,24,40,35,35,43,34,34,41,33,33,37,30,30,35,28,28,34,28,28,32,28,28,
      28,28,28,33,30,30,38,33,33,33,29,29,33,29,29,41,37,37,47,41,41,45,37,37,38,
      32,32,62,53,53,168,144,144,182,153,153,167,144,144,134,119,119,49,46,46,26,23,23,40,33,
      33,41,35,35,39,33,33,40,35,35,41,35,35,43,36,36,41,35,35,110,100,100,158,144,144,
      164,150,150,189,171,171,120,110,110,33,27,27,45,39,39,34,32,32,24,22,22,47,42,42,152,
      138,138,209,188,188,213,190,190,213,190,190,92,80,80,29,25,25,39,36,36,36,31,31,27,26,
      26,31,30,30,35,34,34,32,32,32,31,32,32,34,33,33,37,36,36,38,35,35,36,32,32,
      33,30,30,31,28,28,27,25,25,31,27,27,36,29,29,37,33,33,38,32,32,40,36,36,35,
      32,32,39,34,34,41,35,35,40,35,35,38,35,35,37,34,34,37,32,32,36,33,33,35,32,
      32,35,31,31,41,34,32,44,41,58,51,52,104,30,30,63,25,25,51,14,14,29,12,12,25,
      10,10,22,9,9,19,15,15,30,19,19,37,26,26,54,40,40,83,60,60,123,64,64,131,61,
      61,126,61,61,123,61,61,123,58,58,121,54,54,113,43,43,89,37,37,77,28,28,58,21,21,
      44,19,19,38,19,19,38,20,20,43,24,24,47,24,24,49,23,23,48,22,22,48,23,23,50,
      25,25,51,27,27,55,27,27,56,28,28,59,24,24,51,21,21,43,17,17,33,9,9,16,1,
      1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,49,44,44,172,156,156,169,158,158,167,157,
      157,165,154,154,170,155,155,179,159,159,196,174,174,224,194,194,227,192,192,213,178,178,201,169,169,
      209,177,177,184,158,158,65,57,57,65,55,55,64,54,54,59,54,54,56,50,50,159,137,137,206,
      174,174,204,175,175,204,175,175,214,184,184,230,194,194,231,195,195,220,185,185,216,180,180,220,182,
      182,223,186,186,237,199,199,173,146,146,58,50,50,58,52,52,53,49,49,55,51,51,60,55,55,
      130,117,117,206,187,187,198,178,178,191,170,170,189,167,167,185,163,163,176,151,151,178,155,155,167,
      146,146,108,94,94,54,51,51,46,44,44,55,51,51,49,46,46,35,34,34,32,27,27,118,104,
      104,210,184,184,201,176,176,164,145,145,60,54,54,29,26,26,40,32,32,37,29,29,35,28,28,
      35,31,31,33,28,28,31,28,28,32,28,28,31,28,28,37,33,33,35,31,31,32,30,30,34,
      30,30,41,37,37,46,41,41,46,39,39,40,36,36,57,49,49,155,133,133,180,152,152,166,144,
      144,132,119,119,48,45,45,28,25,25,40,36,36,40,35,35,37,34,34,40,35,35,42,36,36,
      46,39,39,46,38,38,112,100,100,167,152,152,174,155,155,200,178,178,125,110,110,34,28,28,50,
      42,42,35,33,33,28,25,25,52,44,44,149,137,137,200,184,184,206,185,185,205,187,187,105,94,
      94,32,30,30,40,35,35,38,34,34,26,24,24,28,26,26,29,27,27,32,30,30,35,33,33,
      40,35,35,43,36,36,43,38,38,40,35,35,40,34,34,40,34,34,37,35,35,32,29,29,35,
      28,28,38,33,33,39,32,32,41,36,36,33,33,33,35,33,33,33,32,32,33,30,30,35,31,
      31,34,32,32,36,34,34,35,33,33,35,32,32,33,28,27,41,35,32,60,59,113,73,73,150,
      73,73,148,66,66,133,47,47,99,44,44,92,41,41,86,40,40,83,47,47,96,52,52,106,63,
      63,128,67,67,136,63,63,128,61,61,127,61,61,122,60,60,121,58,58,120,56,56,116,57,57,
      114,56,56,114,54,54,111,55,55,112,48,48,100,43,43,87,42,42,86,43,43,90,46,46,95,
      43,43,90,42,42,86,39,39,80,38,38,77,34,34,71,32,32,67,28,28,60,26,26,54,23,
      23,47,16,16,36,8,8,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,32,30,30,140,131,131,169,159,159,170,159,159,166,158,158,159,150,150,166,149,149,185,164,164,
      206,180,180,217,188,188,220,185,185,225,190,190,234,198,198,145,121,121,66,54,54,57,47,47,55,
      46,46,51,45,45,41,37,37,151,127,127,214,181,181,221,185,185,221,188,188,211,184,184,210,177,
      177,203,170,170,193,162,162,187,157,157,203,168,168,217,181,181,222,184,184,166,138,138,66,56,56,
      64,56,56,60,53,53,58,53,53,51,46,46,75,68,68,173,158,158,188,171,171,183,166,166,186,
      166,166,182,160,160,170,147,147,165,144,144,168,150,150,159,143,143,109,98,98,48,45,45,37,35,
      35,37,37,37,30,31,31,37,35,35,133,119,119,216,193,193,211,187,187,177,157,157,64,57,57,
      29,25,25,37,30,30,36,30,30,37,31,31,34,29,29,30,26,26,34,28,28,33,28,28,35,
      31,31,40,36,36,35,34,34,33,32,32,36,33,33,41,36,36,45,38,38,46,38,38,40,36,
      36,55,49,49,158,136,136,188,164,164,176,155,155,150,134,134,58,51,51,29,28,28,41,36,36,
      43,36,36,36,36,36,38,36,36,44,38,38,46,37,37,46,39,39,120,108,108,184,163,163,188,
      164,164,209,182,182,129,111,111,35,30,30,48,41,41,36,32,32,32,29,29,57,50,50,159,147,
      147,192,179,179,192,177,177,197,183,183,79,73,73,24,22,22,43,36,36,42,37,37,35,31,31,
      33,29,29,33,29,29,33,29,29,37,33,33,41,37,37,42,36,36,40,33,33,39,35,35,42,
      36,36,45,38,38,46,39,39,36,32,32,37,30,30,37,34,34,38,34,34,44,37,37,38,35,
      35,37,34,34,38,36,36,38,33,33,38,33,33,38,34,34,37,35,35,34,34,34,35,33,33,
      34,28,25,47,45,69,72,72,142,74,73,149,72,72,145,71,71,145,71,71,148,72,72,147,72,
      72,145,71,71,144,67,67,140,66,66,137,65,65,130,63,63,128,61,61,126,60,60,124,60,60,
      122,58,58,119,56,56,117,56,56,114,56,56,113,53,53,111,52,52,107,50,50,103,49,49,103,
      49,49,99,46,46,96,45,45,93,43,43,87,40,40,84,39,39,79,36,36,75,34,34,71,32,
      32,66,29,29,62,27,27,56,23,23,47,21,21,41,15,15,32,2,2,4,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,25,23,23,132,123,123,176,162,162,173,158,158,
      170,158,158,160,150,150,168,152,152,177,160,160,181,161,161,191,169,169,213,183,183,240,203,203,202,
      172,172,79,65,65,48,40,40,51,43,43,51,42,42,46,39,39,42,35,35,168,145,145,236,201,
      201,233,196,196,227,191,191,210,180,180,197,168,168,185,155,155,178,149,149,177,149,149,193,161,161,
      212,176,176,220,182,182,172,143,143,75,64,64,64,53,53,58,51,51,57,53,53,59,53,53,59,
      54,54,115,104,104,178,161,161,177,160,160,176,158,158,182,163,163,175,157,157,161,147,147,150,139,
      139,156,141,141,154,141,141,95,86,86,37,35,35,33,34,34,31,31,31,36,33,33,128,114,114,
      211,187,187,198,175,175,175,156,156,67,60,60,28,24,24,36,31,31,38,32,32,37,32,32,34,
      28,28,31,27,27,33,28,28,32,28,28,33,32,32,40,36,36,38,35,35,35,33,33,37,36,
      36,41,36,36,42,36,36,43,36,36,36,33,33,53,49,49,160,137,137,189,162,162,176,155,155,
      151,135,135,56,50,50,31,29,29,40,37,37,38,35,35,36,34,34,37,36,36,43,37,37,44,
      37,37,43,37,37,124,112,112,198,175,175,198,174,174,211,184,184,136,117,117,39,33,33,43,37,
      37,36,33,33,35,32,32,58,53,53,158,145,145,185,174,174,182,171,171,193,181,181,98,91,91,
      28,25,25,36,29,29,39,35,35,39,34,34,39,32,32,41,36,36,49,43,43,59,49,49,64,
      55,55,63,56,56,68,57,57,60,51,51,41,37,37,40,34,34,42,36,36,40,35,35,42,35,
      35,39,35,35,38,33,33,45,37,37,44,37,37,44,36,36,45,40,40,47,40,40,47,40,40,
      49,41,41,45,38,38,38,35,35,37,33,30,39,36,54,62,61,126,73,73,148,72,72,145,69,
      69,142,68,68,141,68,68,140,68,68,138,68,68,137,65,65,134,64,64,132,64,64,130,63,63,
      128,60,60,125,60,60,123,60,60,121,58,58,119,56,56,117,56,56,114,55,55,112,53,53,110,
      52,52,107,52,52,106,49,49,102,48,48,99,48,48,97,44,44,93,43,43,89,41,41,85,40,
      40,81,38,38,77,36,36,73,33,33,68,29,29,61,27,27,56,26,26,53,21,21,44,15,15,
      34,7,7,15,1,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      16,15,15,129,115,115,197,173,173,191,171,171,185,169,169,177,161,161,179,161,161,185,168,168,179,
      158,158,189,164,164,206,176,176,170,144,144,84,72,72,48,40,40,46,39,39,51,42,42,49,43,
      43,42,37,37,35,31,31,145,128,128,223,192,192,222,190,190,206,176,176,196,168,168,189,162,162,
      174,147,147,172,145,145,183,153,153,197,164,164,215,179,179,220,182,182,176,148,148,77,66,66,65,
      54,54,58,50,50,56,51,51,66,59,59,64,59,59,74,65,65,132,117,117,177,164,164,173,157,
      157,175,158,158,166,152,152,153,139,139,144,133,133,150,139,139,160,148,148,163,148,148,74,67,67,
      29,29,29,30,30,30,33,32,32,118,106,106,204,182,182,188,168,168,163,149,149,65,59,59,29,
      25,25,39,32,32,37,30,30,36,31,31,34,28,28,32,28,28,32,28,28,32,28,28,33,32,
      32,39,36,36,38,36,36,39,35,35,37,36,36,37,34,34,39,34,34,43,36,36,36,32,32,
      48,43,43,146,127,127,186,164,164,173,155,155,160,144,144,64,57,57,33,29,29,39,36,36,32,
      32,32,33,32,32,38,35,35,43,36,36,41,36,36,40,34,34,115,100,100,205,181,181,201,177,
      177,211,184,184,138,119,119,42,36,36,42,34,34,36,31,31,33,30,30,58,52,52,149,137,137,
      179,169,169,171,164,164,177,169,169,128,119,119,78,70,70,76,68,68,72,64,64,86,74,74,100,
      88,88,106,96,96,111,97,97,110,94,94,107,95,95,110,98,98,116,101,101,94,79,79,44,36,
      36,42,35,35,40,35,35,39,36,36,44,39,39,40,36,36,36,32,32,38,33,33,38,33,33,
      38,33,33,38,36,36,44,39,39,45,38,38,45,39,39,46,39,39,41,35,35,37,31,27,56,
      55,101,74,74,152,73,73,146,68,68,142,68,68,141,68,68,140,68,68,137,66,66,135,67,67,
      136,65,65,132,64,64,130,63,63,128,61,61,126,60,60,125,60,60,123,60,60,121,57,57,118,
      56,56,117,55,55,113,53,53,110,52,52,108,52,52,105,50,50,103,48,48,99,48,48,97,46,
      46,94,43,43,89,41,41,85,40,40,83,38,38,78,36,36,75,34,34,70,32,32,66,28,28,
      60,25,25,52,23,23,47,18,18,38,7,7,15,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,9,8,8,122,109,109,211,187,187,204,181,181,151,
      136,136,152,137,137,171,152,152,191,171,171,191,165,165,199,167,167,185,157,157,94,79,79,45,38,
      38,50,42,42,48,40,40,48,41,41,47,43,43,41,37,37,31,30,30,122,108,108,198,172,172,
      211,182,182,205,178,178,194,169,169,188,162,162,179,152,152,187,158,158,200,168,168,214,178,178,225,
      189,189,226,189,189,189,160,160,85,74,74,65,57,57,66,57,57,62,55,55,64,56,56,65,57,
      57,58,47,47,62,52,52,114,104,104,172,160,160,184,167,167,172,159,159,156,143,143,152,141,141,
      157,149,149,169,156,156,190,174,174,136,125,125,48,45,45,32,30,30,36,34,34,117,105,105,198,
      178,178,181,168,168,159,147,147,65,59,59,29,26,26,39,34,34,37,31,31,39,32,32,36,30,
      30,35,30,30,36,31,31,36,32,32,36,32,32,37,34,34,40,36,36,39,35,35,34,32,32,
      32,32,32,37,35,35,44,39,39,36,29,29,44,41,41,150,132,132,196,177,177,182,165,165,166,
      153,153,68,64,64,30,27,27,35,31,31,32,29,29,32,32,32,35,34,34,41,36,36,39,35,
      35,39,32,32,131,116,116,205,181,181,199,174,174,207,181,181,127,110,110,33,29,29,44,37,37,
      38,32,32,34,28,28,55,50,50,141,132,132,167,162,162,163,159,159,165,160,160,163,153,153,165,
      154,154,169,157,157,172,160,160,188,171,171,189,172,172,178,164,164,164,151,151,153,140,140,147,134,
      134,147,135,135,119,106,106,54,45,45,39,31,31,44,37,37,43,37,37,40,35,35,43,37,37,
      36,32,32,31,31,31,34,32,32,36,34,34,38,35,35,39,36,36,40,36,36,40,34,34,40,
      34,34,44,41,40,41,35,31,50,46,75,69,70,142,73,73,148,69,69,142,68,68,140,68,68,
      140,68,68,137,68,68,137,66,66,135,64,64,133,64,64,130,64,64,129,63,63,128,60,60,124,
      60,60,123,60,60,121,57,57,119,56,56,117,56,56,114,55,55,113,52,52,108,52,52,105,50,
      50,103,48,48,101,48,48,98,45,45,94,44,44,89,43,43,88,40,40,84,38,38,79,36,36,
      74,34,34,70,32,32,65,29,29,61,28,28,56,24,24,47,20,20,41,10,10,21,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,
      6,6,116,101,101,200,175,175,195,171,171,115,100,100,62,52,52,68,60,60,134,119,119,193,166,
      166,194,164,164,213,179,179,154,130,130,62,51,51,49,40,40,47,40,40,43,39,39,44,40,40,
      42,37,37,34,33,33,115,101,101,190,163,163,200,172,172,194,167,167,169,146,146,162,140,140,168,
      144,144,172,148,148,186,161,161,210,180,180,223,188,188,230,193,193,190,162,162,89,76,76,63,54,
      54,64,57,57,49,44,44,43,40,40,45,41,41,52,45,45,54,46,46,51,43,43,87,78,78,
      150,136,136,178,165,165,173,162,162,161,152,152,160,153,153,171,160,160,189,174,174,195,175,175,75,
      67,67,30,28,28,43,38,38,120,105,105,192,173,173,178,164,164,157,145,145,64,60,60,29,27,
      27,40,35,35,40,32,32,39,32,32,36,32,32,36,32,32,39,33,33,40,35,35,38,33,33,
      40,36,36,41,36,36,37,35,35,31,30,30,30,30,30,37,35,35,47,40,40,35,31,31,50,
      43,43,167,148,148,201,182,182,191,176,176,178,164,164,71,65,65,28,24,24,35,28,28,33,30,
      30,35,35,35,36,31,31,39,34,34,39,36,36,35,32,32,115,103,103,189,167,167,192,168,168,
      204,181,181,119,108,108,30,25,25,42,36,36,36,33,33,31,25,25,53,48,48,142,133,133,166,
      161,161,162,157,157,164,162,162,168,163,163,174,165,165,177,165,165,181,171,171,196,183,183,198,181,
      181,191,174,174,183,169,169,181,170,170,172,159,159,176,161,161,143,129,129,31,28,28,37,32,32,
      44,36,36,44,36,36,41,35,35,40,35,35,34,30,30,34,30,30,37,34,34,42,39,39,46,
      39,39,43,39,39,42,38,38,35,32,32,35,31,31,43,39,33,41,38,52,60,61,125,74,74,
      151,69,69,142,68,68,140,68,68,137,68,68,137,66,66,135,66,66,135,66,66,133,64,64,128,
      64,64,129,61,61,126,62,62,127,60,60,121,60,60,121,58,58,119,56,56,117,56,56,114,56,
      56,112,52,52,110,52,52,107,51,51,103,48,48,101,48,48,98,45,45,94,44,44,93,44,44,
      88,40,40,84,39,39,79,36,36,76,34,34,72,31,31,67,29,29,62,27,27,56,24,24,50,
      20,20,43,15,15,32,6,6,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,1,0,0,104,89,89,197,171,171,187,162,162,118,102,
      102,57,50,50,47,43,43,64,57,57,145,127,127,193,166,166,198,169,169,194,165,165,82,69,69,
      46,39,39,47,39,39,42,38,38,45,41,41,43,41,41,34,33,33,121,105,105,200,171,171,201,
      171,171,167,143,143,72,62,62,59,52,52,86,75,75,89,77,77,91,80,80,117,101,101,142,120,
      120,150,126,126,123,105,105,70,60,60,51,45,45,45,42,42,41,40,40,38,38,38,40,36,36,
      46,41,41,47,42,42,43,38,38,47,41,41,77,69,69,134,122,122,166,154,154,173,165,165,166,
      161,161,173,159,159,189,170,170,212,187,187,105,94,94,38,32,32,41,38,38,111,98,98,187,168,
      168,173,160,160,168,159,159,70,65,65,29,26,26,40,35,35,40,33,33,40,33,33,35,31,31,
      38,34,34,40,36,36,43,36,36,42,37,37,40,35,35,41,37,37,37,35,35,30,29,29,27,
      26,26,35,30,30,43,38,38,38,32,32,54,47,47,166,147,147,199,177,177,188,171,171,193,175,
      175,78,70,70,24,19,19,39,31,31,37,34,34,37,35,35,39,34,34,44,37,37,41,36,36,
      35,33,33,105,93,93,178,159,159,186,170,170,198,183,183,112,106,106,29,26,26,42,34,34,36,
      32,32,30,26,26,54,50,50,139,130,130,165,162,162,159,154,154,163,157,157,173,165,165,174,165,
      165,173,159,159,176,161,161,191,176,176,204,184,184,198,177,177,187,168,168,186,174,174,180,167,167,
      189,172,172,148,136,136,27,27,27,39,34,34,46,37,37,41,36,36,40,37,37,40,33,33,43,
      38,38,40,36,36,42,38,38,46,41,41,47,40,40,45,38,38,45,39,39,33,31,31,31,28,
      26,42,39,38,64,62,106,74,74,152,69,69,142,68,68,140,68,68,138,68,68,137,67,67,136,
      66,66,135,64,64,133,64,64,131,63,63,128,64,64,129,60,60,125,60,60,123,60,60,121,57,
      57,119,56,56,117,56,56,115,55,55,112,53,53,110,52,52,108,50,50,103,48,48,101,48,48,
      99,46,46,94,44,44,92,43,43,89,41,41,85,40,40,80,38,38,78,36,36,74,34,34,69,
      30,30,62,27,27,56,25,25,52,22,22,46,15,15,34,6,6,12,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,97,86,86,185,163,163,180,159,159,117,103,103,53,48,48,48,40,40,43,36,36,99,86,86,
      184,160,160,180,156,156,187,163,163,138,119,119,52,47,47,44,37,37,46,42,42,50,45,45,48,
      45,45,37,39,39,113,99,99,204,173,173,210,178,178,165,139,139,59,51,51,50,42,42,58,49,
      49,59,50,50,54,46,46,51,42,42,55,44,44,56,48,48,54,43,43,54,45,45,56,47,47,
      54,47,47,45,42,42,36,36,36,39,36,36,58,51,51,68,62,62,70,64,64,75,66,66,84,
      74,74,79,71,71,74,67,67,120,113,113,156,148,148,177,159,159,192,169,169,211,183,183,145,125,
      125,51,43,43,35,32,32,95,84,84,194,175,175,181,164,164,184,169,169,75,68,68,30,25,25,
      40,36,36,41,35,35,42,35,35,36,33,33,39,35,35,40,36,36,44,37,37,46,40,40,42,
      35,35,42,39,39,34,31,31,30,28,28,27,25,25,29,26,26,37,34,34,40,32,32,56,48,
      48,159,139,139,200,176,176,190,172,172,196,176,176,78,70,70,26,21,21,40,34,34,40,36,36,
      40,36,36,42,36,36,44,38,38,40,36,36,39,34,34,88,78,78,175,159,159,184,172,172,196,
      184,184,114,109,109,28,25,25,43,37,37,37,32,32,39,31,31,60,51,51,137,127,127,164,158,
      158,158,149,149,160,149,149,169,160,160,177,167,167,176,159,159,182,162,162,197,174,174,205,181,181,
      207,181,181,200,178,178,196,179,179,187,174,174,192,180,180,153,144,144,32,30,30,36,33,33,42,
      36,36,39,36,36,39,34,34,38,32,32,43,36,36,43,38,38,45,39,39,45,40,40,42,38,
      38,39,34,34,38,33,33,33,28,27,33,30,29,46,47,80,71,71,149,70,70,142,68,68,139,
      68,68,138,68,68,137,66,66,135,64,64,133,64,64,131,64,64,130,63,63,128,60,60,126,62,
      62,127,61,61,122,60,60,121,58,58,119,56,56,116,56,56,114,55,55,112,52,52,110,52,52,
      108,50,50,103,48,48,101,48,48,99,47,47,97,45,45,94,43,43,88,40,40,86,40,40,84,
      38,38,79,36,36,74,33,33,70,31,31,65,28,28,58,25,25,52,23,23,47,15,15,35,6,
      6,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,76,67,67,168,149,149,167,150,150,121,109,109,
      51,46,46,43,36,36,43,36,36,61,52,52,138,123,123,174,155,155,177,157,157,169,151,151,69,
      61,61,46,40,40,51,47,47,52,47,47,52,47,47,45,44,44,94,83,83,196,166,166,215,181,
      181,183,153,153,86,75,75,52,41,41,63,53,53,66,54,54,60,50,50,56,48,48,61,51,51,
      65,55,55,62,51,51,59,49,49,62,53,53,60,52,52,53,49,49,66,62,62,85,78,78,94,
      87,87,95,88,88,82,76,76,59,53,53,45,41,41,41,37,37,42,39,39,42,37,37,107,97,
      97,181,165,165,190,165,165,207,178,178,173,148,148,60,50,50,31,29,29,95,83,83,220,192,192,
      205,182,182,206,186,186,98,89,89,33,28,28,37,33,33,41,35,35,46,38,38,44,39,39,41,
      37,37,39,35,35,46,39,39,49,40,40,47,38,38,43,38,38,38,34,34,30,29,29,27,25,
      25,28,26,26,30,28,28,37,31,31,52,43,43,136,119,119,208,182,182,202,180,180,199,175,175,
      79,67,67,28,24,24,41,36,36,40,34,34,40,36,36,39,35,35,37,33,33,39,35,35,41,
      36,36,93,83,83,180,166,166,188,174,174,198,183,183,121,115,115,32,31,31,46,39,39,45,36,
      36,44,36,36,59,50,50,139,128,128,164,155,155,159,149,149,160,147,147,162,150,150,175,162,162,
      181,165,165,187,165,165,200,174,174,212,184,184,218,189,189,214,187,187,204,184,184,193,178,178,199,
      185,185,158,146,146,35,30,30,33,28,28,37,34,34,40,35,35,40,34,34,38,32,32,38,33,
      33,40,33,33,40,36,36,39,32,32,39,33,33,44,37,37,39,34,34,33,30,25,42,40,49,
      61,61,128,69,69,141,68,68,138,68,68,137,67,67,136,68,68,137,64,64,131,64,64,130,64,
      64,129,63,63,128,61,61,126,60,60,124,60,60,123,60,60,122,57,57,119,56,56,117,56,56,
      114,55,55,112,52,52,110,52,52,108,51,51,103,48,48,100,48,48,99,48,48,97,45,45,94,
      43,43,88,40,40,85,40,40,83,39,39,79,36,36,74,34,34,69,32,32,65,28,28,59,24,
      24,53,24,24,47,18,18,37,6,6,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      61,55,55,162,148,148,158,144,144,123,112,112,51,46,46,38,33,33,47,40,40,46,39,39,100,
      89,89,164,150,150,170,156,156,170,156,156,111,100,100,52,45,45,51,48,48,53,48,48,54,49,
      49,50,46,46,91,81,81,199,172,172,214,183,183,179,151,151,80,70,70,49,42,42,61,52,52,
      61,52,52,60,52,52,56,49,49,58,52,52,65,52,52,65,53,53,68,55,55,66,55,55,58,
      50,50,76,70,70,145,134,134,173,161,161,166,151,151,99,89,89,29,28,28,27,26,26,36,34,
      34,47,43,43,46,42,42,39,36,36,62,55,55,154,137,137,196,171,171,212,183,183,217,184,184,
      70,58,58,26,25,25,87,75,75,226,192,192,218,189,189,214,189,189,121,107,107,41,37,37,37,
      33,33,43,37,37,48,40,40,43,38,38,41,37,37,40,36,36,44,39,39,48,40,40,45,39,
      39,45,37,37,46,39,39,37,32,32,29,27,27,27,27,27,29,27,27,29,25,25,51,46,46,
      142,123,123,210,182,182,204,179,179,213,182,182,86,73,73,32,25,25,44,36,36,40,34,34,39,
      34,34,37,33,33,34,30,30,35,31,31,40,37,37,97,88,88,187,173,173,193,176,176,202,185,
      185,122,113,113,32,27,27,47,39,39,49,41,41,42,34,34,55,48,48,145,131,131,175,160,160,
      168,151,151,163,147,147,161,146,146,166,151,151,178,159,159,186,163,163,199,174,174,218,190,190,231,
      197,197,225,193,193,206,182,182,193,170,170,204,184,184,156,142,142,26,25,25,33,29,29,41,36,
      36,44,38,38,43,35,35,41,35,35,40,34,34,37,31,31,40,34,34,43,36,36,41,32,32,
      41,36,36,40,37,35,39,38,38,55,56,100,70,70,144,68,68,136,68,68,137,66,66,135,64,
      64,133,65,65,134,64,64,129,64,64,129,64,64,129,61,61,126,60,60,124,60,60,121,60,60,
      121,58,58,119,56,56,116,56,56,114,55,55,112,53,53,110,52,52,108,50,50,103,48,48,101,
      48,48,99,48,48,96,45,45,94,43,43,88,41,41,85,40,40,83,38,38,78,36,36,73,33,
      33,70,31,31,65,28,28,59,25,25,52,23,23,47,18,18,37,6,6,14,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,62,57,57,161,148,148,157,146,146,128,116,116,53,
      47,47,45,40,40,54,46,46,49,43,43,71,62,62,136,123,123,164,153,153,166,151,151,155,137,
      137,62,54,54,50,46,46,55,48,48,56,49,49,51,46,46,79,72,72,180,162,162,201,178,178,
      171,149,149,84,75,75,45,40,40,55,50,50,55,48,48,57,49,49,56,48,48,59,52,52,63,
      51,51,65,52,52,71,58,58,61,51,51,52,45,45,76,65,65,175,159,159,201,183,183,214,188,
      188,124,107,107,26,23,23,32,31,31,38,35,35,45,41,41,44,40,40,44,40,40,37,33,33,
      113,100,100,211,186,186,214,182,182,221,185,185,81,66,66,29,24,24,70,61,61,201,169,169,221,
      188,188,212,184,184,147,128,128,50,44,44,37,33,33,45,37,37,46,38,38,38,32,32,38,34,
      34,39,34,34,40,38,38,45,38,38,44,37,37,44,37,37,47,40,40,45,38,38,38,32,32,
      34,30,30,37,35,35,51,43,43,54,47,47,139,121,121,202,171,171,201,169,169,216,181,181,89,
      75,75,30,25,25,43,35,35,40,33,33,39,32,32,34,30,30,32,28,28,33,29,29,40,34,
      34,101,92,92,191,175,175,196,179,179,202,187,187,133,123,123,40,33,33,42,38,38,45,40,40,
      38,30,30,55,47,47,144,131,131,181,162,162,178,158,158,168,152,152,162,144,144,163,144,144,166,
      146,146,178,157,157,198,171,171,215,186,186,235,200,200,229,197,197,208,180,180,193,168,168,198,175,
      175,157,141,141,28,27,27,30,29,29,38,33,33,38,34,34,38,36,36,39,34,34,41,36,36,
      40,36,36,43,37,37,46,42,42,44,37,38,40,35,34,38,35,34,46,45,79,67,67,139,69,
      69,138,67,67,136,65,65,134,64,64,133,64,64,131,64,64,130,63,63,128,61,61,126,61,61,
      126,60,60,124,60,60,121,58,58,119,57,57,118,56,56,115,55,55,113,53,53,110,52,52,108,
      52,52,106,50,50,104,48,48,101,48,48,98,45,45,94,45,45,93,43,43,89,40,40,85,40,
      40,84,38,38,78,36,36,74,34,34,69,31,31,65,28,28,59,24,24,53,23,23,47,18,18,
      37,6,6,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,58,
      53,53,157,142,142,159,142,142,138,125,125,61,54,54,50,45,45,55,47,47,49,42,42,43,37,
      37,102,93,93,165,152,152,179,160,160,178,157,157,91,79,79,53,45,45,58,49,49,58,52,52,
      53,48,48,68,61,61,156,144,144,175,164,164,157,144,144,87,80,80,41,37,37,50,47,47,52,
      46,46,53,47,47,56,49,49,60,52,52,61,52,52,60,49,49,53,44,44,51,43,43,46,40,
      40,52,43,43,172,154,154,201,180,180,222,192,192,166,142,142,51,45,45,32,32,32,39,36,36,
      43,39,39,42,40,40,45,41,41,32,31,31,97,90,90,200,182,182,206,176,176,205,171,171,69,
      57,57,28,25,25,57,50,50,178,152,152,228,195,195,219,190,190,191,166,166,65,57,57,33,28,
      28,43,38,38,50,41,41,39,33,33,35,32,32,36,32,32,39,35,35,41,36,36,42,37,37,
      44,39,39,44,38,38,44,38,38,41,35,35,49,44,44,67,60,60,74,66,66,73,65,65,134,
      115,115,199,165,165,195,162,162,212,178,178,87,73,73,31,25,25,41,33,33,40,34,34,39,32,
      32,34,30,30,31,27,27,35,30,30,39,34,34,105,94,94,185,169,169,190,174,174,196,179,179,
      135,123,123,43,38,38,39,35,35,42,38,38,35,29,29,50,44,44,139,123,123,172,150,150,170,
      147,147,176,155,155,95,83,83,75,66,66,94,83,83,112,98,98,131,114,114,146,127,127,170,146,
      146,170,144,144,180,153,153,168,147,147,186,162,162,137,121,121,30,27,27,34,31,31,35,32,32,
      33,32,32,33,30,30,36,32,32,44,39,39,47,41,41,44,41,41,44,38,38,47,41,41,38,
      33,31,43,39,44,63,62,125,68,68,140,65,65,134,64,64,133,64,64,131,64,64,129,64,64,
      129,64,64,129,60,60,125,60,60,124,60,60,122,60,60,121,58,58,119,56,56,117,56,56,114,
      56,56,112,53,53,109,52,52,108,52,52,105,50,50,103,48,48,101,48,48,98,46,46,94,44,
      44,92,43,43,88,41,41,85,40,40,83,38,38,79,36,36,73,34,34,70,32,32,64,28,28,
      60,24,25,52,24,24,47,20,20,38,7,7,15,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,41,36,36,139,124,124,163,144,144,153,137,137,70,63,
      63,55,48,48,56,48,48,51,47,47,41,37,37,87,79,79,168,153,153,194,173,173,196,171,171,
      131,112,112,54,47,47,61,51,51,62,52,52,55,47,47,70,62,62,146,138,138,163,156,156,155,
      148,148,92,86,86,40,37,37,50,44,44,50,44,44,50,44,44,48,44,44,44,41,41,40,36,
      36,43,38,38,43,39,39,40,37,37,41,34,33,51,42,42,161,144,144,211,186,186,229,200,200,
      187,162,162,69,61,61,34,32,32,40,36,36,40,39,39,43,41,41,45,43,43,36,32,32,102,
      95,95,193,178,178,187,164,164,193,165,165,76,62,62,29,26,26,37,34,34,136,119,119,234,201,
      201,226,195,195,232,199,199,96,83,83,34,30,30,40,34,34,45,40,40,43,36,36,37,33,33,
      33,29,29,35,31,31,39,35,35,39,37,37,36,35,35,37,33,33,43,39,39,48,44,44,61,
      57,57,67,63,63,67,61,61,75,67,67,133,115,115,201,170,170,207,172,172,225,187,187,96,79,
      79,30,24,24,40,32,32,43,36,36,43,35,35,38,32,32,37,31,31,36,28,28,39,32,32,
      117,106,106,182,168,168,185,168,168,193,175,175,120,108,108,31,30,30,38,36,36,37,33,33,32,
      25,25,49,43,43,139,122,122,171,147,147,168,145,145,184,159,159,72,62,62,28,25,25,44,36,
      36,43,38,38,43,39,39,43,38,38,45,40,40,46,40,40,48,44,44,48,43,43,55,49,49,
      49,43,43,38,34,34,39,33,33,37,32,32,37,32,32,42,34,34,43,35,35,41,33,33,44,
      35,35,45,40,40,42,39,39,47,40,38,39,35,38,57,57,107,66,66,138,64,64,133,64,64,
      131,63,63,130,63,63,130,62,62,126,63,63,128,61,61,125,60,60,121,60,60,121,59,59,120,
      56,56,117,56,56,117,56,56,114,56,56,113,52,52,110,52,52,108,52,52,105,50,50,104,48,
      48,99,48,48,98,45,45,94,43,43,93,43,43,90,41,41,85,39,39,80,38,38,79,36,36,
      74,33,33,69,32,32,64,28,28,57,25,25,53,22,23,47,22,20,33,14,12,12,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,36,
      36,144,125,125,171,149,149,185,164,164,83,74,74,60,52,52,63,56,56,58,51,51,53,47,47,
      49,46,46,138,126,126,195,172,172,206,178,178,177,150,150,66,55,55,58,50,50,59,49,49,56,
      46,46,53,48,48,144,140,140,167,164,164,162,155,155,100,91,91,34,32,32,49,43,43,57,48,
      48,56,48,48,55,49,49,65,59,59,67,60,60,76,66,66,89,79,79,101,89,88,135,124,127,
      160,140,142,204,178,178,208,182,182,210,186,186,214,190,190,120,105,105,34,33,33,38,36,36,41,
      38,38,41,40,40,36,35,35,45,43,43,131,119,119,188,173,173,179,163,163,182,162,162,85,73,
      73,32,28,28,32,29,29,115,101,101,216,189,189,221,191,191,238,203,203,161,138,138,45,37,37,
      40,34,34,45,40,40,47,39,39,41,32,32,34,31,31,32,29,29,36,32,32,41,40,40,52,
      46,46,56,50,50,54,50,50,60,55,55,64,60,60,64,60,60,65,60,60,65,60,60,113,100,
      100,199,168,168,214,179,179,233,194,194,128,109,109,39,32,32,37,30,30,43,37,37,47,39,39,
      43,36,36,41,35,35,37,29,29,35,30,30,124,113,113,181,169,169,178,164,164,180,163,163,115,
      105,105,30,30,30,36,34,34,38,30,30,30,25,25,52,45,45,140,124,124,173,152,152,173,152,
      152,188,163,163,86,74,74,32,29,29,42,36,36,37,33,33,32,30,30,34,32,32,34,29,29,
      34,32,32,35,32,32,35,32,32,36,32,32,37,33,33,40,34,34,40,34,34,42,36,36,43,
      35,35,43,37,37,44,36,36,43,35,35,40,32,32,40,35,35,41,35,35,47,39,34,53,51,
      90,63,64,134,65,65,134,62,63,131,63,64,133,68,67,126,70,69,124,64,64,131,61,61,125,
      60,60,120,60,60,121,58,58,119,56,56,116,56,56,115,56,56,113,53,53,110,52,52,109,52,
      52,106,50,50,104,48,48,101,47,47,98,47,47,96,46,46,96,45,45,92,50,48,84,45,45,
      89,42,42,86,37,37,77,36,36,74,33,33,68,29,29,62,27,27,56,24,24,52,23,23,46,
      12,12,24,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,32,27,27,137,119,119,183,157,157,199,173,173,87,76,76,
      64,56,56,67,60,60,61,56,56,57,51,51,45,41,41,98,90,90,182,167,167,206,183,183,194,
      168,168,102,87,87,52,43,43,52,45,45,51,44,44,52,46,46,144,139,139,176,170,170,173,162,
      162,128,117,117,83,76,76,98,87,87,132,113,113,146,125,125,146,125,125,153,130,130,143,122,122,
      151,130,130,176,152,152,214,181,180,220,206,215,196,173,176,171,145,144,195,173,173,199,179,179,191,
      173,173,154,139,139,74,67,67,48,47,47,55,50,50,53,50,50,52,49,49,99,90,90,183,166,
      166,185,171,171,183,167,167,179,162,162,71,65,65,32,29,29,28,25,25,79,69,69,182,159,159,
      204,175,175,225,189,189,226,190,190,86,73,73,38,32,32,42,37,37,44,38,38,46,37,37,42,
      36,36,36,32,32,33,29,29,46,41,41,125,109,109,144,126,126,97,87,87,73,64,64,67,62,
      62,67,63,63,59,56,56,37,36,36,80,70,70,193,161,161,205,170,170,217,181,181,156,134,134,
      52,43,43,35,28,28,45,37,37,49,41,41,50,42,42,47,42,42,41,34,34,37,31,31,112,
      103,103,175,164,164,169,158,158,170,158,158,109,103,103,31,31,31,40,34,34,41,34,34,32,28,
      28,54,48,48,147,131,131,181,162,162,181,161,161,194,168,168,79,67,67,30,27,27,44,39,39,
      41,35,35,34,32,32,36,33,33,43,36,36,41,36,36,40,35,35,40,35,35,43,36,36,46,
      40,40,46,40,40,48,40,40,47,39,39,49,41,41,48,40,40,45,38,38,42,38,38,40,34,
      34,38,34,34,36,30,26,51,45,55,62,63,126,63,64,140,70,69,135,73,72,124,70,70,135,
      88,83,113,78,76,122,61,61,133,59,60,125,59,59,126,58,58,122,55,55,119,55,55,112,55,
      55,112,53,53,110,52,52,108,51,51,104,52,52,104,48,48,100,48,48,99,46,46,97,43,44,
      96,51,49,83,48,48,91,61,57,81,44,44,85,38,38,78,37,37,74,32,32,67,28,28,60,
      27,27,57,27,27,47,22,21,37,10,10,19,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,13,13,
      125,110,110,190,165,165,205,178,178,110,95,95,69,59,59,67,58,58,68,59,59,64,57,57,53,
      48,48,76,69,69,166,150,150,196,173,173,206,178,178,134,114,114,51,44,44,52,45,45,47,43,
      43,50,44,44,143,132,132,186,173,173,185,173,173,189,175,175,192,173,173,199,177,177,216,185,185,
      221,186,186,219,183,183,211,177,177,187,159,159,191,171,171,211,193,193,181,158,158,104,86,86,54,
      45,45,54,47,47,168,152,152,183,169,169,170,157,157,163,152,152,149,138,138,117,104,104,115,99,
      99,119,104,104,145,125,125,200,175,175,212,186,186,189,168,168,187,167,167,173,154,154,51,46,46,
      31,26,26,38,32,32,47,43,43,140,120,120,198,167,167,201,167,167,225,188,188,184,158,158,68,
      58,58,47,38,38,56,46,46,54,46,46,51,43,43,67,55,55,71,60,60,93,79,79,158,142,
      142,186,166,166,189,168,168,140,124,124,80,70,70,55,50,50,38,39,39,20,22,22,74,65,65,
      201,169,169,206,171,171,212,177,177,188,158,158,63,52,52,35,28,28,44,35,35,48,40,40,52,
      44,44,50,42,42,46,38,38,44,36,36,129,119,119,182,171,171,167,158,158,176,165,165,102,98,
      98,24,22,22,46,39,39,44,37,37,34,30,30,59,51,51,156,141,141,192,172,172,188,167,167,
      194,171,171,86,75,75,33,29,29,44,37,37,41,36,36,36,35,35,40,34,34,42,35,35,40,
      35,35,40,32,32,40,33,33,42,38,38,47,40,40,47,40,40,45,38,38,44,35,35,43,37,
      37,40,34,34,41,36,36,40,36,36,41,34,34,39,31,29,35,31,44,68,65,97,89,85,119,
      83,80,124,80,78,135,96,90,113,82,79,122,89,84,112,71,70,128,67,66,119,73,71,118,78,
      74,111,64,63,122,71,69,108,56,56,115,53,53,109,52,52,105,52,52,105,51,51,104,50,50,
      102,48,48,96,46,46,96,54,52,89,53,51,91,56,54,87,56,54,75,45,46,88,48,46,72,
      35,35,70,30,30,66,33,33,60,32,32,57,27,28,51,26,26,32,13,10,9,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,19,17,17,129,115,115,193,168,168,204,179,179,125,110,110,68,
      59,59,64,54,54,66,58,58,71,63,63,67,58,58,54,46,46,133,117,117,188,164,164,207,179,
      179,179,154,154,72,61,61,52,44,44,51,46,46,53,47,47,155,138,138,195,173,173,186,168,168,
      181,164,164,192,169,169,201,175,175,208,177,177,218,183,183,227,190,190,234,197,197,225,190,190,195,
      174,174,182,168,168,130,120,120,51,44,44,39,32,32,48,40,40,119,105,105,173,162,162,158,150,
      150,158,149,149,183,166,166,185,162,162,187,162,162,197,168,168,216,182,182,238,203,203,220,187,187,
      198,169,169,183,158,158,156,134,134,50,44,44,34,28,28,40,34,34,34,30,30,107,92,92,194,
      164,164,213,178,178,230,193,193,237,201,201,169,143,143,86,73,73,78,66,66,87,75,75,94,80,
      80,122,104,104,140,122,122,162,149,149,173,163,163,184,168,168,211,188,188,215,188,188,100,87,87,
      31,29,29,32,32,32,26,24,24,70,62,62,192,165,165,220,186,186,228,190,190,217,182,182,92,
      78,78,40,33,33,42,36,36,46,37,37,50,42,42,50,42,42,45,37,37,49,41,41,166,153,
      153,192,180,180,174,161,161,184,168,168,112,102,102,28,25,25,45,38,38,43,37,37,34,31,30,
      54,49,46,152,138,138,193,178,179,189,168,168,197,175,175,95,87,87,35,32,32,39,36,36,40,
      38,38,39,36,36,40,36,36,37,35,35,39,37,37,40,35,35,40,36,36,42,38,38,42,39,
      39,42,40,40,43,39,39,45,38,38,47,42,42,44,40,40,34,31,31,40,35,35,42,34,34,
      39,34,30,63,59,71,93,89,124,91,86,123,108,101,110,80,79,136,104,97,107,90,86,121,86,
      81,122,87,82,117,86,82,112,101,93,95,85,79,102,80,76,105,73,70,107,56,57,119,53,53,
      110,51,51,105,51,50,102,48,48,97,48,48,97,46,46,96,43,44,92,51,51,93,63,58,78,
      50,48,91,59,56,69,43,42,69,43,42,66,36,35,66,42,40,57,41,41,48,33,31,37,17,
      15,20,6,6,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,6,6,118,
      105,105,191,167,167,189,167,167,113,99,99,65,57,57,67,59,59,72,63,63,62,55,55,42,36,
      36,18,15,15,75,66,66,191,168,168,213,185,185,198,170,170,105,90,90,45,39,39,53,46,46,
      55,49,49,155,134,134,200,172,172,192,164,164,186,160,160,188,161,161,192,161,161,195,163,163,206,
      173,173,220,185,185,221,185,185,216,181,181,193,166,166,182,160,160,139,125,125,59,54,54,41,34,
      34,40,33,33,79,69,69,168,159,159,159,153,153,168,155,155,180,162,162,190,166,166,202,173,173,
      214,180,180,227,190,190,236,200,200,231,194,194,214,178,178,200,166,166,136,115,115,45,39,39,40,
      33,33,40,35,35,37,33,33,71,60,60,179,153,153,235,200,200,235,199,199,237,200,200,228,191,
      191,183,154,154,149,129,129,139,124,124,144,130,130,156,146,146,162,154,154,166,162,162,171,161,161,
      183,169,169,195,178,178,203,181,181,92,82,82,30,28,28,39,36,36,31,27,27,57,51,51,172,
      150,150,229,198,198,225,193,193,215,187,187,140,124,124,42,36,36,38,32,32,41,35,35,42,35,
      35,46,39,39,52,44,44,109,98,98,185,167,167,195,177,177,195,175,175,206,184,184,120,106,106,
      29,26,26,41,34,34,38,33,32,33,30,32,57,54,76,159,148,148,190,176,176,185,170,170,198,
      179,179,87,78,78,31,28,28,35,33,33,38,37,37,38,35,35,36,35,35,38,36,36,40,39,
      39,40,38,38,39,36,36,35,34,34,34,33,33,36,36,36,36,36,36,36,35,35,38,36,36,
      41,37,37,36,32,32,43,36,36,44,36,32,44,40,60,89,84,99,86,83,130,84,81,133,110,
      102,107,107,101,116,108,101,112,87,84,128,104,97,109,104,97,106,85,82,122,95,89,103,72,70,
      120,83,76,91,60,59,107,65,61,97,63,60,99,61,58,95,47,48,101,45,45,95,48,48,93,
      54,53,79,50,49,90,62,57,78,67,61,66,53,50,78,56,53,68,53,50,60,37,35,72,47,
      44,54,50,48,44,26,26,25,9,7,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,109,95,95,183,163,163,167,152,152,112,103,103,60,55,
      55,46,41,41,28,23,23,13,11,11,3,3,3,0,0,0,24,20,20,170,147,147,206,177,177,
      207,177,177,138,118,118,47,38,38,50,44,44,54,49,49,133,113,113,201,172,172,200,168,168,191,
      162,162,190,162,162,193,164,164,200,168,168,210,175,175,222,187,187,221,186,186,197,164,164,181,153,
      153,176,153,153,137,121,121,58,53,53,39,33,33,41,34,34,55,48,48,130,121,121,170,159,159,
      168,151,151,180,159,159,185,161,161,189,160,160,190,159,159,207,173,173,227,191,191,234,198,198,224,
      187,187,217,180,180,100,82,82,40,32,32,43,38,38,37,33,33,21,18,18,13,10,10,143,121,
      121,242,204,204,237,201,201,234,198,198,229,193,193,224,191,191,204,176,176,178,161,161,166,157,157,
      165,161,161,165,162,162,165,158,158,168,159,159,171,160,160,176,163,163,154,141,141,64,58,58,35,
      31,31,38,34,34,26,24,24,30,25,25,130,115,115,208,185,185,193,171,171,187,166,166,174,156,
      156,118,107,107,77,68,68,80,74,74,90,81,81,100,89,89,121,110,110,185,167,167,202,181,181,
      200,177,177,209,181,181,209,182,182,114,99,99,29,24,24,38,31,30,39,32,31,39,37,72,70,
      68,119,170,159,159,194,180,178,189,178,178,199,186,186,82,75,75,30,27,27,35,32,32,35,34,
      34,35,32,32,37,34,34,38,34,34,41,37,37,42,39,39,42,39,39,41,38,38,39,36,36,
      37,36,36,35,32,32,34,31,31,30,29,29,33,30,30,37,32,32,40,35,35,54,47,42,72,
      67,100,88,84,121,83,80,121,96,90,110,114,105,105,95,90,126,107,100,109,100,94,109,111,102,
      92,92,87,115,85,82,116,93,86,92,73,71,117,82,76,95,69,66,104,75,70,91,69,65,93,
      65,60,85,45,46,101,64,59,81,68,62,77,59,57,89,75,68,76,82,72,68,72,66,64,60,
      57,64,49,48,74,53,49,55,46,45,50,35,34,33,10,11,10,3,3,3,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,25,21,
      21,44,38,38,72,65,65,55,52,52,11,10,10,4,4,4,0,0,0,0,0,0,0,0,0,
      0,0,0,3,1,1,114,98,98,175,151,151,187,161,161,151,130,130,58,50,50,36,33,33,36,
      31,31,126,108,108,209,177,177,212,182,182,203,174,174,198,170,170,200,175,175,206,179,179,216,183,
      183,226,191,191,221,185,185,196,165,165,176,149,149,174,150,150,138,120,120,59,50,50,40,33,33,
      42,36,36,42,35,35,82,74,74,151,138,138,165,146,146,165,143,143,174,152,152,178,154,154,174,
      148,148,196,164,164,220,183,183,234,198,198,236,198,198,186,159,159,64,54,54,36,29,29,26,23,
      23,5,5,5,1,1,1,0,0,0,34,28,28,189,159,159,244,206,206,234,198,198,231,196,196,
      216,189,189,203,180,180,191,173,173,183,168,168,180,168,168,173,165,165,166,158,158,166,157,157,166,
      158,158,179,168,168,109,102,102,35,31,31,34,30,30,21,18,18,3,3,3,2,1,1,79,69,
      69,182,162,162,170,151,151,166,148,148,168,153,153,175,161,161,160,149,149,150,137,137,157,142,142,
      164,148,148,187,167,167,212,188,188,213,189,189,213,188,188,219,190,190,187,161,161,81,70,70,32,
      27,26,37,32,32,40,39,67,43,44,98,63,61,107,156,144,144,193,179,179,185,173,173,188,177,
      177,104,96,96,32,31,31,32,29,29,37,35,35,37,34,34,39,36,36,42,40,40,48,44,44,
      64,57,57,89,78,78,96,83,83,86,73,73,82,71,71,84,71,71,65,55,55,33,30,30,32,
      30,30,35,31,31,39,35,30,59,56,73,82,79,118,84,80,111,87,82,112,84,80,119,102,95,
      106,101,94,106,93,87,106,87,82,114,87,81,106,82,78,113,102,95,92,96,88,85,94,88,101,
      94,86,87,82,76,98,73,68,101,71,66,92,70,66,91,78,70,75,63,60,91,78,70,76,77,
      70,77,75,68,66,69,64,61,56,53,71,51,48,60,45,44,47,33,32,35,17,17,22,3,3,
      3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,23,19,19,53,46,46,77,
      66,66,94,83,83,36,31,31,4,3,3,2,2,2,92,80,80,175,147,147,182,155,155,188,163,
      163,185,162,162,182,160,160,194,169,169,204,175,175,216,184,184,220,187,187,206,178,178,195,169,169,
      191,165,165,153,132,132,66,58,58,38,32,32,36,32,32,19,17,17,5,4,4,70,63,63,156,
      138,138,160,142,142,167,148,148,170,149,149,166,142,142,184,153,153,217,181,181,243,205,205,242,204,
      204,114,95,95,25,21,21,4,4,4,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,
      87,74,74,207,177,177,206,179,179,198,177,177,209,186,186,213,189,189,213,189,189,211,187,187,205,
      184,184,190,176,176,181,169,169,174,161,161,168,160,160,163,156,156,77,70,70,20,16,16,7,6,
      6,1,1,1,0,0,0,0,0,0,26,23,23,117,102,100,157,136,135,156,137,137,158,141,141,
      171,155,155,180,166,166,180,163,163,182,166,166,189,171,171,198,178,178,208,185,185,209,184,184,214,
      186,186,228,198,198,176,150,150,52,43,43,33,28,26,33,32,53,37,38,82,40,40,90,60,59,
      99,153,142,141,179,166,165,170,160,160,172,158,158,86,81,81,48,45,45,68,63,63,79,70,70,
      87,78,78,96,86,86,100,91,91,104,95,95,107,95,95,111,97,97,118,105,105,129,110,110,131,
      113,113,135,116,116,78,65,65,28,22,22,30,26,26,33,30,31,46,44,45,77,71,95,58,58,
      126,75,71,104,73,70,107,78,74,102,93,86,95,67,65,114,71,68,107,78,74,101,73,69,107,
      96,88,91,94,87,91,95,88,93,76,72,107,91,83,87,90,81,80,86,78,81,83,75,75,80,
      72,73,82,74,71,76,69,74,76,69,69,75,68,66,65,61,60,57,54,55,45,44,50,35,35,
      38,11,11,9,2,2,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,1,1,1,3,2,2,13,11,11,58,50,50,66,58,58,62,54,54,89,77,77,104,92,92,
      125,107,107,156,133,133,150,132,132,148,131,131,160,138,138,149,130,130,60,52,52,18,15,15,2,
      2,2,0,0,0,0,0,0,0,0,0,57,51,51,127,114,114,164,147,147,179,159,159,183,158,
      158,196,165,165,223,187,187,228,192,192,138,116,116,30,25,25,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,107,96,96,170,158,158,171,159,159,188,
      172,172,205,184,184,210,186,186,215,187,187,215,186,186,212,185,185,193,175,175,182,165,165,172,162,
      162,106,102,102,26,22,22,2,2,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      43,39,47,144,126,133,164,142,141,160,139,139,161,140,140,172,153,153,182,164,164,182,164,164,188,
      169,169,198,177,177,203,181,181,205,181,181,210,183,183,223,193,193,149,129,129,24,21,21,10,8,
      7,4,4,12,15,15,31,23,24,52,50,49,79,150,137,136,180,165,164,165,152,152,162,147,147,
      155,139,139,158,145,145,161,147,147,161,144,144,160,142,142,147,131,131,126,115,115,120,113,113,118,
      109,109,109,99,99,113,104,104,121,113,113,121,113,113,84,75,75,44,36,36,31,26,26,30,26,
      25,39,36,37,45,45,87,56,56,114,59,57,108,60,59,105,54,54,114,55,55,112,52,52,105,
      53,52,101,66,63,90,67,64,97,73,68,95,75,70,98,77,72,97,87,79,93,93,84,80,92,
      83,80,90,80,76,85,76,72,77,69,77,73,66,73,75,67,65,68,63,61,62,59,58,57,54,
      55,44,42,44,26,27,33,12,12,25,3,3,6,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,23,
      19,19,36,30,30,7,6,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,22,19,19,92,82,82,120,108,108,143,127,127,149,127,127,140,118,118,88,73,73,3,2,2,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,8,7,7,99,90,90,165,153,153,188,174,174,191,174,174,199,177,177,206,176,176,204,172,
      172,216,182,182,217,190,190,197,179,179,115,107,107,22,20,20,2,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,12,12,32,126,109,125,177,151,148,168,143,143,165,
      140,140,173,146,146,183,157,157,188,165,165,193,172,172,201,179,179,202,180,180,207,184,184,213,188,
      188,194,169,169,79,68,68,7,4,4,0,0,0,0,0,0,0,0,0,0,0,0,27,26,35,
      150,135,135,184,165,165,168,156,156,171,157,157,176,159,159,184,165,165,194,175,175,199,178,178,198,
      177,177,191,170,170,175,156,156,162,152,152,156,147,147,156,146,146,159,150,150,170,162,162,147,138,
      138,65,58,58,34,29,29,30,29,29,31,27,24,43,43,59,49,49,99,48,49,105,60,57,92,
      61,58,91,54,53,94,53,52,98,48,48,98,53,51,86,48,49,107,61,58,88,64,61,85,68,
      62,91,79,71,77,79,72,76,75,69,83,67,61,80,65,60,78,67,61,69,55,54,71,54,53,
      68,56,53,59,50,49,52,35,35,38,22,22,25,3,4,10,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,24,23,23,
      16,15,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,5,5,73,68,68,158,146,
      146,195,178,178,206,185,185,210,185,185,210,180,180,229,192,192,209,179,179,118,105,105,29,27,27,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,
      3,1,81,67,66,176,149,149,188,158,158,182,155,155,183,154,154,186,157,157,188,161,161,194,167,
      167,198,172,172,206,179,179,214,188,188,215,187,187,131,114,114,10,10,10,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,17,15,14,134,121,121,171,159,159,164,158,158,172,160,160,180,
      165,165,189,172,172,195,179,179,201,180,180,209,184,184,204,180,180,193,170,170,180,163,163,172,162,
      162,169,161,161,168,163,163,180,171,171,146,135,135,56,49,49,32,29,29,32,28,27,33,31,44,
      43,43,89,46,46,96,45,45,92,46,46,95,45,45,97,53,52,78,50,49,86,46,46,94,56,
      53,77,54,52,83,47,46,89,55,53,78,60,57,84,54,52,80,51,50,73,49,46,73,49,48,
      75,46,45,71,48,48,64,44,42,42,28,29,34,20,21,37,13,14,19,4,4,9,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,41,37,37,79,72,72,101,90,90,125,111,111,120,103,103,
      97,82,82,64,55,55,3,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,4,4,92,76,76,183,155,155,203,170,
      170,201,169,169,195,164,164,191,161,161,190,161,161,190,161,161,204,173,173,219,190,190,165,144,144,
      32,27,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,16,16,130,
      122,122,163,153,153,155,146,146,163,153,153,172,161,161,178,168,168,185,171,171,196,179,179,212,187,
      187,217,189,189,217,191,191,202,182,182,194,177,177,184,169,169,174,159,159,177,162,162,144,132,132,
      59,53,53,28,24,24,33,29,26,38,38,70,42,43,88,42,42,84,40,40,83,41,41,83,41,
      41,83,39,40,84,40,40,85,42,42,85,50,48,73,53,51,71,53,51,74,60,57,66,58,55,
      65,58,54,61,49,48,61,49,47,47,43,41,38,30,29,31,19,20,17,9,9,13,2,2,9,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,8,7,7,5,4,4,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,6,4,4,60,51,51,134,114,114,187,159,159,200,169,169,197,168,168,194,164,164,
      196,167,167,191,163,163,108,93,93,33,30,30,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,19,17,17,137,127,127,170,162,162,162,156,156,164,159,159,171,163,
      163,173,162,162,175,159,159,183,166,166,198,176,176,211,186,186,218,190,190,210,183,183,199,175,175,
      195,175,175,183,161,161,172,152,152,137,122,122,52,47,47,30,26,24,36,33,48,36,36,72,35,
      36,75,35,35,70,34,34,69,34,34,69,35,35,74,42,42,62,37,37,64,43,42,60,43,42,
      62,38,37,49,41,39,38,27,26,33,29,27,34,21,20,21,15,15,13,12,12,11,6,7,8,
      0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,13,13,
      56,48,48,91,77,77,91,78,78,89,78,78,81,70,70,45,39,39,2,2,2,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,18,18,146,133,
      133,184,173,173,173,167,167,166,162,162,167,161,161,172,160,160,174,158,158,175,158,158,181,162,162,
      193,170,170,201,177,177,202,174,174,193,167,167,187,166,166,178,158,158,170,147,147,141,121,121,59,
      52,52,25,22,20,11,9,16,12,13,28,9,9,19,15,15,30,16,16,32,17,17,33,12,12,
      24,10,10,11,15,15,25,11,11,14,7,7,14,7,7,5,4,4,5,0,0,3,0,0,4,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,2,2,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,17,16,16,135,121,121,167,154,154,152,145,145,147,138,138,141,134,134,
      148,137,137,151,137,137,160,147,147,171,157,157,183,163,163,195,171,171,197,171,171,191,167,167,189,
      165,165,186,164,164,187,162,162,152,132,132,54,49,49,8,7,7,0,0,0,0,0,0,0,0,
      0,0,0,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,8,7,7,
      15,13,13,30,28,28,33,31,31,39,35,35,51,48,48,58,53,53,89,79,79,113,102,102,129,
      115,115,147,130,130,158,137,137,154,134,134,157,141,141,163,145,145,180,158,158,160,139,139,44,39,
      39,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,6,6,15,12,12,36,31,
      31,43,39,39,70,63,63,74,65,65,13,10,10,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,
      204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,
      255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,
      204,255,255,204,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,
      204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,
      255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,
      204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,
      255,204,255,255,204,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,
      0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,
      255,204,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,
      204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,
      255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,
      255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,
      0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,255,255,204,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,
      204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,
      255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,
      204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,
      255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,
      0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,255,255,204,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,
      0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,
      0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,
      204,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,
      255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,
      255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,
      255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,
      0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,
      255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,255,255,204,255,255,204,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,
      204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,
      255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,
      204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,
      255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,
      0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,
      255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,
      255,255,204,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,0,
      0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,
      204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,
      255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,255,
      255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
      204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,
      0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,255,255,204,
      255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,204,255,255,204,
      255,255,204,255,255,204,255,255,204,255,255,204,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
      0
    };
  wxImage image( 279, 173, (unsigned char*)data, TRUE );
  wxBitmap bitmap( image );
  return bitmap;
}
